@@ -42,11 +42,10 @@ import org.apache.kafka.common.requests.PushTelemetryRequest;
 import org.apache.kafka.common.requests.PushTelemetryResponse;
 import org.apache.kafka.common.telemetry.ClientTelemetryState;
 import org.apache.kafka.common.utils.Time;
-import org.apache.kafka.common.utils.Utils;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
-import java.nio.ByteBuffer;
+import java.io.IOException;
 import java.time.Duration;
 import java.util.Collections;
 import java.util.List;
@@ -218,10 +217,10 @@ public class ClientTelemetryReporter implements MetricsReporter {
         telemetryProvider.updateLabels(labels);
     }
 
-    public void initiateClose() {
+    public void initiateClose(long timeoutMs) {
         log.debug("Initiate close of ClientTelemetryReporter");
         try {
-            clientTelemetrySender.initiateClose();
+            clientTelemetrySender.initiateClose(timeoutMs);
         } catch (Exception exception) {
             log.error("Failed to initiate close of client telemetry reporter", exception);
         }
@@ -266,11 +265,12 @@ public class ClientTelemetryReporter implements MetricsReporter {
          These are the lower and upper bounds of the jitter that we apply to the initial push
          telemetry API call. This helps to avoid a flood of requests all coming at the same time.
         */
-        private final static double INITIAL_PUSH_JITTER_LOWER = 0.5;
-        private final static double INITIAL_PUSH_JITTER_UPPER = 1.5;
+        private static final double INITIAL_PUSH_JITTER_LOWER = 0.5;
+        private static final double INITIAL_PUSH_JITTER_UPPER = 1.5;
 
         private final ReadWriteLock lock = new ReentrantReadWriteLock();
         private final Condition subscriptionLoaded = lock.writeLock().newCondition();
+        private final Condition terminalPushInProgress = lock.writeLock().newCondition();
         /*
          Initial state should be subscription needed which should allow issuing first telemetry
          request of get telemetry subscription.
@@ -325,6 +325,7 @@ public class ClientTelemetryReporter implements MetricsReporter {
             final long timeMs;
             final String apiName;
             final String msg;
+            final boolean isTraceEnabled = log.isTraceEnabled();
 
             switch (localState) {
                 case SUBSCRIPTION_IN_PROGRESS:
@@ -336,15 +337,15 @@ public class ClientTelemetryReporter implements MetricsReporter {
                     */
                     apiName = (localState == ClientTelemetryState.SUBSCRIPTION_IN_PROGRESS) ? ApiKeys.GET_TELEMETRY_SUBSCRIPTIONS.name : ApiKeys.PUSH_TELEMETRY.name;
                     timeMs = requestTimeoutMs;
-                    msg = String.format("the remaining wait time for the %s network API request, as specified by %s", apiName, CommonClientConfigs.REQUEST_TIMEOUT_MS_CONFIG);
+                    msg = isTraceEnabled ? "" : String.format("the remaining wait time for the %s network API request, as specified by %s", apiName, CommonClientConfigs.REQUEST_TIMEOUT_MS_CONFIG);
                     break;
                 case TERMINATING_PUSH_IN_PROGRESS:
                     timeMs = Long.MAX_VALUE;
-                    msg = String.format("the terminating push is in progress, disabling telemetry for further requests");
+                    msg = isTraceEnabled ? "" : "the terminating push is in progress, disabling telemetry for further requests";
                     break;
                 case TERMINATING_PUSH_NEEDED:
                     timeMs = 0;
-                    msg = String.format("the client should try to submit the final %s network API request ASAP before closing", ApiKeys.PUSH_TELEMETRY.name);
+                    msg = isTraceEnabled ? "" : String.format("the client should try to submit the final %s network API request ASAP before closing", ApiKeys.PUSH_TELEMETRY.name);
                     break;
                 case SUBSCRIPTION_NEEDED:
                 case PUSH_NEEDED:
@@ -352,17 +353,19 @@ public class ClientTelemetryReporter implements MetricsReporter {
                     long timeRemainingBeforeRequest = localLastRequestMs + localIntervalMs - nowMs;
                     if (timeRemainingBeforeRequest <= 0) {
                         timeMs = 0;
-                        msg = String.format("the wait time before submitting the next %s network API request has elapsed", apiName);
+                        msg = isTraceEnabled ? "" : String.format("the wait time before submitting the next %s network API request has elapsed", apiName);
                     } else {
                         timeMs = timeRemainingBeforeRequest;
-                        msg = String.format("the client will wait before submitting the next %s network API request", apiName);
+                        msg = isTraceEnabled ? "" : String.format("the client will wait before submitting the next %s network API request", apiName);
                     }
                     break;
                 default:
                     throw new IllegalStateException("Unknown telemetry state: " + localState);
             }
 
-            log.trace("For telemetry state {}, returning the value {} ms; {}", localState, timeMs, msg);
+            if (isTraceEnabled) {
+                log.trace("For telemetry state {}, returning the value {} ms; {}", localState, timeMs, msg);
+            }
             return timeMs;
         }
 
@@ -598,8 +601,8 @@ public class ClientTelemetryReporter implements MetricsReporter {
         }
 
         @Override
-        public void initiateClose() {
-            log.debug("initiate close for client telemetry, check if terminal push required.");
+        public void initiateClose(long timeoutMs) {
+            log.debug("initiate close for client telemetry, check if terminal push required. Timeout {} ms.", timeoutMs);
 
             lock.writeLock().lock();
             try {
@@ -619,7 +622,14 @@ public class ClientTelemetryReporter implements MetricsReporter {
                     return;
                 }
 
-                log.debug("Updated state to send terminal telemetry push request");
+                try {
+                    log.info("About to wait {} ms. for terminal telemetry push to be submitted", timeoutMs);
+                    if (!terminalPushInProgress.await(timeoutMs, TimeUnit.MILLISECONDS)) {
+                        log.info("Wait for terminal telemetry push to be submitted has elapsed; may not have actually sent request");
+                    }
+                } catch (InterruptedException e) {
+                    log.warn("Error during client telemetry close", e);
+                }
             } finally {
                 lock.writeLock().unlock();
             }
@@ -696,6 +706,10 @@ public class ClientTelemetryReporter implements MetricsReporter {
                 lock.writeLock().unlock();
             }
 
+            return createPushRequest(localSubscription, terminating);
+        }
+
+        private Optional<Builder<?>> createPushRequest(ClientTelemetrySubscription localSubscription, boolean terminating) {
             byte[] payload;
             try (MetricsEmitter emitter = new ClientTelemetryEmitter(localSubscription.selector(), localSubscription.deltaTemporality())) {
                 emitter.init();
@@ -709,7 +723,14 @@ public class ClientTelemetryReporter implements MetricsReporter {
             }
 
             CompressionType compressionType = ClientTelemetryUtils.preferredCompressionType(localSubscription.acceptedCompressionTypes());
-            ByteBuffer buffer = ClientTelemetryUtils.compress(payload, compressionType);
+            byte[] compressedPayload;
+            try {
+                compressedPayload = ClientTelemetryUtils.compress(payload, compressionType);
+            } catch (IOException e) {
+                log.info("Failed to compress telemetry payload for compression: {}, sending uncompressed data", compressionType);
+                compressedPayload = payload;
+                compressionType = CompressionType.NONE;
+            }
 
             AbstractRequest.Builder<?> requestBuilder = new PushTelemetryRequest.Builder(
                 new PushTelemetryRequestData()
@@ -717,7 +738,7 @@ public class ClientTelemetryReporter implements MetricsReporter {
                     .setSubscriptionId(localSubscription.subscriptionId())
                     .setTerminating(terminating)
                     .setCompressionType(compressionType.id)
-                    .setMetrics(Utils.readBytes(buffer)), true);
+                    .setMetrics(compressedPayload), true);
 
             return Optional.of(requestBuilder);
         }
@@ -805,9 +826,13 @@ public class ClientTelemetryReporter implements MetricsReporter {
                 ClientTelemetryState oldState = state;
                 state = oldState.validateTransition(newState);
                 log.debug("Setting telemetry state from {} to {}", oldState, newState);
+
+                if (newState == ClientTelemetryState.TERMINATING_PUSH_IN_PROGRESS) {
+                    terminalPushInProgress.signalAll();
+                }
                 return true;
             } catch (IllegalStateException e) {
-                log.warn("Error updating client telemetry state, disabled telemetry");
+                log.warn("Error updating client telemetry state, disabled telemetry", e);
                 enabled = false;
                 return false;
             } finally {
