@@ -40,8 +40,13 @@ import org.apache.kafka.common.utils.MockTime;
 import org.junit.jupiter.api.AfterEach;
 import org.junit.jupiter.api.BeforeEach;
 import org.junit.jupiter.api.Test;
+import org.junit.jupiter.params.ParameterizedTest;
+import org.junit.jupiter.params.provider.EnumSource;
+import org.mockito.MockedStatic;
 import org.mockito.Mockito;
+import org.mockito.internal.stubbing.answers.CallsRealMethods;
 
+import java.io.IOException;
 import java.time.Duration;
 import java.util.Collections;
 import java.util.HashMap;
@@ -54,10 +59,12 @@ import java.util.concurrent.atomic.AtomicReference;
 
 import static org.junit.jupiter.api.Assertions.assertEquals;
 import static org.junit.jupiter.api.Assertions.assertFalse;
+import static org.junit.jupiter.api.Assertions.assertInstanceOf;
 import static org.junit.jupiter.api.Assertions.assertNotNull;
 import static org.junit.jupiter.api.Assertions.assertNull;
 import static org.junit.jupiter.api.Assertions.assertThrows;
 import static org.junit.jupiter.api.Assertions.assertTrue;
+import static org.mockito.ArgumentMatchers.any;
 
 public class ClientTelemetryReporterTest {
 
@@ -241,7 +248,7 @@ public class ClientTelemetryReporterTest {
         Optional<AbstractRequest.Builder<?>> requestOptional = telemetrySender.createRequest();
         assertNotNull(requestOptional);
         assertTrue(requestOptional.isPresent());
-        assertTrue(requestOptional.get().build() instanceof GetTelemetrySubscriptionsRequest);
+        assertInstanceOf(GetTelemetrySubscriptionsRequest.class, requestOptional.get().build());
         GetTelemetrySubscriptionsRequest request = (GetTelemetrySubscriptionsRequest) requestOptional.get().build();
 
         GetTelemetrySubscriptionsRequest expectedResult = new GetTelemetrySubscriptionsRequest.Builder(
@@ -260,7 +267,7 @@ public class ClientTelemetryReporterTest {
         Optional<AbstractRequest.Builder<?>> requestOptional = telemetrySender.createRequest();
         assertNotNull(requestOptional);
         assertTrue(requestOptional.isPresent());
-        assertTrue(requestOptional.get().build() instanceof GetTelemetrySubscriptionsRequest);
+        assertInstanceOf(GetTelemetrySubscriptionsRequest.class, requestOptional.get().build());
         GetTelemetrySubscriptionsRequest request = (GetTelemetrySubscriptionsRequest) requestOptional.get().build();
 
         GetTelemetrySubscriptionsRequest expectedResult = new GetTelemetrySubscriptionsRequest.Builder(
@@ -284,7 +291,7 @@ public class ClientTelemetryReporterTest {
         Optional<AbstractRequest.Builder<?>> requestOptional = telemetrySender.createRequest();
         assertNotNull(requestOptional);
         assertTrue(requestOptional.isPresent());
-        assertTrue(requestOptional.get().build() instanceof PushTelemetryRequest);
+        assertInstanceOf(PushTelemetryRequest.class, requestOptional.get().build());
         PushTelemetryRequest request = (PushTelemetryRequest) requestOptional.get().build();
 
         PushTelemetryRequest expectedResult = new PushTelemetryRequest.Builder(
@@ -350,6 +357,62 @@ public class ClientTelemetryReporterTest {
         assertEquals(now + 1000, telemetrySender.lastRequestMs());
     }
 
+    @ParameterizedTest
+    @EnumSource(CompressionType.class)
+    public void testCreateRequestPushCompression(CompressionType compressionType) {
+        clientTelemetryReporter.configure(configs);
+        clientTelemetryReporter.contextChange(metricsContext);
+
+        ClientTelemetryReporter.DefaultClientTelemetrySender telemetrySender = (ClientTelemetryReporter.DefaultClientTelemetrySender) clientTelemetryReporter.telemetrySender();
+        assertTrue(telemetrySender.maybeSetState(ClientTelemetryState.SUBSCRIPTION_IN_PROGRESS));
+        assertTrue(telemetrySender.maybeSetState(ClientTelemetryState.PUSH_NEEDED));
+
+        ClientTelemetryReporter.ClientTelemetrySubscription subscription = new ClientTelemetryReporter.ClientTelemetrySubscription(
+            uuid, 1234, 20000, Collections.singletonList(compressionType), true, null);
+        telemetrySender.updateSubscriptionResult(subscription, time.milliseconds());
+
+        Optional<AbstractRequest.Builder<?>> requestOptional = telemetrySender.createRequest();
+        assertNotNull(requestOptional);
+        assertTrue(requestOptional.isPresent());
+        assertInstanceOf(PushTelemetryRequest.class, requestOptional.get().build());
+        PushTelemetryRequest request = (PushTelemetryRequest) requestOptional.get().build();
+
+        assertEquals(subscription.clientInstanceId(), request.data().clientInstanceId());
+        assertEquals(subscription.subscriptionId(), request.data().subscriptionId());
+        assertEquals(compressionType.id, request.data().compressionType());
+        assertEquals(ClientTelemetryState.PUSH_IN_PROGRESS, telemetrySender.state());
+    }
+
+    @Test
+    public void testCreateRequestPushCompressionException() {
+        clientTelemetryReporter.configure(configs);
+        clientTelemetryReporter.contextChange(metricsContext);
+
+        ClientTelemetryReporter.DefaultClientTelemetrySender telemetrySender = (ClientTelemetryReporter.DefaultClientTelemetrySender) clientTelemetryReporter.telemetrySender();
+        assertTrue(telemetrySender.maybeSetState(ClientTelemetryState.SUBSCRIPTION_IN_PROGRESS));
+        assertTrue(telemetrySender.maybeSetState(ClientTelemetryState.PUSH_NEEDED));
+
+        ClientTelemetryReporter.ClientTelemetrySubscription subscription = new ClientTelemetryReporter.ClientTelemetrySubscription(
+            uuid, 1234, 20000, Collections.singletonList(CompressionType.GZIP), true, null);
+        telemetrySender.updateSubscriptionResult(subscription, time.milliseconds());
+
+        try (MockedStatic<ClientTelemetryUtils> mockedCompress = Mockito.mockStatic(ClientTelemetryUtils.class, new CallsRealMethods())) {
+            mockedCompress.when(() -> ClientTelemetryUtils.compress(any(), any())).thenThrow(new IOException());
+
+            Optional<AbstractRequest.Builder<?>> requestOptional = telemetrySender.createRequest();
+            assertNotNull(requestOptional);
+            assertTrue(requestOptional.isPresent());
+            assertInstanceOf(PushTelemetryRequest.class, requestOptional.get().build());
+            PushTelemetryRequest request = (PushTelemetryRequest) requestOptional.get().build();
+
+            assertEquals(subscription.clientInstanceId(), request.data().clientInstanceId());
+            assertEquals(subscription.subscriptionId(), request.data().subscriptionId());
+            // CompressionType.NONE is used when compression fails.
+            assertEquals(CompressionType.NONE.id, request.data().compressionType());
+            assertEquals(ClientTelemetryState.PUSH_IN_PROGRESS, telemetrySender.state());
+        }
+    }
+
     @Test
     public void testHandleResponseGetSubscriptions() {
         ClientTelemetryReporter.DefaultClientTelemetrySender telemetrySender = (ClientTelemetryReporter.DefaultClientTelemetrySender) clientTelemetryReporter.telemetrySender();
@@ -665,48 +728,6 @@ public class ClientTelemetryReporterTest {
         assertTrue(timeMs >= 500 && timeMs <= 1500);
     }
 
-    @Test
-    public void testTelemetryReporterInitiateClose() {
-        ClientTelemetryReporter.DefaultClientTelemetrySender telemetrySender = (ClientTelemetryReporter.DefaultClientTelemetrySender) clientTelemetryReporter.telemetrySender();
-        telemetrySender.updateSubscriptionResult(subscription, time.milliseconds());
-        assertTrue(telemetrySender.maybeSetState(ClientTelemetryState.SUBSCRIPTION_IN_PROGRESS));
-        assertTrue(telemetrySender.maybeSetState(ClientTelemetryState.PUSH_NEEDED));
-
-        clientTelemetryReporter.initiateClose();
-        assertEquals(ClientTelemetryState.TERMINATING_PUSH_NEEDED, ((ClientTelemetryReporter.DefaultClientTelemetrySender) clientTelemetryReporter
-            .telemetrySender()).state());
-    }
-
-    @Test
-    public void testTelemetryReporterInitiateCloseNoSubscription() {
-        clientTelemetryReporter.initiateClose();
-        assertEquals(ClientTelemetryState.SUBSCRIPTION_NEEDED, ((ClientTelemetryReporter.DefaultClientTelemetrySender) clientTelemetryReporter
-            .telemetrySender()).state());
-    }
-
-    @Test
-    public void testTelemetryReporterInitiateCloseAlreadyInTerminatedStates() {
-        ClientTelemetryReporter.DefaultClientTelemetrySender telemetrySender = (ClientTelemetryReporter.DefaultClientTelemetrySender) clientTelemetryReporter.telemetrySender();
-        telemetrySender.updateSubscriptionResult(subscription, time.milliseconds());
-        assertTrue(telemetrySender.maybeSetState(ClientTelemetryState.SUBSCRIPTION_IN_PROGRESS));
-        assertTrue(telemetrySender.maybeSetState(ClientTelemetryState.PUSH_NEEDED));
-        assertTrue(telemetrySender.maybeSetState(ClientTelemetryState.TERMINATING_PUSH_NEEDED));
-
-        clientTelemetryReporter.initiateClose();
-        assertEquals(ClientTelemetryState.TERMINATING_PUSH_NEEDED, ((ClientTelemetryReporter.DefaultClientTelemetrySender) clientTelemetryReporter
-            .telemetrySender()).state());
-
-        assertTrue(telemetrySender.maybeSetState(ClientTelemetryState.TERMINATING_PUSH_IN_PROGRESS));
-        clientTelemetryReporter.initiateClose();
-        assertEquals(ClientTelemetryState.TERMINATING_PUSH_IN_PROGRESS, ((ClientTelemetryReporter.DefaultClientTelemetrySender) clientTelemetryReporter
-            .telemetrySender()).state());
-
-        assertTrue(telemetrySender.maybeSetState(ClientTelemetryState.TERMINATED));
-        clientTelemetryReporter.initiateClose();
-        assertEquals(ClientTelemetryState.TERMINATED, ((ClientTelemetryReporter.DefaultClientTelemetrySender) clientTelemetryReporter
-            .telemetrySender()).state());
-    }
-
     @AfterEach
     public void tearDown() {
         clientTelemetryReporter.close();
