@@ -26,12 +26,14 @@ import org.apache.kafka.common.ClusterResourceListener;
 import org.apache.kafka.common.ClusterResource;
 import org.apache.kafka.common.TopicPartition;
 import org.apache.kafka.common.config.ConfigException;
+import org.apache.kafka.common.header.internals.RecordHeaders;
 
 import java.util.ArrayList;
 import java.util.HashMap;
 import java.util.List;
 import java.util.Locale;
 import java.util.Map;
+import java.util.Optional;
 import java.util.concurrent.atomic.AtomicInteger;
 import java.util.concurrent.atomic.AtomicReference;
 
@@ -39,6 +41,9 @@ public class MockConsumerInterceptor implements ClusterResourceListener, Consume
     public static final AtomicInteger INIT_COUNT = new AtomicInteger(0);
     public static final AtomicInteger CLOSE_COUNT = new AtomicInteger(0);
     public static final AtomicInteger ON_COMMIT_COUNT = new AtomicInteger(0);
+    public static final AtomicInteger CONFIG_COUNT = new AtomicInteger(0);
+    public static final AtomicInteger THROW_CONFIG_EXCEPTION = new AtomicInteger(0);
+    public static final AtomicInteger THROW_ON_CONFIG_EXCEPTION_THRESHOLD = new AtomicInteger(0);
     public static final AtomicReference<ClusterResource> CLUSTER_META = new AtomicReference<>();
     public static final ClusterResource NO_CLUSTER_ID = new ClusterResource("no_cluster_id");
     public static final AtomicReference<ClusterResource> CLUSTER_ID_BEFORE_ON_CONSUME = new AtomicReference<>(NO_CLUSTER_ID);
@@ -53,10 +58,14 @@ public class MockConsumerInterceptor implements ClusterResourceListener, Consume
         Object clientIdValue = configs.get(ConsumerConfig.CLIENT_ID_CONFIG);
         if (clientIdValue == null)
             throw new ConfigException("Mock consumer interceptor expects configuration " + ProducerConfig.CLIENT_ID_CONFIG);
+
+        CONFIG_COUNT.incrementAndGet();
+        if (CONFIG_COUNT.get() == THROW_ON_CONFIG_EXCEPTION_THRESHOLD.get()) {
+            throw new ConfigException("Failed to instantiate interceptor. Reached configuration exception threshold.");
+        }
     }
 
     @Override
-    @SuppressWarnings("deprecation")
     public ConsumerRecords<String, String> onConsume(ConsumerRecords<String, String> records) {
 
         // This will ensure that we get the cluster metadata when onConsume is called for the first time
@@ -69,13 +78,14 @@ public class MockConsumerInterceptor implements ClusterResourceListener, Consume
             for (ConsumerRecord<String, String> record: records.records(tp)) {
                 lst.add(new ConsumerRecord<>(record.topic(), record.partition(), record.offset(),
                                              record.timestamp(), record.timestampType(),
-                                             record.checksum(), record.serializedKeySize(),
+                                             record.serializedKeySize(),
                                              record.serializedValueSize(),
-                                             record.key(), record.value().toUpperCase(Locale.ROOT)));
+                                             record.key(), record.value().toUpperCase(Locale.ROOT),
+                                             new RecordHeaders(), Optional.empty()));
             }
             recordMap.put(tp, lst);
         }
-        return new ConsumerRecords<String, String>(recordMap);
+        return new ConsumerRecords<>(recordMap);
     }
 
     @Override
@@ -88,10 +98,16 @@ public class MockConsumerInterceptor implements ClusterResourceListener, Consume
         CLOSE_COUNT.incrementAndGet();
     }
 
+    public static void setThrowOnConfigExceptionThreshold(int value) {
+        THROW_ON_CONFIG_EXCEPTION_THRESHOLD.set(value);
+    }
+
     public static void resetCounters() {
         INIT_COUNT.set(0);
         CLOSE_COUNT.set(0);
         ON_COMMIT_COUNT.set(0);
+        CONFIG_COUNT.set(0);
+        THROW_CONFIG_EXCEPTION.set(0);
         CLUSTER_META.set(null);
         CLUSTER_ID_BEFORE_ON_CONSUME.set(NO_CLUSTER_ID);
     }
