@@ -25,6 +25,7 @@ import org.junit.jupiter.api.Timeout;
 
 import java.math.BigDecimal;
 import java.math.BigInteger;
+import java.nio.ByteBuffer;
 import java.nio.charset.StandardCharsets;
 import java.text.SimpleDateFormat;
 import java.util.ArrayList;
@@ -37,6 +38,7 @@ import java.util.Map;
 
 import static org.junit.jupiter.api.Assertions.assertEquals;
 import static org.junit.jupiter.api.Assertions.assertFalse;
+import static org.junit.jupiter.api.Assertions.assertInstanceOf;
 import static org.junit.jupiter.api.Assertions.assertNotNull;
 import static org.junit.jupiter.api.Assertions.assertNull;
 import static org.junit.jupiter.api.Assertions.assertThrows;
@@ -79,6 +81,20 @@ public class ValuesTest {
         INT_LIST.add(-987654321);
     }
 
+    @Test
+    public void shouldParseNullString() {
+        SchemaAndValue schemaAndValue = Values.parseString(null);
+        assertNull(schemaAndValue.schema());
+        assertNull(schemaAndValue.value());
+    }
+
+    @Test
+    public void shouldParseEmptyString() {
+        SchemaAndValue schemaAndValue = Values.parseString("");
+        assertEquals(Schema.STRING_SCHEMA, schemaAndValue.schema());
+        assertEquals("", schemaAndValue.value());
+    }
+
     @Test
     @Timeout(5)
     public void shouldNotEncounterInfiniteLoop() {
@@ -244,6 +260,20 @@ public class ValuesTest {
 
     @Test
     public void shouldConvertNullValue() {
+        assertRoundTrip(Schema.INT8_SCHEMA, Schema.STRING_SCHEMA, null);
+        assertRoundTrip(Schema.OPTIONAL_INT8_SCHEMA, Schema.STRING_SCHEMA, null);
+        assertRoundTrip(Schema.INT16_SCHEMA, Schema.STRING_SCHEMA, null);
+        assertRoundTrip(Schema.OPTIONAL_INT16_SCHEMA, Schema.STRING_SCHEMA, null);
+        assertRoundTrip(Schema.INT32_SCHEMA, Schema.STRING_SCHEMA, null);
+        assertRoundTrip(Schema.OPTIONAL_INT32_SCHEMA, Schema.STRING_SCHEMA, null);
+        assertRoundTrip(Schema.INT64_SCHEMA, Schema.STRING_SCHEMA, null);
+        assertRoundTrip(Schema.OPTIONAL_INT64_SCHEMA, Schema.STRING_SCHEMA, null);
+        assertRoundTrip(Schema.FLOAT32_SCHEMA, Schema.STRING_SCHEMA, null);
+        assertRoundTrip(Schema.OPTIONAL_FLOAT32_SCHEMA, Schema.STRING_SCHEMA, null);
+        assertRoundTrip(Schema.FLOAT64_SCHEMA, Schema.STRING_SCHEMA, null);
+        assertRoundTrip(Schema.OPTIONAL_FLOAT64_SCHEMA, Schema.STRING_SCHEMA, null);
+        assertRoundTrip(Schema.BOOLEAN_SCHEMA, Schema.STRING_SCHEMA, null);
+        assertRoundTrip(Schema.OPTIONAL_BOOLEAN_SCHEMA, Schema.STRING_SCHEMA, null);
         assertRoundTrip(Schema.STRING_SCHEMA, Schema.STRING_SCHEMA, null);
         assertRoundTrip(Schema.OPTIONAL_STRING_SCHEMA, Schema.STRING_SCHEMA, null);
     }
@@ -251,14 +281,22 @@ public class ValuesTest {
     @Test
     public void shouldConvertBooleanValues() {
         assertRoundTrip(Schema.BOOLEAN_SCHEMA, Schema.BOOLEAN_SCHEMA, Boolean.FALSE);
+        assertShortCircuit(Schema.BOOLEAN_SCHEMA, Boolean.FALSE);
         SchemaAndValue resultFalse = roundTrip(Schema.BOOLEAN_SCHEMA, "false");
         assertEquals(Schema.BOOLEAN_SCHEMA, resultFalse.schema());
         assertEquals(Boolean.FALSE, resultFalse.value());
+        resultFalse = roundTrip(Schema.BOOLEAN_SCHEMA, "0");
+        assertEquals(Schema.BOOLEAN_SCHEMA, resultFalse.schema());
+        assertEquals(Boolean.FALSE, resultFalse.value());
 
         assertRoundTrip(Schema.BOOLEAN_SCHEMA, Schema.BOOLEAN_SCHEMA, Boolean.TRUE);
+        assertShortCircuit(Schema.BOOLEAN_SCHEMA, Boolean.TRUE);
         SchemaAndValue resultTrue = roundTrip(Schema.BOOLEAN_SCHEMA, "true");
         assertEquals(Schema.BOOLEAN_SCHEMA, resultTrue.schema());
         assertEquals(Boolean.TRUE, resultTrue.value());
+        resultTrue = roundTrip(Schema.BOOLEAN_SCHEMA, "1");
+        assertEquals(Schema.BOOLEAN_SCHEMA, resultTrue.schema());
+        assertEquals(Boolean.TRUE, resultTrue.value());
     }
 
     @Test
@@ -266,6 +304,38 @@ public class ValuesTest {
         assertThrows(DataException.class, () -> Values.convertToBoolean(Schema.STRING_SCHEMA, "\"green\""));
     }
 
+    @Test
+    public void shouldConvertInt8() {
+        assertRoundTrip(Schema.INT8_SCHEMA, Schema.INT8_SCHEMA, (byte) 0);
+        assertRoundTrip(Schema.INT8_SCHEMA, Schema.INT8_SCHEMA, (byte) 1);
+    }
+
+    @Test
+    public void shouldConvertInt64() {
+        assertRoundTrip(Schema.INT64_SCHEMA, Schema.INT64_SCHEMA, (long) 1);
+        assertShortCircuit(Schema.INT64_SCHEMA, (long) 1);
+    }
+
+    @Test
+    public void shouldConvertFloat32() {
+        assertRoundTrip(Schema.FLOAT32_SCHEMA, Schema.FLOAT32_SCHEMA, (float) 1);
+        assertShortCircuit(Schema.FLOAT32_SCHEMA, (float) 1);
+    }
+
+    @Test
+    public void shouldConvertFloat64() {
+        assertRoundTrip(Schema.FLOAT64_SCHEMA, Schema.FLOAT64_SCHEMA, (double) 1);
+        assertShortCircuit(Schema.FLOAT64_SCHEMA, (double) 1);
+    }
+
+    @Test
+    public void shouldConvertEmptyStruct() {
+        Struct struct = new Struct(SchemaBuilder.struct().build());
+        assertThrows(DataException.class, () -> Values.convertToStruct(struct.schema(), null));
+        assertThrows(DataException.class, () -> Values.convertToStruct(struct.schema(), ""));
+        Values.convertToStruct(struct.schema(), struct);
+    }
+
     @Test
     public void shouldConvertSimpleString() {
         assertRoundTrip(Schema.STRING_SCHEMA,  "simple");
@@ -359,37 +429,125 @@ public class ValuesTest {
         assertEquals(3, list.size());
         assertEquals(1, ((Number) list.get(0)).intValue());
         assertEquals(2, ((Number) list.get(1)).intValue());
-        assertEquals(thirdValue, ((Number) list.get(2)).intValue());
+        assertEquals(thirdValue, list.get(2));
     }
 
-    /**
-     * The parsed array has byte values and one int value, so we should return list with single unified type of integers.
-     */
     @Test
-    public void shouldConvertStringOfListWithMixedElementTypesIntoListWithDifferentElementTypes() {
-        String str = "[1, 2, \"three\"]";
-        List<?> list = Values.convertToList(Schema.STRING_SCHEMA, str);
+    public void shouldConvertIntegralTypesToFloat() {
+        float thirdValue = Float.MAX_VALUE;
+        List<?> list = Values.convertToList(Schema.STRING_SCHEMA, "[1, 2, " + thirdValue + "]");
         assertEquals(3, list.size());
         assertEquals(1, ((Number) list.get(0)).intValue());
         assertEquals(2, ((Number) list.get(1)).intValue());
-        assertEquals("three", list.get(2));
+        assertEquals(thirdValue, list.get(2));
+    }
+
+    @Test
+    public void shouldConvertIntegralTypesToDouble() {
+        double thirdValue = Double.MAX_VALUE;
+        List<?> list = Values.convertToList(Schema.STRING_SCHEMA, "[1, 2, " + thirdValue + "]");
+        assertEquals(3, list.size());
+        assertEquals(1, ((Number) list.get(0)).intValue());
+        assertEquals(2, ((Number) list.get(1)).intValue());
+        assertEquals(thirdValue, list.get(2));
     }
 
     /**
      * We parse into different element types, but cannot infer a common element schema.
+     * This behavior should be independent of the order that the elements appear in the string
      */
     @Test
-    public void shouldParseStringListWithMultipleElementTypesAndReturnListWithNoSchema() {
-        String str = "[1, 2, 3, \"four\"]";
+    public void shouldParseStringListWithMultipleElementTypes() {
+        assertParseStringArrayWithNoSchema(
+                Arrays.asList((byte) 1, (byte) 2, (short) 300, "four"),
+                "[1, 2, 300, \"four\"]");
+        assertParseStringArrayWithNoSchema(
+                Arrays.asList((byte) 2, (short) 300, "four", (byte) 1),
+                "[2, 300, \"four\", 1]");
+        assertParseStringArrayWithNoSchema(
+                Arrays.asList((short) 300, "four", (byte) 1, (byte) 2),
+                "[300, \"four\", 1, 2]");
+        assertParseStringArrayWithNoSchema(
+                Arrays.asList("four", (byte) 1, (byte) 2, (short) 300),
+                "[\"four\", 1, 2, 300]");
+    }
+
+    private void assertParseStringArrayWithNoSchema(List<Object> expected, String str) {
         SchemaAndValue result = Values.parseString(str);
         assertEquals(Type.ARRAY, result.schema().type());
         assertNull(result.schema().valueSchema());
         List<?> list = (List<?>) result.value();
-        assertEquals(4, list.size());
-        assertEquals(1, ((Number) list.get(0)).intValue());
-        assertEquals(2, ((Number) list.get(1)).intValue());
-        assertEquals(3, ((Number) list.get(2)).intValue());
-        assertEquals("four", list.get(3));
+        assertEquals(expected, list);
+    }
+
+    /**
+     * Maps with an inconsistent key type don't find a common type for the keys or the values
+     * This behavior should be independent of the order that the pairs appear in the string
+     */
+    @Test
+    public void shouldParseStringMapWithMultipleKeyTypes() {
+        Map<Object, Object> expected = new HashMap<>();
+        expected.put((byte) 1, (byte) 1);
+        expected.put((byte) 2, (byte) 1);
+        expected.put((short) 300, (short) 300);
+        expected.put("four", (byte) 1);
+        assertParseStringMapWithNoSchema(expected, "{1:1, 2:1, 300:300, \"four\":1}");
+        assertParseStringMapWithNoSchema(expected, "{2:1, 300:300, \"four\":1, 1:1}");
+        assertParseStringMapWithNoSchema(expected, "{300:300, \"four\":1, 1:1, 2:1}");
+        assertParseStringMapWithNoSchema(expected, "{\"four\":1, 1:1, 2:1, 300:300}");
+    }
+
+    /**
+     * Maps with a consistent key type may still not have a common type for the values
+     * This behavior should be independent of the order that the pairs appear in the string
+     */
+    @Test
+    public void shouldParseStringMapWithMultipleValueTypes() {
+        Map<Object, Object> expected = new HashMap<>();
+        expected.put((short) 1, (byte) 1);
+        expected.put((short) 2, (byte) 1);
+        expected.put((short) 300, (short) 300);
+        expected.put((short) 4, "four");
+        assertParseStringMapWithNoSchema(expected, "{1:1, 2:1, 300:300, 4:\"four\"}");
+        assertParseStringMapWithNoSchema(expected, "{2:1, 300:300, 4:\"four\", 1:1}");
+        assertParseStringMapWithNoSchema(expected, "{300:300, 4:\"four\", 1:1, 2:1}");
+        assertParseStringMapWithNoSchema(expected, "{4:\"four\", 1:1, 2:1, 300:300}");
+    }
+
+    private void assertParseStringMapWithNoSchema(Map<Object, Object> expected, String str) {
+        SchemaAndValue result = Values.parseString(str);
+        assertEquals(Type.MAP, result.schema().type());
+        assertNull(result.schema().valueSchema());
+        Map<?, ?> list = (Map<?, ?>) result.value();
+        assertEquals(expected, list);
+    }
+
+    @Test
+    public void shouldParseNestedArray() {
+        SchemaAndValue schemaAndValue = Values.parseString("[[]]");
+        assertEquals(Type.ARRAY, schemaAndValue.schema().type());
+        assertEquals(Type.ARRAY, schemaAndValue.schema().valueSchema().type());
+    }
+
+    @Test
+    public void shouldParseArrayContainingMap() {
+        SchemaAndValue schemaAndValue = Values.parseString("[{}]");
+        assertEquals(Type.ARRAY, schemaAndValue.schema().type());
+        assertEquals(Type.MAP, schemaAndValue.schema().valueSchema().type());
+    }
+
+    @Test
+    public void shouldParseNestedMap() {
+        SchemaAndValue schemaAndValue = Values.parseString("{\"a\":{}}");
+        assertEquals(Type.MAP, schemaAndValue.schema().type());
+        assertEquals(Type.MAP, schemaAndValue.schema().valueSchema().type());
+    }
+
+    @Test
+    public void shouldParseMapContainingArray() {
+        SchemaAndValue schemaAndValue = Values.parseString("{\"a\":[]}");
+        assertEquals(Type.MAP, schemaAndValue.schema().type());
+        assertEquals(Type.ARRAY, schemaAndValue.schema().valueSchema().type());
     }
 
     /**
@@ -403,6 +561,22 @@ public class ValuesTest {
         assertEquals(str, result.value());
     }
 
+    @Test
+    public void shouldParseStringListWithNullLastAsString() {
+        String str = "[1, null]";
+        SchemaAndValue result = Values.parseString(str);
+        assertEquals(Type.STRING, result.schema().type());
+        assertEquals(str, result.value());
+    }
+
+    @Test
+    public void shouldParseStringListWithNullFirstAsString() {
+        String str = "[null, 1]";
+        SchemaAndValue result = Values.parseString(str);
+        assertEquals(Type.STRING, result.schema().type());
+        assertEquals(str, result.value());
+    }
+
     @Test
     public void shouldParseTimestampStringAsTimestamp() throws Exception {
         String str = "2019-08-23T14:34:54.346Z";
@@ -543,6 +717,13 @@ public class ValuesTest {
         assertEquals(Collections.singletonMap(keyStr, expected), result.value());
     }
 
+    @Test
+    public void shouldFailToConvertNullTime() {
+        assertThrows(DataException.class, () -> Values.convertToTime(null, null));
+        assertThrows(DataException.class, () -> Values.convertToDate(null, null));
+        assertThrows(DataException.class, () -> Values.convertToTimestamp(null, null));
+    }
+
     /**
      * This is technically invalid JSON, and we don't want to simply ignore the blank elements.
      */
@@ -744,6 +925,95 @@ public class ValuesTest {
         assertEquals(current, ts4);
     }
 
+    @Test
+    public void shouldConvertDecimalValues() {
+        // Various forms of the same number should all be parsed to the same BigDecimal
+        Number number = 1.0f;
+        String string = number.toString();
+        BigDecimal value = new BigDecimal(string);
+        byte[] bytes = Decimal.fromLogical(Decimal.schema(1), value);
+        ByteBuffer buffer = ByteBuffer.wrap(bytes);
+
+        assertEquals(value, Values.convertToDecimal(null, number, 1));
+        assertEquals(value, Values.convertToDecimal(null, string, 1));
+        assertEquals(value, Values.convertToDecimal(null, value, 1));
+        assertEquals(value, Values.convertToDecimal(null, bytes, 1));
+        assertEquals(value, Values.convertToDecimal(null, buffer, 1));
+    }
+
+    @Test
+    public void shouldFailToConvertNullToDecimal() {
+        assertThrows(DataException.class, () -> Values.convertToDecimal(null, null, 1));
+    }
+
+    @Test
+    public void shouldInferByteSchema() {
+        byte[] bytes = new byte[1];
+        Schema byteSchema = Values.inferSchema(bytes);
+        assertEquals(Schema.BYTES_SCHEMA, byteSchema);
+        Schema byteBufferSchema = Values.inferSchema(ByteBuffer.wrap(bytes));
+        assertEquals(Schema.BYTES_SCHEMA, byteBufferSchema);
+    }
+
+    @Test
+    public void shouldInferStructSchema() {
+        Struct struct = new Struct(SchemaBuilder.struct().build());
+        Schema structSchema = Values.inferSchema(struct);
+        assertEquals(struct.schema(), structSchema);
+    }
+
+    @Test
+    public void shouldInferNoSchemaForEmptyList() {
+        Schema listSchema = Values.inferSchema(Collections.emptyList());
+        assertNull(listSchema);
+    }
+
+    @Test
+    public void shouldInferNoSchemaForListContainingObject() {
+        Schema listSchema = Values.inferSchema(Collections.singletonList(new Object()));
+        assertNull(listSchema);
+    }
+
+    @Test
+    public void shouldInferNoSchemaForEmptyMap() {
+        Schema listSchema = Values.inferSchema(Collections.emptyMap());
+        assertNull(listSchema);
+    }
+
+    @Test
+    public void shouldInferNoSchemaForMapContainingObject() {
+        Schema listSchema = Values.inferSchema(Collections.singletonMap(new Object(), new Object()));
+        assertNull(listSchema);
+    }
+
+    /**
+     * Test parsing distinct number-like types (strings containing numbers, and logical Decimals) in the same list
+     * The parser does not convert Numbers to Decimals, or Strings containing numbers to Numbers automatically.
+     */
+    @Test
+    public void shouldNotConvertArrayValuesToDecimal() {
+        List<Object> decimals = Arrays.asList("\"1.0\"", BigDecimal.valueOf(Long.MAX_VALUE).add(BigDecimal.ONE),
+                BigDecimal.valueOf(Long.MIN_VALUE).subtract(BigDecimal.ONE), (byte) 1, (byte) 1);
+        List<Object> expected = new ArrayList<>(decimals); // most values are directly reproduced with the same type
+        expected.set(0, "1.0"); // The quotes are parsed away, but the value remains a string
+        SchemaAndValue schemaAndValue = Values.parseString(decimals.toString());
+        Schema schema = schemaAndValue.schema();
+        assertEquals(Type.ARRAY, schema.type());
+        assertNull(schema.valueSchema());
+        assertEquals(expected, schemaAndValue.value());
+    }
+
+    @Test
+    public void shouldParseArrayOfOnlyDecimals() {
+        List<Object> decimals = Arrays.asList(BigDecimal.valueOf(Long.MAX_VALUE).add(BigDecimal.ONE),
+                BigDecimal.valueOf(Long.MIN_VALUE).subtract(BigDecimal.ONE));
+        SchemaAndValue schemaAndValue = Values.parseString(decimals.toString());
+        Schema schema = schemaAndValue.schema();
+        assertEquals(Type.ARRAY, schema.type());
+        assertEquals(Decimal.schema(0), schema.valueSchema());
+        assertEquals(decimals, schemaAndValue.value());
+    }
+
     @Test
     public void canConsume() {
     }
@@ -755,14 +1025,14 @@ public class ValuesTest {
             String.valueOf(value)
         );
         assertEquals(Decimal.schema(0), schemaAndValue.schema());
-        assertTrue(schemaAndValue.value() instanceof BigDecimal);
+        assertInstanceOf(BigDecimal.class, schemaAndValue.value());
         assertEquals(value, ((BigDecimal) schemaAndValue.value()).unscaledValue());
         value = BigInteger.valueOf(Long.MIN_VALUE).subtract(new BigInteger("1"));
         schemaAndValue = Values.parseString(
             String.valueOf(value)
         );
         assertEquals(Decimal.schema(0), schemaAndValue.schema());
-        assertTrue(schemaAndValue.value() instanceof BigDecimal);
+        assertInstanceOf(BigDecimal.class, schemaAndValue.value());
         assertEquals(value, ((BigDecimal) schemaAndValue.value()).unscaledValue());
     }
 
@@ -773,14 +1043,14 @@ public class ValuesTest {
             String.valueOf(value)
         );
         assertEquals(Schema.INT8_SCHEMA, schemaAndValue.schema());
-        assertTrue(schemaAndValue.value() instanceof Byte);
+        assertInstanceOf(Byte.class, schemaAndValue.value());
         assertEquals(value.byteValue(), ((Byte) schemaAndValue.value()).byteValue());
         value = Byte.MIN_VALUE;
         schemaAndValue = Values.parseString(
             String.valueOf(value)
         );
         assertEquals(Schema.INT8_SCHEMA, schemaAndValue.schema());
-        assertTrue(schemaAndValue.value() instanceof Byte);
+        assertInstanceOf(Byte.class, schemaAndValue.value());
         assertEquals(value.byteValue(), ((Byte) schemaAndValue.value()).byteValue());
     }
 
@@ -791,14 +1061,14 @@ public class ValuesTest {
             String.valueOf(value)
         );
         assertEquals(Schema.INT16_SCHEMA, schemaAndValue.schema());
-        assertTrue(schemaAndValue.value() instanceof Short);
+        assertInstanceOf(Short.class, schemaAndValue.value());
         assertEquals(value.shortValue(), ((Short) schemaAndValue.value()).shortValue());
         value = Short.MIN_VALUE;
         schemaAndValue = Values.parseString(
             String.valueOf(value)
         );
         assertEquals(Schema.INT16_SCHEMA, schemaAndValue.schema());
-        assertTrue(schemaAndValue.value() instanceof Short);
+        assertInstanceOf(Short.class, schemaAndValue.value());
         assertEquals(value.shortValue(), ((Short) schemaAndValue.value()).shortValue());
     }
 
@@ -809,14 +1079,14 @@ public class ValuesTest {
             String.valueOf(value)
         );
         assertEquals(Schema.INT32_SCHEMA, schemaAndValue.schema());
-        assertTrue(schemaAndValue.value() instanceof Integer);
+        assertInstanceOf(Integer.class, schemaAndValue.value());
         assertEquals(value.intValue(), ((Integer) schemaAndValue.value()).intValue());
         value = Integer.MIN_VALUE;
         schemaAndValue = Values.parseString(
             String.valueOf(value)
         );
         assertEquals(Schema.INT32_SCHEMA, schemaAndValue.schema());
-        assertTrue(schemaAndValue.value() instanceof Integer);
+        assertInstanceOf(Integer.class, schemaAndValue.value());
         assertEquals(value.intValue(), ((Integer) schemaAndValue.value()).intValue());
     }
 
@@ -827,14 +1097,14 @@ public class ValuesTest {
             String.valueOf(value)
         );
         assertEquals(Schema.INT64_SCHEMA, schemaAndValue.schema());
-        assertTrue(schemaAndValue.value() instanceof Long);
+        assertInstanceOf(Long.class, schemaAndValue.value());
         assertEquals(value.longValue(), ((Long) schemaAndValue.value()).longValue());
         value = Long.MIN_VALUE;
         schemaAndValue = Values.parseString(
             String.valueOf(value)
         );
         assertEquals(Schema.INT64_SCHEMA, schemaAndValue.schema());
-        assertTrue(schemaAndValue.value() instanceof Long);
+        assertInstanceOf(Long.class, schemaAndValue.value());
         assertEquals(value.longValue(), ((Long) schemaAndValue.value()).longValue());
     }
 
@@ -845,14 +1115,14 @@ public class ValuesTest {
             String.valueOf(value)
         );
         assertEquals(Schema.FLOAT32_SCHEMA, schemaAndValue.schema());
-        assertTrue(schemaAndValue.value() instanceof Float);
+        assertInstanceOf(Float.class, schemaAndValue.value());
         assertEquals(value, (Float) schemaAndValue.value(), 0);
         value = -Float.MAX_VALUE;
         schemaAndValue = Values.parseString(
             String.valueOf(value)
         );
         assertEquals(Schema.FLOAT32_SCHEMA, schemaAndValue.schema());
-        assertTrue(schemaAndValue.value() instanceof Float);
+        assertInstanceOf(Float.class, schemaAndValue.value());
         assertEquals(value, (Float) schemaAndValue.value(), 0);
     }
 
@@ -863,17 +1133,27 @@ public class ValuesTest {
             String.valueOf(value)
         );
         assertEquals(Schema.FLOAT64_SCHEMA, schemaAndValue.schema());
-        assertTrue(schemaAndValue.value() instanceof Double);
+        assertInstanceOf(Double.class, schemaAndValue.value());
         assertEquals(value, (Double) schemaAndValue.value(), 0);
         value = -Double.MAX_VALUE;
         schemaAndValue = Values.parseString(
             String.valueOf(value)
         );
         assertEquals(Schema.FLOAT64_SCHEMA, schemaAndValue.schema());
-        assertTrue(schemaAndValue.value() instanceof Double);
+        assertInstanceOf(Double.class, schemaAndValue.value());
         assertEquals(value, (Double) schemaAndValue.value(), 0);
     }
 
+    @Test
+    public void shouldParseFractionalPartsAsIntegerWhenNoFractionalPart() {
+        assertEquals(new SchemaAndValue(Schema.INT8_SCHEMA, (byte) 1), Values.parseString("1.0"));
+        assertEquals(new SchemaAndValue(Schema.FLOAT32_SCHEMA, 1.1f), Values.parseString("1.1"));
+        assertEquals(new SchemaAndValue(Schema.INT16_SCHEMA, (short) 300), Values.parseString("300.0"));
+        assertEquals(new SchemaAndValue(Schema.FLOAT32_SCHEMA, 300.01f), Values.parseString("300.01"));
+        assertEquals(new SchemaAndValue(Schema.INT32_SCHEMA, 66000), Values.parseString("66000.0"));
+        assertEquals(new SchemaAndValue(Schema.FLOAT32_SCHEMA, 66000.0008f), Values.parseString("66000.0008"));
+    }
+
     protected void assertParsed(String input) {
         assertParsed(input, input);
     }
@@ -936,47 +1216,48 @@ public class ValuesTest {
             desiredSchema = Values.inferSchema(input);
             assertNotNull(desiredSchema);
         }
+        return convertTo(desiredSchema, serialized);
+    }
+
+    protected SchemaAndValue convertTo(Schema desiredSchema, Object value) {
         Object newValue = null;
-        Schema newSchema = null;
         switch (desiredSchema.type()) {
             case STRING:
-                newValue = Values.convertToString(Schema.STRING_SCHEMA, serialized);
+                newValue = Values.convertToString(Schema.STRING_SCHEMA, value);
                 break;
             case INT8:
-                newValue = Values.convertToByte(Schema.STRING_SCHEMA, serialized);
+                newValue = Values.convertToByte(Schema.STRING_SCHEMA, value);
                 break;
             case INT16:
-                newValue = Values.convertToShort(Schema.STRING_SCHEMA, serialized);
+                newValue = Values.convertToShort(Schema.STRING_SCHEMA, value);
                 break;
             case INT32:
-                newValue = Values.convertToInteger(Schema.STRING_SCHEMA, serialized);
+                newValue = Values.convertToInteger(Schema.STRING_SCHEMA, value);
                 break;
             case INT64:
-                newValue = Values.convertToLong(Schema.STRING_SCHEMA, serialized);
+                newValue = Values.convertToLong(Schema.STRING_SCHEMA, value);
                 break;
             case FLOAT32:
-                newValue = Values.convertToFloat(Schema.STRING_SCHEMA, serialized);
+                newValue = Values.convertToFloat(Schema.STRING_SCHEMA, value);
                 break;
             case FLOAT64:
-                newValue = Values.convertToDouble(Schema.STRING_SCHEMA, serialized);
+                newValue = Values.convertToDouble(Schema.STRING_SCHEMA, value);
                 break;
             case BOOLEAN:
-                newValue = Values.convertToBoolean(Schema.STRING_SCHEMA, serialized);
+                newValue = Values.convertToBoolean(Schema.STRING_SCHEMA, value);
                 break;
             case ARRAY:
-                newValue = Values.convertToList(Schema.STRING_SCHEMA, serialized);
+                newValue = Values.convertToList(Schema.STRING_SCHEMA, value);
                 break;
             case MAP:
-                newValue = Values.convertToMap(Schema.STRING_SCHEMA, serialized);
+                newValue = Values.convertToMap(Schema.STRING_SCHEMA, value);
                 break;
             case STRUCT:
-                newValue = Values.convertToStruct(Schema.STRING_SCHEMA, serialized);
-                break;
             case BYTES:
                 fail("unexpected schema type");
                 break;
         }
-        newSchema = Values.inferSchema(newValue);
+        Schema newSchema = Values.inferSchema(newValue);
         return new SchemaAndValue(newSchema, newValue);
     }
 
@@ -1000,4 +1281,16 @@ public class ValuesTest {
             assertEquals(result, result2);
         }
     }
+
+    protected void assertShortCircuit(Schema schema, Object value) {
+        SchemaAndValue result = convertTo(schema, value);
+
+        if (value == null) {
+            assertNull(result.schema());
+            assertNull(result.value());
+        } else {
+            assertEquals(value, result.value());
+            assertEquals(schema, result.schema());
+        }
+    }
 }
