@@ -16,6 +16,7 @@
  */
 package org.apache.kafka.connect.runtime;
 
+import org.apache.kafka.clients.CommonClientConfigs;
 import org.apache.kafka.common.MetricName;
 import org.apache.kafka.common.metrics.Sensor;
 import org.apache.kafka.common.metrics.stats.Avg;
@@ -37,6 +38,7 @@ import static org.junit.Assert.assertNotNull;
 import static org.junit.Assert.assertNotSame;
 import static org.junit.Assert.assertSame;
 import static org.junit.Assert.assertThrows;
+import static org.junit.Assert.assertTrue;
 
 public class ConnectMetricsTest {
 
@@ -145,6 +147,30 @@ public class ConnectMetricsTest {
         assertSame(originalSensor, recreatedSensor);
     }
 
+    @Test
+    public void testMetricReporters() {
+        assertEquals(1, metrics.metrics().reporters().size());
+    }
+
+    @Test
+    @SuppressWarnings("deprecation")
+    public void testDisableJmxReporter() {
+        Map<String, String> props = new HashMap<>(DEFAULT_WORKER_CONFIG);
+        props.put(CommonClientConfigs.AUTO_INCLUDE_JMX_REPORTER_CONFIG, "false");
+        ConnectMetrics cm = new ConnectMetrics("worker-testDisableJmxReporter", new WorkerConfig(WorkerConfig.baseConfigDef(), props), new MockTime(), "cluster-1");
+        assertTrue(cm.metrics().reporters().isEmpty());
+        cm.stop();
+    }
+
+    @Test
+    public void testExplicitlyEnableJmxReporter() {
+        Map<String, String> props = new HashMap<>(DEFAULT_WORKER_CONFIG);
+        props.put(CommonClientConfigs.METRIC_REPORTER_CLASSES_CONFIG, "org.apache.kafka.common.metrics.JmxReporter");
+        ConnectMetrics cm = new ConnectMetrics("worker-testExplicitlyEnableJmxReporter", new WorkerConfig(WorkerConfig.baseConfigDef(), props), new MockTime(), "cluster-1");
+        assertEquals(1, cm.metrics().reporters().size());
+        cm.stop();
+    }
+
     private Sensor addToGroup(ConnectMetrics connectMetrics, boolean shouldClose) {
         ConnectMetricsRegistry registry = connectMetrics.registry();
         ConnectMetrics.MetricGroup metricGroup = connectMetrics.group(registry.taskGroupName(),
