@@ -32,6 +32,7 @@ import org.apache.kafka.common.config.ConfigDef;
 import org.apache.kafka.common.config.ConfigException;
 import org.apache.kafka.common.config.provider.ConfigProvider;
 import org.apache.kafka.common.utils.LogCaptureAppender;
+import org.apache.kafka.common.utils.Utils;
 import org.apache.kafka.connect.components.Versioned;
 import org.apache.kafka.connect.connector.policy.AllConnectorClientConfigOverridePolicy;
 import org.apache.kafka.connect.connector.policy.ConnectorClientConfigOverridePolicy;
@@ -71,6 +72,7 @@ import static org.junit.Assert.assertNull;
 import static org.junit.Assert.assertSame;
 import static org.junit.Assert.assertThrows;
 import static org.junit.Assert.assertTrue;
+import static org.junit.jupiter.api.Assertions.assertInstanceOf;
 
 public class PluginsTest {
 
@@ -151,7 +153,7 @@ public class PluginsTest {
                                                                      WorkerConfig.HEADER_CONVERTER_CLASS_CONFIG,
                                                                      ClassLoaderUsage.CURRENT_CLASSLOADER);
         assertNotNull(headerConverter);
-        assertTrue(headerConverter instanceof TestHeaderConverter);
+        assertInstanceOf(TestHeaderConverter.class, headerConverter);
         this.headerConverter = (TestHeaderConverter) headerConverter;
 
         // Validate extra configs got passed through to overridden converters
@@ -162,7 +164,7 @@ public class PluginsTest {
                                                      WorkerConfig.HEADER_CONVERTER_CLASS_CONFIG,
                                                      ClassLoaderUsage.PLUGINS);
         assertNotNull(headerConverter);
-        assertTrue(headerConverter instanceof TestHeaderConverter);
+        assertInstanceOf(TestHeaderConverter.class, headerConverter);
         this.headerConverter = (TestHeaderConverter) headerConverter;
 
         // Validate extra configs got passed through to overridden converters
@@ -207,7 +209,7 @@ public class PluginsTest {
                                                      WorkerConfig.HEADER_CONVERTER_CLASS_CONFIG,
                                                      ClassLoaderUsage.PLUGINS);
         assertNotNull(headerConverter);
-        assertTrue(headerConverter instanceof SimpleHeaderConverter);
+        assertInstanceOf(SimpleHeaderConverter.class, headerConverter);
     }
 
     @Test
@@ -449,7 +451,7 @@ public class PluginsTest {
     }
 
     @Test
-    public void pluginClassLoaderReadVersionFromResourceExistingOnlyInChild() throws Exception {
+    public void pluginClassLoaderReadVersionFromResourceExistingOnlyInChild() {
         assertClassLoaderReadsVersionFromResource(
                 TestPlugin.ALIASED_STATIC_FIELD,
                 TestPlugin.READ_VERSION_FROM_RESOURCE_V1,
@@ -458,7 +460,7 @@ public class PluginsTest {
     }
 
     @Test
-    public void pluginClassLoaderReadVersionFromResourceExistingOnlyInParent() throws Exception {
+    public void pluginClassLoaderReadVersionFromResourceExistingOnlyInParent() {
         assertClassLoaderReadsVersionFromResource(
                 TestPlugin.READ_VERSION_FROM_RESOURCE_V1,
                 TestPlugin.ALIASED_STATIC_FIELD,
@@ -467,7 +469,7 @@ public class PluginsTest {
     }
 
     @Test
-    public void pluginClassLoaderReadVersionFromResourceExistingInParentAndChild() throws Exception {
+    public void pluginClassLoaderReadVersionFromResourceExistingInParentAndChild() {
         assertClassLoaderReadsVersionFromResource(
                 TestPlugin.READ_VERSION_FROM_RESOURCE_V1,
                 TestPlugin.READ_VERSION_FROM_RESOURCE_V2,
@@ -595,6 +597,25 @@ public class PluginsTest {
         }
     }
 
+    @Test
+    public void testAliasesInConverters() throws ClassNotFoundException {
+        ClassLoader connectorLoader = plugins.connectorLoader(TestPlugin.SAMPLING_CONNECTOR.className());
+        try (LoaderSwap loaderSwap = plugins.withClassLoader(connectorLoader)) {
+            String configKey = "config.key";
+            String alias = "SamplingConverter";
+            assertTrue(TestPlugin.SAMPLING_CONVERTER.className().contains(alias));
+            ConfigDef def = new ConfigDef().define(configKey, ConfigDef.Type.CLASS, ConfigDef.Importance.HIGH, "docstring");
+            AbstractConfig config = new AbstractConfig(def, Collections.singletonMap(configKey, alias));
+
+            assertNotNull(config.getClass(configKey));
+            assertNotNull(config.getConfiguredInstance(configKey, Converter.class));
+            assertNotNull(plugins.newConverter(config, configKey, ClassLoaderUsage.CURRENT_CLASSLOADER));
+            assertNotNull(plugins.newConverter(config, configKey, ClassLoaderUsage.PLUGINS));
+
+            assertNotNull(Utils.newInstance(alias, Converter.class));
+        }
+    }
+
     private void assertClassLoaderReadsVersionFromResource(
             TestPlugin parentResource, TestPlugin childResource, String className, String... expectedVersions) {
         URL[] systemPath = TestPlugins.pluginPath(parentResource)
@@ -657,23 +678,11 @@ public class PluginsTest {
         }
     }
 
-    public static void assertInstanceOf(Class<?> expected, Object actual, String message) {
-        assertTrue(
-            "Expected an instance of " + expected.getSimpleName() + ", found " + actual + " instead: " + message,
-            expected.isInstance(actual)
-        );
-    }
-
     protected void instantiateAndConfigureConverter(String configPropName, ClassLoaderUsage classLoaderUsage) {
         converter = (TestConverter) plugins.newConverter(config, configPropName, classLoaderUsage);
         assertNotNull(converter);
     }
 
-    protected void instantiateAndConfigureHeaderConverter(String configPropName) {
-        headerConverter = (TestHeaderConverter) plugins.newHeaderConverter(config, configPropName, ClassLoaderUsage.CURRENT_CLASSLOADER);
-        assertNotNull(headerConverter);
-    }
-
     protected void instantiateAndConfigureInternalConverter(boolean isKey, Map<String, String> config) {
         internalConverter = (TestInternalConverter) plugins.newInternalConverter(isKey, TestInternalConverter.class.getName(), config);
         assertNotNull(internalConverter);
