@@ -23,15 +23,25 @@ import org.apache.kafka.metadata.properties.MetaPropertiesEnsemble;
 import org.apache.kafka.metadata.properties.MetaPropertiesVersion;
 
 import java.io.File;
-import java.nio.file.Paths;
+import java.util.Collections;
+import java.util.HashMap;
+import java.util.Map;
+import java.util.Objects;
 import java.util.Optional;
 
 public class ControllerNode implements TestKitNode {
+    public static Builder builder() {
+        return new Builder();
+    }
+
     public static class Builder {
         private int id = -1;
-        private String baseDirectory = null;
-        private String metadataDirectory = null;
-        private Uuid clusterId = null;
+        private String baseDirectory;
+        private Uuid clusterId;
+        private boolean combined;
+        private Map<String, String> propertyOverrides = Collections.emptyMap();
+
+        private Builder() {}
 
         public int id() {
             return id;
@@ -42,32 +52,35 @@ public class ControllerNode implements TestKitNode {
             return this;
         }
 
-        public Builder setMetadataDirectory(String metadataDirectory) {
-            this.metadataDirectory = metadataDirectory;
+        public Builder setClusterId(Uuid clusterId) {
+            this.clusterId = clusterId;
             return this;
         }
 
-        public ControllerNode build(
-            String baseDirectory,
-            Uuid clusterId,
-            boolean combined
-        ) {
+        public Builder setBaseDirectory(String baseDirectory) {
+            this.baseDirectory = baseDirectory;
+            return this;
+        }
+
+        public Builder setCombined(boolean combined) {
+            this.combined = combined;
+            return this;
+        }
+
+        public Builder setPropertyOverrides(Map<String, String> propertyOverrides) {
+            this.propertyOverrides = Collections.unmodifiableMap(new HashMap<>(propertyOverrides));
+            return this;
+        }
+
+        public ControllerNode build() {
             if (id == -1) {
-                throw new RuntimeException("You must set the node id.");
+                throw new IllegalArgumentException("You must set the node id.");
             }
             if (baseDirectory == null) {
-                throw new RuntimeException("You must set the base directory.");
-            }
-            if (metadataDirectory == null) {
-                if (combined) {
-                    metadataDirectory = String.format("combined_%d", id);
-                } else {
-                    metadataDirectory = String.format("controller_%d", id);
-                }
-            }
-            if (!Paths.get(metadataDirectory).isAbsolute()) {
-                metadataDirectory = new File(baseDirectory, metadataDirectory).getAbsolutePath();
+                throw new IllegalArgumentException("You must set the base directory.");
             }
+            String metadataDirectory = new File(baseDirectory,
+                combined ? String.format("combined_%d_0", id) : String.format("controller_%d", id)).getAbsolutePath();
             MetaPropertiesEnsemble.Copier copier =
                 new MetaPropertiesEnsemble.Copier(MetaPropertiesEnsemble.EMPTY);
             copier.setMetaLogDir(Optional.of(metadataDirectory));
@@ -77,7 +90,7 @@ public class ControllerNode implements TestKitNode {
                 setNodeId(id).
                 setDirectoryId(copier.generateValidDirectoryId()).
                 build());
-            return new ControllerNode(copier.copy(), combined);
+            return new ControllerNode(copier.copy(), combined, propertyOverrides);
         }
     }
 
@@ -85,12 +98,16 @@ public class ControllerNode implements TestKitNode {
 
     private final boolean combined;
 
-    ControllerNode(
+    private final Map<String, String> propertyOverrides;
+
+    private ControllerNode(
         MetaPropertiesEnsemble initialMetaPropertiesEnsemble,
-        boolean combined
+        boolean combined,
+        Map<String, String> propertyOverrides
     ) {
-        this.initialMetaPropertiesEnsemble = initialMetaPropertiesEnsemble;
+        this.initialMetaPropertiesEnsemble = Objects.requireNonNull(initialMetaPropertiesEnsemble);
         this.combined = combined;
+        this.propertyOverrides = Objects.requireNonNull(propertyOverrides);
     }
 
     @Override
@@ -102,4 +119,8 @@ public class ControllerNode implements TestKitNode {
     public boolean combined() {
         return combined;
     }
+
+    public Map<String, String> propertyOverrides() {
+        return propertyOverrides;
+    }
 }
