@@ -5,7 +5,7 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *    http://www.apache.org/licenses/LICENSE-2.0
  *
  * Unless required by applicable law or agreed to in writing, software
@@ -26,19 +26,19 @@ import javax.security.auth.login.Configuration
 import kafka.utils.{CoreUtils, Logging, TestInfoUtils, TestUtils}
 import kafka.zk.{AdminZkClient, EmbeddedZookeeper, KafkaZkClient}
 import org.apache.kafka.clients.consumer.GroupProtocol
-import org.apache.kafka.common.config.TopicConfig
+import org.apache.kafka.common.metadata.FeatureLevelRecord
 import org.apache.kafka.common.metrics.Metrics
-import org.apache.kafka.common.{DirectoryId, Uuid}
 import org.apache.kafka.common.security.JaasUtils
 import org.apache.kafka.common.security.auth.SecurityProtocol
 import org.apache.kafka.common.utils.{Exit, Time}
-import org.apache.kafka.server.config.ServerTopicConfigSynonyms
+import org.apache.kafka.common.{DirectoryId, Uuid}
 import org.apache.kafka.metadata.bootstrap.BootstrapMetadata
-import org.apache.kafka.common.metadata.FeatureLevelRecord
 import org.apache.kafka.metadata.properties.MetaPropertiesEnsemble.VerificationFlag.{REQUIRE_AT_LEAST_ONE_VALID, REQUIRE_METADATA_LOG_DIR}
 import org.apache.kafka.metadata.properties.{MetaProperties, MetaPropertiesEnsemble, MetaPropertiesVersion}
-import org.apache.kafka.raft.RaftConfig.{AddressSpec, InetAddressSpec}
+import org.apache.kafka.network.SocketServerConfigs
+import org.apache.kafka.raft.QuorumConfig
 import org.apache.kafka.server.common.{ApiMessageAndVersion, MetadataVersion}
+import org.apache.kafka.server.config.{KRaftConfigs, ServerConfigs}
 import org.apache.kafka.server.fault.{FaultHandler, MockFaultHandler}
 import org.apache.zookeeper.client.ZKClientConfig
 import org.apache.zookeeper.{WatchedEvent, Watcher, ZooKeeper}
@@ -90,7 +90,7 @@ class KRaftQuorumImplementation(
   val controllerServer: ControllerServer,
   val faultHandlerFactory: FaultHandlerFactory,
   val metadataDir: File,
-  val controllerQuorumVotersFuture: CompletableFuture[util.Map[Integer, AddressSpec]],
+  val controllerQuorumVotersFuture: CompletableFuture[util.Map[Integer, InetSocketAddress]],
   val clusterId: String,
   val log: Logging,
   val faultHandler: FaultHandler
@@ -103,7 +103,7 @@ class KRaftQuorumImplementation(
   ): KafkaBroker = {
     val metaPropertiesEnsemble = {
       val loader = new MetaPropertiesEnsemble.Loader()
-      config.logDirs.foreach(loader.addLogDir(_))
+      config.logDirs.foreach(loader.addLogDir)
       loader.addMetadataLogDir(config.metadataLogDir)
       val ensemble = loader.load()
       val copier = new MetaPropertiesEnsemble.Copier(ensemble)
@@ -116,7 +116,7 @@ class KRaftQuorumImplementation(
           build())
       })
       copier.setPreWriteHandler((logDir, _, _) => {
-        Files.createDirectories(Paths.get(logDir));
+        Files.createDirectories(Paths.get(logDir))
       })
       copier.writeLogDirChanges()
       copier.copy()
@@ -124,12 +124,15 @@ class KRaftQuorumImplementation(
     metaPropertiesEnsemble.verify(Optional.of(clusterId),
       OptionalInt.of(config.nodeId),
       util.EnumSet.of(REQUIRE_AT_LEAST_ONE_VALID, REQUIRE_METADATA_LOG_DIR))
-    val sharedServer = new SharedServer(config,
+    val sharedServer = new SharedServer(
+      config,
       metaPropertiesEnsemble,
-      Time.SYSTEM,
+      time,
       new Metrics(),
       controllerQuorumVotersFuture,
-      faultHandlerFactory)
+      controllerQuorumVotersFuture.get().values(),
+      faultHandlerFactory
+    )
     var broker: BrokerServer = null
     try {
       broker = new BrokerServer(sharedServer)
@@ -180,7 +183,7 @@ abstract class QuorumTestHarness extends Logging {
   protected def metadataVersion: MetadataVersion = MetadataVersion.latestTesting()
 
   private var testInfo: TestInfo = _
-  private var implementation: QuorumImplementation = _
+  protected var implementation: QuorumImplementation = _
 
   val bootstrapRecords: ListBuffer[ApiMessageAndVersion] = ListBuffer()
 
@@ -262,7 +265,7 @@ abstract class QuorumTestHarness extends Logging {
     this.testInfo = testInfo
     Exit.setExitProcedure((code, message) => {
       try {
-        throw new RuntimeException(s"exit(${code}, ${message}) called!")
+        throw new RuntimeException(s"exit($code, $message) called!")
       } catch {
         case e: Throwable => error("test error", e)
           throw e
@@ -272,7 +275,7 @@ abstract class QuorumTestHarness extends Logging {
     })
     Exit.setHaltProcedure((code, message) => {
       try {
-        throw new RuntimeException(s"halt(${code}, ${message}) called!")
+        throw new RuntimeException(s"halt($code, $message) called!")
       } catch {
         case e: Throwable => error("test error", e)
           throw e
@@ -312,18 +315,23 @@ abstract class QuorumTestHarness extends Logging {
   def optionalMetadataRecords: Option[ArrayBuffer[ApiMessageAndVersion]] = None
 
   private def newKRaftQuorum(testInfo: TestInfo): KRaftQuorumImplementation = {
+    newKRaftQuorum(new Properties())
+  }
+
+  protected def newKRaftQuorum(overridingProps: Properties): KRaftQuorumImplementation = {
     val propsList = kraftControllerConfigs()
     if (propsList.size != 1) {
       throw new RuntimeException("Only one KRaft controller is supported for now.")
     }
-    val props = propsList(0)
-    props.setProperty(KafkaConfig.ServerMaxStartupTimeMsProp, TimeUnit.MINUTES.toMillis(10).toString)
-    props.setProperty(KafkaConfig.ProcessRolesProp, "controller")
-    props.setProperty(KafkaConfig.UnstableMetadataVersionsEnableProp, "true")
-    if (props.getProperty(KafkaConfig.NodeIdProp) == null) {
-      props.setProperty(KafkaConfig.NodeIdProp, "1000")
+    val props = propsList.head
+    props.putAll(overridingProps)
+    props.setProperty(KRaftConfigs.SERVER_MAX_STARTUP_TIME_MS_CONFIG, TimeUnit.MINUTES.toMillis(10).toString)
+    props.setProperty(KRaftConfigs.PROCESS_ROLES_CONFIG, "controller")
+    props.setProperty(ServerConfigs.UNSTABLE_FEATURE_VERSIONS_ENABLE_CONFIG, "true")
+    if (props.getProperty(KRaftConfigs.NODE_ID_CONFIG) == null) {
+      props.setProperty(KRaftConfigs.NODE_ID_CONFIG, "1000")
     }
-    val nodeId = Integer.parseInt(props.getProperty(KafkaConfig.NodeIdProp))
+    val nodeId = Integer.parseInt(props.getProperty(KRaftConfigs.NODE_ID_CONFIG))
     val metadataDir = TestUtils.tempDir()
     val metaProperties = new MetaProperties.Builder().
       setVersion(MetaPropertiesVersion.V1).
@@ -335,7 +343,7 @@ abstract class QuorumTestHarness extends Logging {
     val metadataRecords = new util.ArrayList[ApiMessageAndVersion]
     metadataRecords.add(new ApiMessageAndVersion(new FeatureLevelRecord().
                         setName(MetadataVersion.FEATURE_NAME).
-                        setFeatureLevel(metadataVersion.featureLevel()), 0.toShort));
+                        setFeatureLevel(metadataVersion.featureLevel()), 0.toShort))
 
     optionalMetadataRecords.foreach { metadataArguments =>
       for (record <- metadataArguments) metadataRecords.add(record)
@@ -343,28 +351,29 @@ abstract class QuorumTestHarness extends Logging {
 
     val bootstrapMetadata = BootstrapMetadata.fromRecords(metadataRecords, "test harness")
 
-    props.setProperty(KafkaConfig.MetadataLogDirProp, metadataDir.getAbsolutePath)
+    props.setProperty(KRaftConfigs.METADATA_LOG_DIR_CONFIG, metadataDir.getAbsolutePath)
     val proto = controllerListenerSecurityProtocol.toString
-    props.setProperty(KafkaConfig.ListenerSecurityProtocolMapProp, s"CONTROLLER:$proto")
-    props.setProperty(KafkaConfig.ListenersProp, s"CONTROLLER://localhost:0")
-    props.setProperty(KafkaConfig.ControllerListenerNamesProp, "CONTROLLER")
-    props.setProperty(KafkaConfig.QuorumVotersProp, s"$nodeId@localhost:0")
-    // Setting the configuration to the same value set on the brokers via TestUtils to keep KRaft based and Zk based controller configs are consistent.
-    props.setProperty(ServerTopicConfigSynonyms.serverSynonym(TopicConfig.FILE_DELETE_DELAY_MS_CONFIG), "1000")
+    props.setProperty(SocketServerConfigs.LISTENER_SECURITY_PROTOCOL_MAP_CONFIG, s"CONTROLLER:$proto")
+    props.setProperty(SocketServerConfigs.LISTENERS_CONFIG, s"CONTROLLER://localhost:0")
+    props.setProperty(KRaftConfigs.CONTROLLER_LISTENER_NAMES_CONFIG, "CONTROLLER")
+    props.setProperty(QuorumConfig.QUORUM_VOTERS_CONFIG, s"$nodeId@localhost:0")
     val config = new KafkaConfig(props)
-    val controllerQuorumVotersFuture = new CompletableFuture[util.Map[Integer, AddressSpec]]
+    val controllerQuorumVotersFuture = new CompletableFuture[util.Map[Integer, InetSocketAddress]]
     val metaPropertiesEnsemble = new MetaPropertiesEnsemble.Loader().
       addMetadataLogDir(metadataDir.getAbsolutePath).
       load()
     metaPropertiesEnsemble.verify(Optional.of(metaProperties.clusterId().get()),
       OptionalInt.of(nodeId),
       util.EnumSet.of(REQUIRE_AT_LEAST_ONE_VALID, REQUIRE_METADATA_LOG_DIR))
-    val sharedServer = new SharedServer(config,
+    val sharedServer = new SharedServer(
+      config,
       metaPropertiesEnsemble,
       Time.SYSTEM,
       new Metrics(),
       controllerQuorumVotersFuture,
-      faultHandlerFactory)
+      Collections.emptyList(),
+      faultHandlerFactory
+    )
     var controllerServer: ControllerServer = null
     try {
       controllerServer = new ControllerServer(
@@ -377,8 +386,9 @@ abstract class QuorumTestHarness extends Logging {
           error("Error completing controller socket server future", e)
           controllerQuorumVotersFuture.completeExceptionally(e)
         } else {
-          controllerQuorumVotersFuture.complete(Collections.singletonMap(nodeId,
-            new InetAddressSpec(new InetSocketAddress("localhost", port))))
+          controllerQuorumVotersFuture.complete(
+            Collections.singletonMap(nodeId, new InetSocketAddress("localhost", port))
+          )
         }
       })
       controllerServer.startup()
@@ -388,13 +398,15 @@ abstract class QuorumTestHarness extends Logging {
         CoreUtils.swallow(sharedServer.stopForController(), this)
         throw e
     }
-    new KRaftQuorumImplementation(controllerServer,
+    new KRaftQuorumImplementation(
+      controllerServer,
       faultHandlerFactory,
       metadataDir,
       controllerQuorumVotersFuture,
       metaProperties.clusterId.get(),
       this,
-      faultHandler)
+      faultHandler
+    )
   }
 
   private def newZooKeeperQuorum(): ZooKeeperQuorumImplementation = {
