@@ -20,7 +20,7 @@ package kafka.coordinator.group
 import kafka.common.OffsetAndMetadata
 import org.apache.kafka.clients.consumer.ConsumerPartitionAssignor.Subscription
 import org.apache.kafka.clients.consumer.internals.ConsumerProtocol
-import org.apache.kafka.common.TopicPartition
+import org.apache.kafka.common.{TopicIdPartition, TopicPartition, Uuid}
 import org.apache.kafka.common.protocol.Errors
 import org.apache.kafka.common.utils.{MockTime, Time}
 import org.junit.jupiter.api.Assertions._
@@ -160,6 +160,7 @@ class GroupMetadataTest {
     assertThrows(classOf[IllegalStateException], () => group.transitionTo(CompletingRebalance))
   }
 
+  @Test
   def testDeadToDeadIllegalTransition(): Unit = {
     group.transitionTo(PreparingRebalance)
     group.transitionTo(Dead)
@@ -262,7 +263,7 @@ class GroupMetadataTest {
   @Test
   def testOffsetRemovalDuringTransitionFromEmptyToNonEmpty(): Unit = {
     val topic = "foo"
-    val partition = new TopicPartition(topic, 0)
+    val topicIdPartition = new TopicIdPartition(Uuid.randomUuid(), 0, topic)
     val time = new MockTime()
     group = new GroupMetadata("groupId", Empty, time)
 
@@ -277,10 +278,10 @@ class GroupMetadataTest {
     val offset = offsetAndMetadata(offset = 37, timestamp = time.milliseconds())
     val commitRecordOffset = 3
 
-    group.prepareOffsetCommit(Map(partition -> offset))
+    group.prepareOffsetCommit(Map(topicIdPartition -> offset))
     assertTrue(group.hasOffsets)
-    assertEquals(None, group.offset(partition))
-    group.onOffsetCommitAppend(partition, CommitRecordMetadataAndOffset(Some(commitRecordOffset), offset))
+    assertEquals(None, group.offset(topicIdPartition.topicPartition))
+    group.onOffsetCommitAppend(topicIdPartition, CommitRecordMetadataAndOffset(Some(commitRecordOffset), offset))
 
     val offsetRetentionMs = 50000L
     time.sleep(offsetRetentionMs + 1)
@@ -386,170 +387,203 @@ class GroupMetadataTest {
 
   @Test
   def testOffsetCommit(): Unit = {
-    val partition = new TopicPartition("foo", 0)
+    val partition = new TopicIdPartition(Uuid.randomUuid(), 0, "foo")
     val offset = offsetAndMetadata(37)
     val commitRecordOffset = 3
 
     group.prepareOffsetCommit(Map(partition -> offset))
     assertTrue(group.hasOffsets)
-    assertEquals(None, group.offset(partition))
+    assertEquals(None, group.offset(partition.topicPartition))
 
     group.onOffsetCommitAppend(partition, CommitRecordMetadataAndOffset(Some(commitRecordOffset), offset))
     assertTrue(group.hasOffsets)
-    assertEquals(Some(offset), group.offset(partition))
+    assertEquals(Some(offset), group.offset(partition.topicPartition))
   }
 
   @Test
   def testOffsetCommitFailure(): Unit = {
-    val partition = new TopicPartition("foo", 0)
+    val topicIdPartition = new TopicIdPartition(Uuid.randomUuid(), 0, "foo")
     val offset = offsetAndMetadata(37)
 
-    group.prepareOffsetCommit(Map(partition -> offset))
+    group.prepareOffsetCommit(Map(topicIdPartition -> offset))
     assertTrue(group.hasOffsets)
-    assertEquals(None, group.offset(partition))
+    assertEquals(Some(offset), group.pendingOffsetCommit(topicIdPartition))
+    assertEquals(None, group.offset(topicIdPartition.topicPartition))
 
-    group.failPendingOffsetWrite(partition, offset)
+    group.failPendingOffsetWrite(topicIdPartition, offset)
     assertFalse(group.hasOffsets)
-    assertEquals(None, group.offset(partition))
+    assertEquals(None, group.pendingOffsetCommit(topicIdPartition))
+    assertEquals(None, group.offset(topicIdPartition.topicPartition))
   }
 
   @Test
   def testOffsetCommitFailureWithAnotherPending(): Unit = {
-    val partition = new TopicPartition("foo", 0)
+    val topicIdPartition = new TopicIdPartition(Uuid.randomUuid(), 0, "foo")
     val firstOffset = offsetAndMetadata(37)
     val secondOffset = offsetAndMetadata(57)
 
-    group.prepareOffsetCommit(Map(partition -> firstOffset))
+    group.prepareOffsetCommit(Map(topicIdPartition -> firstOffset))
     assertTrue(group.hasOffsets)
-    assertEquals(None, group.offset(partition))
+    assertEquals(Some(firstOffset), group.pendingOffsetCommit(topicIdPartition))
+    assertEquals(None, group.offset(topicIdPartition.topicPartition))
 
-    group.prepareOffsetCommit(Map(partition -> secondOffset))
+    group.prepareOffsetCommit(Map(topicIdPartition -> secondOffset))
     assertTrue(group.hasOffsets)
+    assertEquals(Some(secondOffset), group.pendingOffsetCommit(topicIdPartition))
+    assertEquals(None, group.offset(topicIdPartition.topicPartition))
 
-    group.failPendingOffsetWrite(partition, firstOffset)
+    group.failPendingOffsetWrite(topicIdPartition, firstOffset)
     assertTrue(group.hasOffsets)
-    assertEquals(None, group.offset(partition))
+    assertEquals(Some(secondOffset), group.pendingOffsetCommit(topicIdPartition))
+    assertEquals(None, group.offset(topicIdPartition.topicPartition))
 
-    group.onOffsetCommitAppend(partition, CommitRecordMetadataAndOffset(Some(3L), secondOffset))
+    group.onOffsetCommitAppend(topicIdPartition, CommitRecordMetadataAndOffset(Some(3L), secondOffset))
     assertTrue(group.hasOffsets)
-    assertEquals(Some(secondOffset), group.offset(partition))
+    assertEquals(None, group.pendingOffsetCommit(topicIdPartition))
+    assertEquals(Some(secondOffset), group.offset(topicIdPartition.topicPartition))
   }
 
   @Test
   def testOffsetCommitWithAnotherPending(): Unit = {
-    val partition = new TopicPartition("foo", 0)
+    val topicIdPartition = new TopicIdPartition(Uuid.randomUuid(), 0, "foo")
     val firstOffset = offsetAndMetadata(37)
     val secondOffset = offsetAndMetadata(57)
 
-    group.prepareOffsetCommit(Map(partition -> firstOffset))
+    group.prepareOffsetCommit(Map(topicIdPartition -> firstOffset))
     assertTrue(group.hasOffsets)
-    assertEquals(None, group.offset(partition))
+    assertEquals(Some(firstOffset), group.pendingOffsetCommit(topicIdPartition))
+    assertEquals(None, group.offset(topicIdPartition.topicPartition))
 
-    group.prepareOffsetCommit(Map(partition -> secondOffset))
+    group.prepareOffsetCommit(Map(topicIdPartition -> secondOffset))
     assertTrue(group.hasOffsets)
+    assertEquals(Some(secondOffset), group.pendingOffsetCommit(topicIdPartition))
+    assertEquals(None, group.offset(topicIdPartition.topicPartition))
 
-    group.onOffsetCommitAppend(partition, CommitRecordMetadataAndOffset(Some(4L), firstOffset))
+    group.onOffsetCommitAppend(topicIdPartition, CommitRecordMetadataAndOffset(Some(4L), firstOffset))
     assertTrue(group.hasOffsets)
-    assertEquals(Some(firstOffset), group.offset(partition))
+    assertEquals(Some(secondOffset), group.pendingOffsetCommit(topicIdPartition))
+    assertEquals(Some(firstOffset), group.offset(topicIdPartition.topicPartition))
 
-    group.onOffsetCommitAppend(partition, CommitRecordMetadataAndOffset(Some(5L), secondOffset))
+    group.onOffsetCommitAppend(topicIdPartition, CommitRecordMetadataAndOffset(Some(5L), secondOffset))
     assertTrue(group.hasOffsets)
-    assertEquals(Some(secondOffset), group.offset(partition))
+    assertEquals(None, group.pendingOffsetCommit(topicIdPartition))
+    assertEquals(Some(secondOffset), group.offset(topicIdPartition.topicPartition))
   }
 
   @Test
   def testConsumerBeatsTransactionalOffsetCommit(): Unit = {
-    val partition = new TopicPartition("foo", 0)
+    val topicIdPartition = new TopicIdPartition(Uuid.randomUuid(), 0, "foo")
     val producerId = 13232L
     val txnOffsetCommit = offsetAndMetadata(37)
     val consumerOffsetCommit = offsetAndMetadata(57)
 
-    group.prepareTxnOffsetCommit(producerId, Map(partition -> txnOffsetCommit))
+    group.prepareTxnOffsetCommit(producerId, Map(topicIdPartition -> txnOffsetCommit))
     assertTrue(group.hasOffsets)
-    assertEquals(None, group.offset(partition))
+    assertEquals(Some(CommitRecordMetadataAndOffset(None, txnOffsetCommit)), group.pendingTxnOffsetCommit(producerId, topicIdPartition))
+    assertEquals(None, group.offset(topicIdPartition.topicPartition))
 
-    group.prepareOffsetCommit(Map(partition -> consumerOffsetCommit))
+    group.prepareOffsetCommit(Map(topicIdPartition -> consumerOffsetCommit))
     assertTrue(group.hasOffsets)
+    assertEquals(Some(consumerOffsetCommit), group.pendingOffsetCommit(topicIdPartition))
+    assertEquals(None, group.offset(topicIdPartition.topicPartition))
 
-    group.onTxnOffsetCommitAppend(producerId, partition, CommitRecordMetadataAndOffset(Some(3L), txnOffsetCommit))
-    group.onOffsetCommitAppend(partition, CommitRecordMetadataAndOffset(Some(4L), consumerOffsetCommit))
+    group.onTxnOffsetCommitAppend(producerId, topicIdPartition, CommitRecordMetadataAndOffset(Some(3L), txnOffsetCommit))
+    group.onOffsetCommitAppend(topicIdPartition, CommitRecordMetadataAndOffset(Some(4L), consumerOffsetCommit))
     assertTrue(group.hasOffsets)
-    assertEquals(Some(consumerOffsetCommit), group.offset(partition))
+    assertEquals(Some(CommitRecordMetadataAndOffset(Some(3), txnOffsetCommit)), group.pendingTxnOffsetCommit(producerId, topicIdPartition))
+    assertEquals(None, group.pendingOffsetCommit(topicIdPartition))
+    assertEquals(Some(consumerOffsetCommit), group.offset(topicIdPartition.topicPartition))
 
     group.completePendingTxnOffsetCommit(producerId, isCommit = true)
     assertTrue(group.hasOffsets)
+    assertEquals(None, group.pendingTxnOffsetCommit(producerId, topicIdPartition))
+    assertEquals(None, group.pendingOffsetCommit(topicIdPartition))
     // This is the crucial assertion which validates that we materialize offsets in offset order, not transactional order.
-    assertEquals(Some(consumerOffsetCommit), group.offset(partition))
+    assertEquals(Some(consumerOffsetCommit), group.offset(topicIdPartition.topicPartition))
   }
 
   @Test
   def testTransactionBeatsConsumerOffsetCommit(): Unit = {
-    val partition = new TopicPartition("foo", 0)
+    val topicIdPartition = new TopicIdPartition(Uuid.randomUuid(), 0, "foo")
     val producerId = 13232L
     val txnOffsetCommit = offsetAndMetadata(37)
     val consumerOffsetCommit = offsetAndMetadata(57)
 
-    group.prepareTxnOffsetCommit(producerId, Map(partition -> txnOffsetCommit))
+    group.prepareTxnOffsetCommit(producerId, Map(topicIdPartition -> txnOffsetCommit))
     assertTrue(group.hasOffsets)
-    assertEquals(None, group.offset(partition))
+    assertEquals(Some(CommitRecordMetadataAndOffset(None, txnOffsetCommit)), group.pendingTxnOffsetCommit(producerId, topicIdPartition))
+    assertEquals(None, group.offset(topicIdPartition.topicPartition))
 
-    group.prepareOffsetCommit(Map(partition -> consumerOffsetCommit))
+    group.prepareOffsetCommit(Map(topicIdPartition -> consumerOffsetCommit))
     assertTrue(group.hasOffsets)
+    assertEquals(Some(consumerOffsetCommit), group.pendingOffsetCommit(topicIdPartition))
 
-    group.onOffsetCommitAppend(partition, CommitRecordMetadataAndOffset(Some(3L), consumerOffsetCommit))
-    group.onTxnOffsetCommitAppend(producerId, partition, CommitRecordMetadataAndOffset(Some(4L), txnOffsetCommit))
+    group.onOffsetCommitAppend(topicIdPartition, CommitRecordMetadataAndOffset(Some(3L), consumerOffsetCommit))
+    group.onTxnOffsetCommitAppend(producerId, topicIdPartition, CommitRecordMetadataAndOffset(Some(4L), txnOffsetCommit))
     assertTrue(group.hasOffsets)
+    assertEquals(Some(CommitRecordMetadataAndOffset(Some(4), txnOffsetCommit)), group.pendingTxnOffsetCommit(producerId, topicIdPartition))
+    assertEquals(None, group.pendingOffsetCommit(topicIdPartition))
     // The transactional offset commit hasn't been committed yet, so we should materialize the consumer offset commit.
-    assertEquals(Some(consumerOffsetCommit), group.offset(partition))
+    assertEquals(Some(consumerOffsetCommit), group.offset(topicIdPartition.topicPartition))
 
     group.completePendingTxnOffsetCommit(producerId, isCommit = true)
     assertTrue(group.hasOffsets)
     // The transactional offset commit has been materialized and the transactional commit record is later in the log,
     // so it should be materialized.
-    assertEquals(Some(txnOffsetCommit), group.offset(partition))
+    assertEquals(None, group.pendingTxnOffsetCommit(producerId, topicIdPartition))
+    assertEquals(None, group.pendingOffsetCommit(topicIdPartition))
+    assertEquals(Some(txnOffsetCommit), group.offset(topicIdPartition.topicPartition))
   }
 
   @Test
   def testTransactionalCommitIsAbortedAndConsumerCommitWins(): Unit = {
-    val partition = new TopicPartition("foo", 0)
+    val topicIdPartition = new TopicIdPartition(Uuid.randomUuid(), 0, "foo")
     val producerId = 13232L
     val txnOffsetCommit = offsetAndMetadata(37)
     val consumerOffsetCommit = offsetAndMetadata(57)
 
-    group.prepareTxnOffsetCommit(producerId, Map(partition -> txnOffsetCommit))
+    group.prepareTxnOffsetCommit(producerId, Map(topicIdPartition -> txnOffsetCommit))
     assertTrue(group.hasOffsets)
-    assertEquals(None, group.offset(partition))
+    assertEquals(None, group.offset(topicIdPartition.topicPartition))
+    assertEquals(Some(CommitRecordMetadataAndOffset(None, txnOffsetCommit)), group.pendingTxnOffsetCommit(producerId, topicIdPartition))
 
-    group.prepareOffsetCommit(Map(partition -> consumerOffsetCommit))
+    group.prepareOffsetCommit(Map(topicIdPartition -> consumerOffsetCommit))
     assertTrue(group.hasOffsets)
+    assertEquals(None, group.offset(topicIdPartition.topicPartition))
+    assertEquals(Some(CommitRecordMetadataAndOffset(None, txnOffsetCommit)), group.pendingTxnOffsetCommit(producerId, topicIdPartition))
+    assertEquals(Some(consumerOffsetCommit), group.pendingOffsetCommit(topicIdPartition))
 
-    group.onOffsetCommitAppend(partition, CommitRecordMetadataAndOffset(Some(3L), consumerOffsetCommit))
-    group.onTxnOffsetCommitAppend(producerId, partition, CommitRecordMetadataAndOffset(Some(4L), txnOffsetCommit))
+    group.onOffsetCommitAppend(topicIdPartition, CommitRecordMetadataAndOffset(Some(3L), consumerOffsetCommit))
+    group.onTxnOffsetCommitAppend(producerId, topicIdPartition, CommitRecordMetadataAndOffset(Some(4L), txnOffsetCommit))
     assertTrue(group.hasOffsets)
+    assertEquals(Some(CommitRecordMetadataAndOffset(Some(4L), txnOffsetCommit)), group.pendingTxnOffsetCommit(producerId, topicIdPartition))
     // The transactional offset commit hasn't been committed yet, so we should materialize the consumer offset commit.
-    assertEquals(Some(consumerOffsetCommit), group.offset(partition))
+    assertEquals(Some(consumerOffsetCommit), group.offset(topicIdPartition.topicPartition))
 
     group.completePendingTxnOffsetCommit(producerId, isCommit = false)
     assertTrue(group.hasOffsets)
     // The transactional offset commit should be discarded and the consumer offset commit should continue to be
     // materialized.
     assertFalse(group.hasPendingOffsetCommitsFromProducer(producerId))
-    assertEquals(Some(consumerOffsetCommit), group.offset(partition))
+    assertEquals(None, group.pendingTxnOffsetCommit(producerId, topicIdPartition))
+    assertEquals(Some(consumerOffsetCommit), group.offset(topicIdPartition.topicPartition))
   }
 
   @Test
   def testFailedTxnOffsetCommitLeavesNoPendingState(): Unit = {
-    val partition = new TopicPartition("foo", 0)
+    val topicIdPartition = new TopicIdPartition(Uuid.randomUuid(), 0, "foo")
     val producerId = 13232L
     val txnOffsetCommit = offsetAndMetadata(37)
 
-    group.prepareTxnOffsetCommit(producerId, Map(partition -> txnOffsetCommit))
+    group.prepareTxnOffsetCommit(producerId, Map(topicIdPartition -> txnOffsetCommit))
     assertTrue(group.hasPendingOffsetCommitsFromProducer(producerId))
     assertTrue(group.hasOffsets)
-    assertEquals(None, group.offset(partition))
-    group.failPendingTxnOffsetCommit(producerId, partition)
+    assertEquals(Some(CommitRecordMetadataAndOffset(None, txnOffsetCommit)), group.pendingTxnOffsetCommit(producerId, topicIdPartition))
+    assertEquals(None, group.offset(topicIdPartition.topicPartition))
+    group.failPendingTxnOffsetCommit(producerId, topicIdPartition)
     assertFalse(group.hasOffsets)
     assertFalse(group.hasPendingOffsetCommitsFromProducer(producerId))
+    assertEquals(None, group.pendingTxnOffsetCommit(producerId, topicIdPartition))
 
     // The commit marker should now have no effect.
     group.completePendingTxnOffsetCommit(producerId, isCommit = true)
@@ -706,21 +740,21 @@ class GroupMetadataTest {
 
   @Test
   def testHasPendingNonTxnOffsets(): Unit = {
-    val partition = new TopicPartition("foo", 0)
+    val partition = new TopicIdPartition(Uuid.randomUuid(), 0, "foo")
     val offset = offsetAndMetadata(37)
 
     group.prepareOffsetCommit(Map(partition -> offset))
-    assertTrue(group.hasPendingOffsetCommitsForTopicPartition(partition))
+    assertTrue(group.hasPendingOffsetCommitsForTopicPartition(partition.topicPartition))
   }
 
   @Test
   def testHasPendingTxnOffsets(): Unit = {
-    val txnPartition = new TopicPartition("foo", 1)
+    val txnPartition = new TopicIdPartition(Uuid.randomUuid(), 0, "foo")
     val offset = offsetAndMetadata(37)
     val producerId = 5
 
     group.prepareTxnOffsetCommit(producerId, Map(txnPartition -> offset))
-    assertTrue(group.hasPendingOffsetCommitsForTopicPartition(txnPartition))
+    assertTrue(group.hasPendingOffsetCommitsForTopicPartition(txnPartition.topicPartition))
 
     assertFalse(group.hasPendingOffsetCommitsForTopicPartition(new TopicPartition("non-exist", 0)))
   }
@@ -765,7 +799,7 @@ class GroupMetadataTest {
     assertTrue(group.has(memberId))
     assertTrue(group.hasStaticMember(groupInstanceId))
 
-    // We aren ot permitted to add the member again if it is already present
+    // We are not permitted to add the member again if it is already present
     assertThrows(classOf[IllegalStateException], () => group.add(member))
   }
 
