@@ -19,15 +19,16 @@ package kafka.zk.migration
 import kafka.api.LeaderAndIsr
 import kafka.controller.{LeaderIsrAndControllerEpoch, ReplicaAssignment}
 import kafka.coordinator.transaction.{ProducerIdManager, ZkProducerIdManager}
-import kafka.server.{ConfigType, KafkaConfig}
-import org.apache.kafka.common.config.{ConfigResource, TopicConfig}
+import org.apache.kafka.common.config.{ConfigResource, SslConfigs, TopicConfig}
 import org.apache.kafka.common.errors.ControllerMovedException
 import org.apache.kafka.common.metadata.{ConfigRecord, MetadataRecordType, PartitionRecord, ProducerIdsRecord, TopicRecord}
-import org.apache.kafka.common.{TopicPartition, Uuid}
+import org.apache.kafka.common.{DirectoryId, TopicPartition, Uuid}
 import org.apache.kafka.image.{MetadataDelta, MetadataImage, MetadataProvenance}
 import org.apache.kafka.metadata.migration.{KRaftMigrationZkWriter, ZkMigrationLeadershipState}
 import org.apache.kafka.metadata.{LeaderRecoveryState, PartitionRegistration}
+import org.apache.kafka.server.config.ReplicationConfigs
 import org.apache.kafka.server.common.ApiMessageAndVersion
+import org.apache.kafka.server.config.ConfigType
 import org.junit.jupiter.api.Assertions.{assertEquals, assertThrows, assertTrue, fail}
 import org.junit.jupiter.api.Test
 
@@ -79,8 +80,24 @@ class ZkMigrationClientTest extends ZkMigrationTestHarness {
     assertEquals(0, migrationState.migrationZkVersion())
 
     val partitions = Map(
-      0 -> new PartitionRegistration.Builder().setReplicas(Array(0, 1, 2)).setIsr(Array(1, 2)).setLeader(1).setLeaderRecoveryState(LeaderRecoveryState.RECOVERED).setLeaderEpoch(6).setPartitionEpoch(-1).build(),
-      1 -> new PartitionRegistration.Builder().setReplicas(Array(1, 2, 3)).setIsr(Array(3)).setLeader(3).setLeaderRecoveryState(LeaderRecoveryState.RECOVERED).setLeaderEpoch(7).setPartitionEpoch(-1).build()
+      0 -> new PartitionRegistration.Builder()
+        .setReplicas(Array(0, 1, 2))
+        .setDirectories(DirectoryId.migratingArray(3))
+        .setIsr(Array(1, 2))
+        .setLeader(1)
+        .setLeaderRecoveryState(LeaderRecoveryState.RECOVERED)
+        .setLeaderEpoch(6)
+        .setPartitionEpoch(-1)
+        .build(),
+      1 -> new PartitionRegistration.Builder()
+        .setReplicas(Array(1, 2, 3))
+        .setDirectories(DirectoryId.migratingArray(3))
+        .setIsr(Array(3))
+        .setLeader(3)
+        .setLeaderRecoveryState(LeaderRecoveryState.RECOVERED)
+        .setLeaderEpoch(7)
+        .setPartitionEpoch(-1)
+        .build()
     ).map { case (k, v) => Integer.valueOf(k) -> v }.asJava
     migrationState = migrationClient.topicClient().updateTopicPartitions(Map("test" -> partitions).asJava, migrationState)
     assertEquals(1, migrationState.migrationZkVersion())
@@ -106,8 +123,24 @@ class ZkMigrationClientTest extends ZkMigrationTestHarness {
     assertEquals(0, migrationState.migrationZkVersion())
 
     val partitions = Map(
-      0 -> new PartitionRegistration.Builder().setReplicas(Array(0, 1, 2)).setIsr(Array(0, 1, 2)).setLeader(0).setLeaderRecoveryState(LeaderRecoveryState.RECOVERED).setLeaderEpoch(0).setPartitionEpoch(-1).build(),
-      1 -> new PartitionRegistration.Builder().setReplicas(Array(1, 2, 3)).setIsr(Array(1, 2, 3)).setLeader(1).setLeaderRecoveryState(LeaderRecoveryState.RECOVERED).setLeaderEpoch(0).setPartitionEpoch(-1).build()
+      0 -> new PartitionRegistration.Builder()
+        .setReplicas(Array(0, 1, 2))
+        .setDirectories(DirectoryId.unassignedArray(3))
+        .setIsr(Array(0, 1, 2))
+        .setLeader(0)
+        .setLeaderRecoveryState(LeaderRecoveryState.RECOVERED)
+        .setLeaderEpoch(0)
+        .setPartitionEpoch(-1)
+        .build(),
+      1 -> new PartitionRegistration.Builder()
+        .setReplicas(Array(1, 2, 3))
+        .setDirectories(DirectoryId.unassignedArray(3))
+        .setIsr(Array(1, 2, 3))
+        .setLeader(1)
+        .setLeaderRecoveryState(LeaderRecoveryState.RECOVERED)
+        .setLeaderEpoch(0)
+        .setPartitionEpoch(-1)
+        .build()
     ).map { case (k, v) => Integer.valueOf(k) -> v }.asJava
     migrationState = migrationClient.topicClient().createTopic("test", Uuid.randomUuid(), partitions, migrationState)
     assertEquals(1, migrationState.migrationZkVersion())
@@ -129,8 +162,24 @@ class ZkMigrationClientTest extends ZkMigrationTestHarness {
     assertEquals(0, migrationState.migrationZkVersion())
 
     val partitions = Map(
-      0 -> new PartitionRegistration.Builder().setReplicas(Array(0, 1, 2)).setIsr(Array(0, 1, 2)).setLeader(0).setLeaderRecoveryState(LeaderRecoveryState.RECOVERED).setLeaderEpoch(0).setPartitionEpoch(-1).build(),
-      1 -> new PartitionRegistration.Builder().setReplicas(Array(1, 2, 3)).setIsr(Array(1, 2, 3)).setLeader(1).setLeaderRecoveryState(LeaderRecoveryState.RECOVERED).setLeaderEpoch(0).setPartitionEpoch(-1).build()
+      0 -> new PartitionRegistration.Builder()
+        .setReplicas(Array(0, 1, 2))
+        .setDirectories(DirectoryId.unassignedArray(3))
+        .setIsr(Array(0, 1, 2))
+        .setLeader(0)
+        .setLeaderRecoveryState(LeaderRecoveryState.RECOVERED)
+        .setLeaderEpoch(0)
+        .setPartitionEpoch(-1)
+        .build(),
+      1 -> new PartitionRegistration.Builder()
+        .setReplicas(Array(1, 2, 3))
+        .setDirectories(DirectoryId.unassignedArray(3))
+        .setIsr(Array(1, 2, 3))
+        .setLeader(1)
+        .setLeaderRecoveryState(LeaderRecoveryState.RECOVERED)
+        .setLeaderEpoch(0)
+        .setPartitionEpoch(-1)
+        .build()
     ).map { case (k, v) => Integer.valueOf(k) -> v }.asJava
     val topicId = Uuid.randomUuid()
     migrationState = migrationClient.topicClient().createTopic("test", topicId, partitions, migrationState)
@@ -157,7 +206,7 @@ class ZkMigrationClientTest extends ZkMigrationTestHarness {
     // can claim it. This is because KRaft leadership comes from Raft and we are just synchronizing it to ZK.
     var otherNodeState = ZkMigrationLeadershipState.EMPTY
       .withNewKRaftController(3001, 43)
-      .withKRaftMetadataOffsetAndEpoch(100, 42);
+      .withKRaftMetadataOffsetAndEpoch(100, 42)
     otherNodeState = migrationClient.claimControllerLeadership(otherNodeState)
     assertEquals(2, otherNodeState.zkControllerEpochZkVersion())
     assertEquals(3001, otherNodeState.kraftControllerId())
@@ -268,7 +317,7 @@ class ZkMigrationClientTest extends ZkMigrationTestHarness {
 
   @Test
   def testTopicAndBrokerConfigsMigrationWithSnapshots(): Unit = {
-    val kraftWriter = new KRaftMigrationZkWriter(migrationClient)
+    val kraftWriter = new KRaftMigrationZkWriter(migrationClient, fail(_))
 
     // Add add some topics and broker configs and create new image.
     val topicName = "testTopic"
@@ -281,8 +330,8 @@ class ZkMigrationClientTest extends ZkMigrationTestHarness {
     val replicas = List(1, 2, 3).map(int2Integer).asJava
     val topicId = Uuid.randomUuid()
     val props = new Properties()
-    props.put(KafkaConfig.DefaultReplicationFactorProp, "1") // normal config
-    props.put(KafkaConfig.SslKeystorePasswordProp, SECRET) // sensitive config
+    props.put(ReplicationConfigs.DEFAULT_REPLICATION_FACTOR_CONFIG, "1") // normal config
+    props.put(SslConfigs.SSL_KEYSTORE_PASSWORD_CONFIG, SECRET) // sensitive config
 
     //    // Leave Zk in an incomplete state.
     //    zkClient.createTopicAssignment(topicName, Some(topicId), Map(tp -> Seq(1)))
@@ -348,12 +397,12 @@ class ZkMigrationClientTest extends ZkMigrationTestHarness {
     }
 
     // Verify the broker and topic configs (including sensitive configs).
-    val brokerProps = zkClient.getEntityConfigs(ConfigType.Broker, brokerId)
-    val topicProps = zkClient.getEntityConfigs(ConfigType.Topic, topicName)
+    val brokerProps = zkClient.getEntityConfigs(ConfigType.BROKER, brokerId)
+    val topicProps = zkClient.getEntityConfigs(ConfigType.TOPIC, topicName)
     assertEquals(2, brokerProps.size())
 
     brokerProps.asScala.foreach { case (key, value) =>
-      if (key == KafkaConfig.SslKeystorePasswordProp) {
+      if (key == SslConfigs.SSL_KEYSTORE_PASSWORD_CONFIG) {
         assertEquals(SECRET, encoder.decode(value).value)
       } else {
         assertEquals(props.getProperty(key), value)
@@ -361,7 +410,7 @@ class ZkMigrationClientTest extends ZkMigrationTestHarness {
     }
 
     topicProps.asScala.foreach { case (key, value) =>
-      if (key == KafkaConfig.SslKeystorePasswordProp) {
+      if (key == SslConfigs.SSL_KEYSTORE_PASSWORD_CONFIG) {
         assertEquals(SECRET, encoder.decode(value).value)
       } else {
         assertEquals(props.getProperty(key), value)
@@ -375,8 +424,24 @@ class ZkMigrationClientTest extends ZkMigrationTestHarness {
 
     val topicId = Uuid.randomUuid()
     val partitions = Map(
-      0 -> new PartitionRegistration.Builder().setReplicas(Array(0, 1, 2)).setIsr(Array(0, 1, 2)).setLeader(0).setLeaderRecoveryState(LeaderRecoveryState.RECOVERED).setLeaderEpoch(0).setPartitionEpoch(-1).build(),
-      1 -> new PartitionRegistration.Builder().setReplicas(Array(1, 2, 3)).setIsr(Array(1, 2, 3)).setLeader(1).setLeaderRecoveryState(LeaderRecoveryState.RECOVERED).setLeaderEpoch(0).setPartitionEpoch(-1).build()
+      0 -> new PartitionRegistration.Builder()
+        .setReplicas(Array(0, 1, 2))
+        .setDirectories(DirectoryId.unassignedArray(3))
+        .setIsr(Array(0, 1, 2))
+        .setLeader(0)
+        .setLeaderRecoveryState(LeaderRecoveryState.RECOVERED)
+        .setLeaderEpoch(0)
+        .setPartitionEpoch(-1)
+        .build(),
+      1 -> new PartitionRegistration.Builder()
+        .setReplicas(Array(1, 2, 3))
+        .setDirectories(DirectoryId.unassignedArray(3))
+        .setIsr(Array(1, 2, 3))
+        .setLeader(1)
+        .setLeaderRecoveryState(LeaderRecoveryState.RECOVERED)
+        .setLeaderEpoch(0)
+        .setPartitionEpoch(-1)
+        .build()
     ).map { case (k, v) => Integer.valueOf(k) -> v }.asJava
     migrationState = migrationClient.topicClient().createTopic("test", topicId, partitions, migrationState)
     assertEquals(1, migrationState.migrationZkVersion())
@@ -384,8 +449,24 @@ class ZkMigrationClientTest extends ZkMigrationTestHarness {
     // Change assignment in partitions and update the topic assignment. See the change is
     // reflected.
     val changedPartitions = Map(
-      0 -> new PartitionRegistration.Builder().setReplicas(Array(1, 2, 3)).setIsr(Array(1, 2, 3)).setLeader(0).setLeaderRecoveryState(LeaderRecoveryState.RECOVERED).setLeaderEpoch(0).setPartitionEpoch(-1).build(),
-      1 -> new PartitionRegistration.Builder().setReplicas(Array(0, 1, 2)).setIsr(Array(0, 1, 2)).setLeader(1).setLeaderRecoveryState(LeaderRecoveryState.RECOVERED).setLeaderEpoch(0).setPartitionEpoch(-1).build()
+      0 -> new PartitionRegistration.Builder()
+        .setReplicas(Array(1, 2, 3))
+        .setDirectories(DirectoryId.unassignedArray(3))
+        .setIsr(Array(1, 2, 3))
+        .setLeader(0)
+        .setLeaderRecoveryState(LeaderRecoveryState.RECOVERED)
+        .setLeaderEpoch(0)
+        .setPartitionEpoch(-1)
+        .build(),
+      1 -> new PartitionRegistration.Builder()
+        .setReplicas(Array(0, 1, 2))
+        .setDirectories(DirectoryId.unassignedArray(3))
+        .setIsr(Array(0, 1, 2))
+        .setLeader(1)
+        .setLeaderRecoveryState(LeaderRecoveryState.RECOVERED)
+        .setLeaderEpoch(0)
+        .setPartitionEpoch(-1)
+        .build()
     ).map { case (k, v) => Integer.valueOf(k) -> v }.asJava
     migrationState = migrationClient.topicClient().updateTopic("test", topicId, changedPartitions, migrationState)
     assertEquals(2, migrationState.migrationZkVersion())
@@ -393,7 +474,7 @@ class ZkMigrationClientTest extends ZkMigrationTestHarness {
     // Read the changed partition with zkClient.
     val topicReplicaAssignmentFromZk = zkClient.getReplicaAssignmentAndTopicIdForTopics(Set("test"))
     assertEquals(1, topicReplicaAssignmentFromZk.size)
-    assertEquals(Some(topicId), topicReplicaAssignmentFromZk.head.topicId);
+    assertEquals(Some(topicId), topicReplicaAssignmentFromZk.head.topicId)
     topicReplicaAssignmentFromZk.head.assignment.foreach { case (tp, assignment) =>
       tp.partition() match {
         case p if p <=1 =>
@@ -406,7 +487,15 @@ class ZkMigrationClientTest extends ZkMigrationTestHarness {
 
     // Add a new Partition.
     val newPartition = Map(
-      2 -> new PartitionRegistration.Builder().setReplicas(Array(2, 3, 4)).setIsr(Array(2, 3, 4)).setLeader(1).setLeaderRecoveryState(LeaderRecoveryState.RECOVERED).setLeaderEpoch(0).setPartitionEpoch(-1).build()
+      2 -> new PartitionRegistration.Builder()
+        .setReplicas(Array(2, 3, 4))
+        .setDirectories(DirectoryId.unassignedArray(3))
+        .setIsr(Array(2, 3, 4))
+        .setLeader(1)
+        .setLeaderRecoveryState(LeaderRecoveryState.RECOVERED)
+        .setLeaderEpoch(0)
+        .setPartitionEpoch(-1)
+        .build()
     ).map { case (k, v) => int2Integer(k) -> v }.asJava
     migrationState = migrationClient.topicClient().createTopicPartitions(Map("test" -> newPartition).asJava, migrationState)
     assertEquals(3, migrationState.migrationZkVersion())
