@@ -77,7 +77,9 @@ import org.apache.kafka.timeline.SnapshotRegistry;
 import org.slf4j.Logger;
 
 import java.util.ArrayList;
+import java.util.HashSet;
 import java.util.List;
+import java.util.Set;
 import java.util.concurrent.CompletableFuture;
 import java.util.concurrent.TimeUnit;
 
@@ -91,14 +93,14 @@ import java.util.concurrent.TimeUnit;
  * 2) The replay methods which apply records to the hard state. Those are used in the request
  *    handling as well as during the initial loading of the records from the partitions.
  */
-public class GroupCoordinatorShard implements CoordinatorShard<Record> {
+public class GroupCoordinatorShard implements CoordinatorShard<CoordinatorRecord> {
 
-    public static class Builder implements CoordinatorShardBuilder<GroupCoordinatorShard, Record> {
+    public static class Builder implements CoordinatorShardBuilder<GroupCoordinatorShard, CoordinatorRecord> {
         private final GroupCoordinatorConfig config;
         private LogContext logContext;
         private SnapshotRegistry snapshotRegistry;
         private Time time;
-        private CoordinatorTimer<Void, Record> timer;
+        private CoordinatorTimer<Void, CoordinatorRecord> timer;
         private CoordinatorMetrics coordinatorMetrics;
         private TopicPartition topicPartition;
 
@@ -109,7 +111,7 @@ public class GroupCoordinatorShard implements CoordinatorShard<Record> {
         }
 
         @Override
-        public CoordinatorShardBuilder<GroupCoordinatorShard, Record> withLogContext(
+        public CoordinatorShardBuilder<GroupCoordinatorShard, CoordinatorRecord> withLogContext(
             LogContext logContext
         ) {
             this.logContext = logContext;
@@ -117,7 +119,7 @@ public class GroupCoordinatorShard implements CoordinatorShard<Record> {
         }
 
         @Override
-        public CoordinatorShardBuilder<GroupCoordinatorShard, Record> withTime(
+        public CoordinatorShardBuilder<GroupCoordinatorShard, CoordinatorRecord> withTime(
             Time time
         ) {
             this.time = time;
@@ -125,15 +127,15 @@ public class GroupCoordinatorShard implements CoordinatorShard<Record> {
         }
 
         @Override
-        public CoordinatorShardBuilder<GroupCoordinatorShard, Record> withTimer(
-            CoordinatorTimer<Void, Record> timer
+        public CoordinatorShardBuilder<GroupCoordinatorShard, CoordinatorRecord> withTimer(
+            CoordinatorTimer<Void, CoordinatorRecord> timer
         ) {
             this.timer = timer;
             return this;
         }
 
         @Override
-        public CoordinatorShardBuilder<GroupCoordinatorShard, Record> withCoordinatorMetrics(
+        public CoordinatorShardBuilder<GroupCoordinatorShard, CoordinatorRecord> withCoordinatorMetrics(
             CoordinatorMetrics coordinatorMetrics
         ) {
             this.coordinatorMetrics = coordinatorMetrics;
@@ -141,13 +143,13 @@ public class GroupCoordinatorShard implements CoordinatorShard<Record> {
         }
 
         @Override
-        public CoordinatorShardBuilder<GroupCoordinatorShard, Record> withTopicPartition(TopicPartition topicPartition) {
+        public CoordinatorShardBuilder<GroupCoordinatorShard, CoordinatorRecord> withTopicPartition(TopicPartition topicPartition) {
             this.topicPartition = topicPartition;
             return this;
         }
 
         @Override
-        public CoordinatorShardBuilder<GroupCoordinatorShard, Record> withSnapshotRegistry(
+        public CoordinatorShardBuilder<GroupCoordinatorShard, CoordinatorRecord> withSnapshotRegistry(
             SnapshotRegistry snapshotRegistry
         ) {
             this.snapshotRegistry = snapshotRegistry;
@@ -187,6 +189,7 @@ public class GroupCoordinatorShard implements CoordinatorShard<Record> {
                 .withClassicGroupNewMemberJoinTimeoutMs(config.classicGroupNewMemberJoinTimeoutMs)
                 .withClassicGroupMinSessionTimeoutMs(config.classicGroupMinSessionTimeoutMs)
                 .withClassicGroupMaxSessionTimeoutMs(config.classicGroupMaxSessionTimeoutMs)
+                .withConsumerGroupMigrationPolicy(config.consumerGroupMigrationPolicy)
                 .withGroupCoordinatorMetricsShard(metricsShard)
                 .build();
 
@@ -242,7 +245,7 @@ public class GroupCoordinatorShard implements CoordinatorShard<Record> {
     /**
      * The coordinator timer.
      */
-    private final CoordinatorTimer<Void, Record> timer;
+    private final CoordinatorTimer<Void, CoordinatorRecord> timer;
 
     /**
      * The group coordinator config.
@@ -273,7 +276,7 @@ public class GroupCoordinatorShard implements CoordinatorShard<Record> {
         GroupMetadataManager groupMetadataManager,
         OffsetMetadataManager offsetMetadataManager,
         Time time,
-        CoordinatorTimer<Void, Record> timer,
+        CoordinatorTimer<Void, CoordinatorRecord> timer,
         GroupCoordinatorConfig config,
         CoordinatorMetrics coordinatorMetrics,
         CoordinatorMetricsShard metricsShard
@@ -297,7 +300,7 @@ public class GroupCoordinatorShard implements CoordinatorShard<Record> {
      * @return A Result containing the ConsumerGroupHeartbeat response and
      *         a list of records to update the state machine.
      */
-    public CoordinatorResult<ConsumerGroupHeartbeatResponseData, Record> consumerGroupHeartbeat(
+    public CoordinatorResult<ConsumerGroupHeartbeatResponseData, CoordinatorRecord> consumerGroupHeartbeat(
         RequestContext context,
         ConsumerGroupHeartbeatRequestData request
     ) {
@@ -313,7 +316,7 @@ public class GroupCoordinatorShard implements CoordinatorShard<Record> {
      * @return A Result containing the JoinGroup response and
      *         a list of records to update the state machine.
      */
-    public CoordinatorResult<Void, Record> classicGroupJoin(
+    public CoordinatorResult<Void, CoordinatorRecord> classicGroupJoin(
         RequestContext context,
         JoinGroupRequestData request,
         CompletableFuture<JoinGroupResponseData> responseFuture
@@ -334,7 +337,7 @@ public class GroupCoordinatorShard implements CoordinatorShard<Record> {
      * @return A Result containing the SyncGroup response and
      *         a list of records to update the state machine.
      */
-    public CoordinatorResult<Void, Record> classicGroupSync(
+    public CoordinatorResult<Void, CoordinatorRecord> classicGroupSync(
         RequestContext context,
         SyncGroupRequestData request,
         CompletableFuture<SyncGroupResponseData> responseFuture
@@ -352,9 +355,10 @@ public class GroupCoordinatorShard implements CoordinatorShard<Record> {
      * @param context The request context.
      * @param request The actual Heartbeat request.
      *
-     * @return The HeartbeatResponse.
+     * @return A Result containing the heartbeat response and
+     *         a list of records to update the state machine.
      */
-    public HeartbeatResponseData classicGroupHeartbeat(
+    public CoordinatorResult<HeartbeatResponseData, CoordinatorRecord> classicGroupHeartbeat(
         RequestContext context,
         HeartbeatRequestData request
     ) {
@@ -372,13 +376,13 @@ public class GroupCoordinatorShard implements CoordinatorShard<Record> {
      * @return A Result containing the DeleteGroupsResponseData.DeletableGroupResultCollection response and
      *         a list of records to update the state machine.
      */
-    public CoordinatorResult<DeleteGroupsResponseData.DeletableGroupResultCollection, Record> deleteGroups(
+    public CoordinatorResult<DeleteGroupsResponseData.DeletableGroupResultCollection, CoordinatorRecord> deleteGroups(
         RequestContext context,
         List<String> groupIds
     ) throws ApiException {
         final DeleteGroupsResponseData.DeletableGroupResultCollection resultCollection =
             new DeleteGroupsResponseData.DeletableGroupResultCollection(groupIds.size());
-        final List<Record> records = new ArrayList<>();
+        final List<CoordinatorRecord> records = new ArrayList<>();
         int numDeletedOffsets = 0;
         final List<String> deletedGroups = new ArrayList<>();
 
@@ -386,7 +390,7 @@ public class GroupCoordinatorShard implements CoordinatorShard<Record> {
             try {
                 groupMetadataManager.validateDeleteGroup(groupId);
                 numDeletedOffsets += offsetMetadataManager.deleteAllOffsets(groupId, records);
-                groupMetadataManager.deleteGroup(groupId, records);
+                groupMetadataManager.createGroupTombstoneRecords(groupId, records);
                 deletedGroups.add(groupId);
 
                 resultCollection.add(
@@ -450,7 +454,7 @@ public class GroupCoordinatorShard implements CoordinatorShard<Record> {
      * @return A Result containing the OffsetCommitResponse response and
      *         a list of records to update the state machine.
      */
-    public CoordinatorResult<OffsetCommitResponseData, Record> commitOffset(
+    public CoordinatorResult<OffsetCommitResponseData, CoordinatorRecord> commitOffset(
         RequestContext context,
         OffsetCommitRequestData request
     ) throws ApiException {
@@ -466,7 +470,7 @@ public class GroupCoordinatorShard implements CoordinatorShard<Record> {
      * @return A Result containing the TxnOffsetCommitResponse response and
      *         a list of records to update the state machine.
      */
-    public CoordinatorResult<TxnOffsetCommitResponseData, Record> commitTransactionalOffset(
+    public CoordinatorResult<TxnOffsetCommitResponseData, CoordinatorRecord> commitTransactionalOffset(
         RequestContext context,
         TxnOffsetCommitRequestData request
     ) throws ApiException {
@@ -476,16 +480,25 @@ public class GroupCoordinatorShard implements CoordinatorShard<Record> {
     /**
      * Handles a ListGroups request.
      *
-     * @param statesFilter    The states of the groups we want to list.
-     *                        If empty all groups are returned with their state.
-     * @param committedOffset A specified committed offset corresponding to this shard
+     * @param statesFilter      The states of the groups we want to list.
+     *                          If empty, all groups are returned with their state.
+     *                          If invalid, no groups are returned.
+     * @param typesFilter       The types of the groups we want to list.
+     *                          If empty, all groups are returned with their type.
+     *                          If invalid, no groups are returned.
+     * @param committedOffset   A specified committed offset corresponding to this shard.
      * @return A list containing the ListGroupsResponseData.ListedGroup
      */
     public List<ListGroupsResponseData.ListedGroup> listGroups(
         List<String> statesFilter,
+        List<String> typesFilter,
         long committedOffset
     ) throws ApiException {
-        return groupMetadataManager.listGroups(statesFilter, committedOffset);
+
+        Set<String> statesFilterSet = new HashSet<>(statesFilter);
+        Set<String> typesFilterSet = new HashSet<>(typesFilter);
+
+        return groupMetadataManager.listGroups(statesFilterSet, typesFilterSet, committedOffset);
     }
 
     /**
@@ -529,7 +542,7 @@ public class GroupCoordinatorShard implements CoordinatorShard<Record> {
      * @return A Result containing the LeaveGroup response and
      *         a list of records to update the state machine.
      */
-    public CoordinatorResult<LeaveGroupResponseData, Record> classicGroupLeave(
+    public CoordinatorResult<LeaveGroupResponseData, CoordinatorRecord> classicGroupLeave(
         RequestContext context,
         LeaveGroupRequestData request
     ) throws ApiException {
@@ -545,7 +558,7 @@ public class GroupCoordinatorShard implements CoordinatorShard<Record> {
      * @return A Result containing the OffsetDeleteResponse response and
      *         a list of records to update the state machine.
      */
-    public CoordinatorResult<OffsetDeleteResponseData, Record> deleteOffsets(
+    public CoordinatorResult<OffsetDeleteResponseData, CoordinatorRecord> deleteOffsets(
         RequestContext context,
         OffsetDeleteRequestData request
     ) throws ApiException {
@@ -558,9 +571,9 @@ public class GroupCoordinatorShard implements CoordinatorShard<Record> {
      *
      * @return The list of tombstones (offset commit and group metadata) to append.
      */
-    public CoordinatorResult<Void, Record> cleanupGroupMetadata() {
+    public CoordinatorResult<Void, CoordinatorRecord> cleanupGroupMetadata() {
         long startMs = time.milliseconds();
-        List<Record> records = new ArrayList<>();
+        List<CoordinatorRecord> records = new ArrayList<>();
         groupMetadataManager.groupIds().forEach(groupId -> {
             boolean allOffsetsExpired = offsetMetadataManager.cleanupExpiredOffsets(groupId, records);
             if (allOffsetsExpired) {
@@ -589,6 +602,23 @@ public class GroupCoordinatorShard implements CoordinatorShard<Record> {
         );
     }
 
+    /**
+     * Remove offsets of the partitions that have been deleted.
+     *
+     * @param topicPartitions   The partitions that have been deleted.
+     * @return The list of tombstones (offset commit) to append.
+     */
+    public CoordinatorResult<Void, CoordinatorRecord> onPartitionsDeleted(
+        List<TopicPartition> topicPartitions
+    ) {
+        final long startTimeMs = time.milliseconds();
+        final List<CoordinatorRecord> records = offsetMetadataManager.onPartitionsDeleted(topicPartitions);
+
+        log.info("Generated {} tombstone records in {} milliseconds while deleting offsets for partitions {}.",
+            records.size(), time.milliseconds() - startTimeMs, topicPartitions);
+
+        return new CoordinatorResult<>(records);
+    }
 
     /**
      * The coordinator has been loaded. This is used to apply any
@@ -611,6 +641,7 @@ public class GroupCoordinatorShard implements CoordinatorShard<Record> {
     public void onUnloaded() {
         timer.cancel(GROUP_EXPIRATION_KEY);
         coordinatorMetrics.deactivateMetricsShard(metricsShard);
+        groupMetadataManager.onUnloaded();
     }
 
     /**
@@ -639,6 +670,7 @@ public class GroupCoordinatorShard implements CoordinatorShard<Record> {
     /**
      * Replays the Record to update the hard state of the group coordinator.
      *
+     * @param offset        The offset of the record in the log.
      * @param producerId    The producer id.
      * @param producerEpoch The producer epoch.
      * @param record        The record to apply to the state machine.
@@ -646,9 +678,10 @@ public class GroupCoordinatorShard implements CoordinatorShard<Record> {
      */
     @Override
     public void replay(
+        long offset,
         long producerId,
         short producerEpoch,
-        Record record
+        CoordinatorRecord record
     ) throws RuntimeException {
         ApiMessageAndVersion key = record.key();
         ApiMessageAndVersion value = record.value();
@@ -657,6 +690,7 @@ public class GroupCoordinatorShard implements CoordinatorShard<Record> {
             case 0:
             case 1:
                 offsetMetadataManager.replay(
+                    offset,
                     producerId,
                     (OffsetCommitKey) key.message(),
                     (OffsetCommitValue) messageOrNull(value)
