@@ -31,7 +31,6 @@ import kafka.server.ControllerMutationQuotaManager;
 import kafka.server.FetchManager;
 import kafka.server.KafkaApis;
 import kafka.server.KafkaConfig;
-import kafka.server.KafkaConfig$;
 import kafka.server.MetadataCache;
 import kafka.server.QuotaFactory;
 import kafka.server.ReplicaManager;
@@ -60,8 +59,10 @@ import org.apache.kafka.common.security.auth.KafkaPrincipal;
 import org.apache.kafka.common.security.auth.SecurityProtocol;
 import org.apache.kafka.common.utils.Time;
 import org.apache.kafka.coordinator.group.GroupCoordinator;
-import org.apache.kafka.server.common.Features;
+import org.apache.kafka.server.common.FinalizedFeatures;
 import org.apache.kafka.server.common.MetadataVersion;
+import org.apache.kafka.server.config.ServerConfigs;
+import org.apache.kafka.server.config.ZkConfigs;
 import org.mockito.Mockito;
 import org.openjdk.jmh.annotations.Benchmark;
 import org.openjdk.jmh.annotations.BenchmarkMode;
@@ -96,34 +97,33 @@ import java.util.stream.IntStream;
 @OutputTimeUnit(TimeUnit.NANOSECONDS)
 
 public class MetadataRequestBenchmark {
+    private final RequestChannel requestChannel = Mockito.mock(RequestChannel.class, Mockito.withSettings().stubOnly());
+    private final RequestChannel.Metrics requestChannelMetrics = Mockito.mock(RequestChannel.Metrics.class);
+    private final ReplicaManager replicaManager = Mockito.mock(ReplicaManager.class);
+    private final GroupCoordinator groupCoordinator = Mockito.mock(GroupCoordinator.class);
+    private final ZkAdminManager adminManager = Mockito.mock(ZkAdminManager.class);
+    private final TransactionCoordinator transactionCoordinator = Mockito.mock(TransactionCoordinator.class);
+    private final KafkaController kafkaController = Mockito.mock(KafkaController.class);
+    private final AutoTopicCreationManager autoTopicCreationManager = Mockito.mock(AutoTopicCreationManager.class);
+    private final KafkaZkClient kafkaZkClient = Mockito.mock(KafkaZkClient.class);
+    private final Metrics metrics = new Metrics();
+    private final int brokerId = 1;
+    private final ZkMetadataCache metadataCache = MetadataCache.zkMetadataCache(brokerId,
+        MetadataVersion.latestTesting(), BrokerFeatures.createEmpty(), false);
+    private final ClientQuotaManager clientQuotaManager = Mockito.mock(ClientQuotaManager.class);
+    private final ClientRequestQuotaManager clientRequestQuotaManager = Mockito.mock(ClientRequestQuotaManager.class);
+    private final ControllerMutationQuotaManager controllerMutationQuotaManager = Mockito.mock(ControllerMutationQuotaManager.class);
+    private final ReplicationQuotaManager replicaQuotaManager = Mockito.mock(ReplicationQuotaManager.class);
+    private final QuotaFactory.QuotaManagers quotaManagers = new QuotaFactory.QuotaManagers(clientQuotaManager,
+        clientQuotaManager, clientRequestQuotaManager, controllerMutationQuotaManager, replicaQuotaManager,
+        replicaQuotaManager, replicaQuotaManager, Option.empty());
+    private final FetchManager fetchManager = Mockito.mock(FetchManager.class);
+    private final BrokerTopicStats brokerTopicStats = new BrokerTopicStats(false);
+    private final KafkaPrincipal principal = new KafkaPrincipal(KafkaPrincipal.USER_TYPE, "test-user");
     @Param({"500", "1000", "5000"})
     private int topicCount;
     @Param({"10", "20", "50"})
     private int partitionCount;
-
-    private RequestChannel requestChannel = Mockito.mock(RequestChannel.class, Mockito.withSettings().stubOnly());
-    private RequestChannel.Metrics requestChannelMetrics = Mockito.mock(RequestChannel.Metrics.class);
-    private ReplicaManager replicaManager = Mockito.mock(ReplicaManager.class);
-    private GroupCoordinator groupCoordinator = Mockito.mock(GroupCoordinator.class);
-    private ZkAdminManager adminManager = Mockito.mock(ZkAdminManager.class);
-    private TransactionCoordinator transactionCoordinator = Mockito.mock(TransactionCoordinator.class);
-    private KafkaController kafkaController = Mockito.mock(KafkaController.class);
-    private AutoTopicCreationManager autoTopicCreationManager = Mockito.mock(AutoTopicCreationManager.class);
-    private KafkaZkClient kafkaZkClient = Mockito.mock(KafkaZkClient.class);
-    private Metrics metrics = new Metrics();
-    private int brokerId = 1;
-    private ZkMetadataCache metadataCache = MetadataCache.zkMetadataCache(brokerId,
-        MetadataVersion.latestTesting(), BrokerFeatures.createEmpty(), null, false);
-    private ClientQuotaManager clientQuotaManager = Mockito.mock(ClientQuotaManager.class);
-    private ClientRequestQuotaManager clientRequestQuotaManager = Mockito.mock(ClientRequestQuotaManager.class);
-    private ControllerMutationQuotaManager controllerMutationQuotaManager = Mockito.mock(ControllerMutationQuotaManager.class);
-    private ReplicationQuotaManager replicaQuotaManager = Mockito.mock(ReplicationQuotaManager.class);
-    private QuotaFactory.QuotaManagers quotaManagers = new QuotaFactory.QuotaManagers(clientQuotaManager,
-        clientQuotaManager, clientRequestQuotaManager, controllerMutationQuotaManager, replicaQuotaManager,
-        replicaQuotaManager, replicaQuotaManager, Option.empty());
-    private FetchManager fetchManager = Mockito.mock(FetchManager.class);
-    private BrokerTopicStats brokerTopicStats = new BrokerTopicStats(Optional.empty());
-    private KafkaPrincipal principal = new KafkaPrincipal(KafkaPrincipal.USER_TYPE, "test-user");
     private KafkaApis kafkaApis;
     private RequestChannel.Request allTopicMetadataRequest;
 
@@ -177,8 +177,8 @@ public class MetadataRequestBenchmark {
 
     private KafkaApis createKafkaApis() {
         Properties kafkaProps =  new Properties();
-        kafkaProps.put(KafkaConfig$.MODULE$.ZkConnectProp(), "zk");
-        kafkaProps.put(KafkaConfig$.MODULE$.BrokerIdProp(), brokerId + "");
+        kafkaProps.put(ZkConfigs.ZK_CONNECT_CONFIG, "zk");
+        kafkaProps.put(ServerConfigs.BROKER_ID_CONFIG, brokerId + "");
         KafkaConfig config = new KafkaConfig(kafkaProps);
         return new KafkaApisBuilder().
             setRequestChannel(requestChannel).
@@ -204,7 +204,7 @@ public class MetadataRequestBenchmark {
                     ApiMessageType.ListenerType.ZK_BROKER,
                     false,
                     false,
-                    () -> Features.fromKRaftVersion(MetadataVersion.latestTesting()))).
+                    () -> FinalizedFeatures.fromKRaftVersion(MetadataVersion.latestTesting()))).
             build();
     }
 
