@@ -36,15 +36,11 @@ import org.apache.kafka.server.authorizer.AuthorizationResult;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
-import java.util.ArrayList;
 import java.util.Collections;
 import java.util.EnumSet;
-import java.util.HashMap;
 import java.util.Iterator;
-import java.util.List;
 import java.util.NavigableSet;
 import java.util.Set;
-import java.util.TreeSet;
 
 import static org.apache.kafka.common.acl.AclOperation.ALL;
 import static org.apache.kafka.common.acl.AclOperation.ALTER;
@@ -105,17 +101,14 @@ public class StandardAuthorizerData {
     /**
      * The result to return if no ACLs match.
      */
-    private final DefaultRule defaultRule;
+    private final DefaultRule noAclRule;
 
     /**
-     * Contains all of the current ACLs sorted by (resource type, resource name).
+     * Contains all of the current ACLs
      */
-    private final TreeSet<StandardAcl> aclsByResource;
+    private AclCache aclCache;
+
 
-    /**
-     * Contains all of the current ACLs indexed by UUID.
-     */
-    private final HashMap<Uuid, StandardAcl> aclsById;
 
     private static Logger createLogger(int nodeId) {
         return new LogContext("[StandardAuthorizer " + nodeId + "] ").logger(StandardAuthorizerData.class);
@@ -131,7 +124,7 @@ public class StandardAuthorizerData {
             false,
             Collections.emptySet(),
             DENIED,
-            new TreeSet<>(), new HashMap<>());
+            new AclCache());
     }
 
     private StandardAuthorizerData(Logger log,
@@ -139,16 +132,14 @@ public class StandardAuthorizerData {
                                    boolean loadingComplete,
                                    Set<String> superUsers,
                                    AuthorizationResult defaultResult,
-                                   TreeSet<StandardAcl> aclsByResource,
-                                   HashMap<Uuid, StandardAcl> aclsById) {
+                                   AclCache aclCache) {
         this.log = log;
         this.auditLog = auditLogger();
         this.aclMutator = aclMutator;
         this.loadingComplete = loadingComplete;
         this.superUsers = superUsers;
-        this.defaultRule = new DefaultRule(defaultResult);
-        this.aclsByResource = aclsByResource;
-        this.aclsById = aclsById;
+        this.noAclRule = new DefaultRule(defaultResult);
+        this.aclCache = aclCache;
     }
 
     StandardAuthorizerData copyWithNewAclMutator(AclMutator newAclMutator) {
@@ -157,9 +148,8 @@ public class StandardAuthorizerData {
             newAclMutator,
             loadingComplete,
             superUsers,
-            defaultRule.result,
-            aclsByResource,
-            aclsById);
+            noAclRule.result,
+            aclCache);
     }
 
     StandardAuthorizerData copyWithNewLoadingComplete(boolean newLoadingComplete) {
@@ -167,9 +157,8 @@ public class StandardAuthorizerData {
             aclMutator,
             newLoadingComplete,
             superUsers,
-            defaultRule.result,
-            aclsByResource,
-            aclsById);
+            noAclRule.result,
+            aclCache);
     }
 
     StandardAuthorizerData copyWithNewConfig(int nodeId,
@@ -181,35 +170,24 @@ public class StandardAuthorizerData {
             loadingComplete,
             newSuperUsers,
             newDefaultResult,
-            aclsByResource,
-            aclsById);
+            aclCache);
     }
 
-    StandardAuthorizerData copyWithNewAcls(TreeSet<StandardAcl> aclsByResource, HashMap<Uuid,
-        StandardAcl> aclsById) {
+    StandardAuthorizerData copyWithNewAcls(AclCache aclCache) {
         StandardAuthorizerData newData =  new StandardAuthorizerData(
             log,
             aclMutator,
             loadingComplete,
             superUsers,
-            defaultRule.result,
-            aclsByResource,
-            aclsById);
-        log.info("Initialized with {} acl(s).", aclsById.size());
+            noAclRule.result,
+            aclCache);
+        log.info("Initialized with {} acl(s).", aclCache.count());
         return newData;
     }
 
     void addAcl(Uuid id, StandardAcl acl) {
         try {
-            StandardAcl prevAcl = aclsById.putIfAbsent(id, acl);
-            if (prevAcl != null) {
-                throw new RuntimeException("An ACL with ID " + id + " already exists.");
-            }
-            if (!aclsByResource.add(acl)) {
-                aclsById.remove(id);
-                throw new RuntimeException("Unable to add the ACL with ID " + id +
-                    " to aclsByResource");
-            }
+            aclCache = aclCache.addAcl(id, acl);
             log.trace("Added ACL {}: {}", id, acl);
         } catch (Throwable e) {
             log.error("addAcl error", e);
@@ -219,15 +197,9 @@ public class StandardAuthorizerData {
 
     void removeAcl(Uuid id) {
         try {
-            StandardAcl acl = aclsById.remove(id);
-            if (acl == null) {
-                throw new RuntimeException("ID " + id + " not found in aclsById.");
-            }
-            if (!aclsByResource.remove(acl)) {
-                throw new RuntimeException("Unable to remove the ACL with ID " + id +
-                    " from aclsByResource");
-            }
-            log.trace("Removed ACL {}: {}", id, acl);
+            AclCache aclCacheSnapshot = aclCache.removeAcl(id);
+            log.trace("Removed ACL {}: {}", id, aclCacheSnapshot.getAcl(id));
+            aclCache = aclCacheSnapshot;
         } catch (Throwable e) {
             log.error("removeAcl error", e);
             throw e;
@@ -239,11 +211,11 @@ public class StandardAuthorizerData {
     }
 
     AuthorizationResult defaultResult() {
-        return defaultRule.result;
+        return noAclRule.result;
     }
 
     int aclCount() {
-        return aclsById.size();
+        return aclCache.count();
     }
 
     /**
@@ -259,6 +231,9 @@ public class StandardAuthorizerData {
         AuthorizableRequestContext requestContext,
         Action action
     ) {
+        if (action.resourcePattern().patternType() != LITERAL) {
+            throw new IllegalArgumentException("Only literal resources are supported. Got: " + action.resourcePattern().patternType());
+        }
         KafkaPrincipal principal = baseKafkaPrincipal(requestContext);
         final MatchingRule rule;
 
@@ -268,20 +243,12 @@ public class StandardAuthorizerData {
         } else if (!loadingComplete) {
             throw new AuthorizerNotReadyException();
         } else {
-            MatchingAclRule aclRule = findAclRule(
+            rule = findAclRule(
                 matchingPrincipals(requestContext),
                 requestContext.clientAddress().getHostAddress(),
                 action
             );
-
-            if (aclRule != null) {
-                rule = aclRule;
-            } else {
-                // If nothing matched, we return the default result.
-                rule = defaultRule;
-            }
         }
-
         logAuditMessage(principal, requestContext, action, rule);
         return rule.result();
     }
@@ -344,7 +311,7 @@ public class StandardAuthorizerData {
         }
     }
 
-    private MatchingAclRule findAclRule(
+    private MatchingRule findAclRule(
         Set<KafkaPrincipal> matchingPrincipals,
         String host,
         Action action
@@ -352,8 +319,10 @@ public class StandardAuthorizerData {
         // This code relies on the ordering of StandardAcl within the NavigableMap.
         // Entries are sorted by resource type first, then REVERSE resource name.
         // Therefore, we can find all the applicable ACLs by starting at
-        // (resource_type, resource_name) and stepping forwards until we reach an ACL with
-        // a resource name which is not a prefix of the current one.
+        // (resource_type, resource_name) and stepping forwards until we reach
+        // an ACL with a resource name which is not a prefix of the current one.
+        // At that point, we need to search for if there are any more ACLs at
+        // the first divergence point.
         //
         // For example, when trying to authorize a TOPIC resource named foobar, we would
         // start at element 2, and continue on to 3 and 4 following map:
@@ -362,10 +331,13 @@ public class StandardAuthorizerData {
         // 2. rs=TOPIC rn=foobar pt=PREFIX
         // 3. rs=TOPIC rn=foob pt=LITERAL
         // 4. rs=TOPIC rn=foo pt=PREFIX
-        // 5. rs=TOPIC rn=eeee pt=LITERAL
+        // 5. rs=TOPIC rn=fb pt=PREFIX
+        // 6. rs=TOPIC rn=fa pt=PREFIX
+        // 7. rs=TOPIC rn=f  pt=PREFIX
+        // 8. rs=TOPIC rn=eeee pt=LITERAL
         //
-        // Once we reached element 5, we would stop scanning.
-        MatchingAclBuilder matchingAclBuilder = new MatchingAclBuilder();
+        // Once we reached element 5, we would jump to element 7.
+        MatchingRuleBuilder matchingRuleBuilder = new MatchingRuleBuilder(noAclRule);
         StandardAcl exemplar = new StandardAcl(
             action.resourcePattern().resourceType(),
             action.resourcePattern().name(),
@@ -374,9 +346,10 @@ public class StandardAuthorizerData {
             "",
             AclOperation.UNKNOWN,
             AclPermissionType.UNKNOWN);
-        checkSection(action, exemplar, matchingPrincipals, host, matchingAclBuilder);
-        if (matchingAclBuilder.foundDeny()) {
-            return matchingAclBuilder.build();
+        AclCache aclCacheSnapshot = aclCache;
+        checkSection(aclCacheSnapshot, action, exemplar, matchingPrincipals, host, matchingRuleBuilder);
+        if (matchingRuleBuilder.foundDeny()) {
+            return matchingRuleBuilder.build();
         }
 
         // In addition to ACLs for this specific resource name, there can also be wildcard
@@ -390,45 +363,72 @@ public class StandardAuthorizerData {
             "",
             AclOperation.UNKNOWN,
             AclPermissionType.UNKNOWN);
-        checkSection(action, exemplar, matchingPrincipals, host, matchingAclBuilder);
-        return matchingAclBuilder.build();
+        checkSection(aclCacheSnapshot, action, exemplar, matchingPrincipals, host, matchingRuleBuilder);
+        return matchingRuleBuilder.build();
+    }
+
+    static int matchesUpTo(
+        String resource,
+        String pattern
+    ) {
+        int i = 0;
+        while (true) {
+            if (resource.length() == i) break;
+            if (pattern.length() == i) break;
+            if (resource.charAt(i) != pattern.charAt(i)) break;
+            i++;
+        }
+        return i;
     }
 
     private void checkSection(
-        Action action,
-        StandardAcl exemplar,
-        Set<KafkaPrincipal> matchingPrincipals,
-        String host,
-        MatchingAclBuilder matchingAclBuilder
+            AclCache aclCacheSnapshot, Action action,
+            StandardAcl exemplar,
+            Set<KafkaPrincipal> matchingPrincipals,
+            String host,
+            MatchingRuleBuilder matchingRuleBuilder
     ) {
-        NavigableSet<StandardAcl> tailSet = aclsByResource.tailSet(exemplar, true);
         String resourceName = action.resourcePattern().name();
-        for (Iterator<StandardAcl> iterator = tailSet.iterator();
-             iterator.hasNext(); ) {
+        NavigableSet<StandardAcl> tailSet = aclCacheSnapshot.aclsByResource().tailSet(exemplar, true);
+        Iterator<StandardAcl> iterator = tailSet.iterator();
+        while (iterator.hasNext()) {
             StandardAcl acl = iterator.next();
             if (!acl.resourceType().equals(action.resourcePattern().resourceType())) {
                 // We've stepped outside the section for the resource type we care about and
                 // should stop scanning.
                 break;
             }
-            if (resourceName.startsWith(acl.resourceName())) {
-                if (acl.patternType() == LITERAL && !resourceName.equals(acl.resourceName())) {
+            int matchesUpTo = matchesUpTo(resourceName, acl.resourceName());
+            if (matchesUpTo == acl.resourceName().length()) {
+                if (acl.patternType() == LITERAL && matchesUpTo != resourceName.length()) {
                     // This is a literal ACL whose name is a prefix of the resource name, but
                     // which doesn't match it exactly. We should skip over this ACL, but keep
                     // scanning in case there are any relevant PREFIX ACLs.
                     continue;
                 }
+
             } else if (!(acl.resourceName().equals(WILDCARD) && acl.patternType() == LITERAL)) {
                 // If the ACL resource name is NOT a prefix of the current resource name,
                 // and we're not dealing with the special case of a wildcard ACL, we've
-                // stepped outside of the section we care about and should stop scanning.
-                break;
+                // stepped outside of the section we care about. Scan for any other potential
+                // prefix matches.
+                exemplar = new StandardAcl(exemplar.resourceType(),
+                    exemplar.resourceName().substring(0, matchesUpTo),
+                    exemplar.patternType(),
+                    exemplar.principal(),
+                    exemplar.host(),
+                    exemplar.operation(),
+                    exemplar.permissionType());
+                tailSet = aclCacheSnapshot.aclsByResource().tailSet(exemplar, true);
+                iterator = tailSet.iterator();
+                continue;
             }
+            matchingRuleBuilder.hasResourceAcls = true;
             AuthorizationResult result = findResult(action, matchingPrincipals, host, acl);
             if (ALLOWED == result) {
-                matchingAclBuilder.allowAcl = acl;
+                matchingRuleBuilder.allowAcl = acl;
             } else if (DENIED == result) {
-                matchingAclBuilder.denyAcl = acl;
+                matchingRuleBuilder.denyAcl = acl;
                 return;
             }
         }
@@ -498,7 +498,7 @@ public class StandardAuthorizerData {
         }
         // Check if the operation field matches. Here we hit a slight complication.
         // ACLs for various operations (READ, WRITE, DELETE, ALTER), "imply" the presence
-        // of DESCRIBE, even if it isn't explictly stated. A similar rule applies to
+        // of DESCRIBE, even if it isn't explicitly stated. A similar rule applies to
         // DESCRIBE_CONFIGS.
         //
         // But this rule only applies to ALLOW ACLs. So for example, a DENY ACL for READ
@@ -533,14 +533,7 @@ public class StandardAuthorizerData {
      * @return Iterable over AclBindings matching the filter.
      */
     Iterable<AclBinding> acls(AclBindingFilter filter) {
-        List<AclBinding> aclBindingList = new ArrayList<>();
-        aclsByResource.forEach(acl -> {
-            AclBinding aclBinding = acl.toBinding();
-            if (filter.matches(aclBinding)) {
-                aclBindingList.add(aclBinding);
-            }
-        });
-        return aclBindingList;
+        return aclCache.acls(filter);
     }
 
     private interface MatchingRule {
@@ -599,30 +592,35 @@ public class StandardAuthorizerData {
         }
     }
 
-    private static class MatchingAclBuilder {
+    private static class MatchingRuleBuilder {
+        private static final DefaultRule DENY_RULE = new DefaultRule(DENIED);
+        private final DefaultRule noAclRule;
         private StandardAcl denyAcl;
         private StandardAcl allowAcl;
+        private boolean hasResourceAcls;
+
+        public MatchingRuleBuilder(DefaultRule noAclRule) {
+            this.noAclRule = noAclRule;
+        }
 
         boolean foundDeny() {
             return denyAcl != null;
         }
 
-        MatchingAclRule build() {
+        MatchingRule build() {
             if (denyAcl != null) {
                 return new MatchingAclRule(denyAcl, DENIED);
             } else if (allowAcl != null) {
                 return new MatchingAclRule(allowAcl, ALLOWED);
+            } else if (!hasResourceAcls) {
+                return noAclRule;
             } else {
-                return null;
+                return DENY_RULE;
             }
         }
     }
 
-    TreeSet<StandardAcl> getAclsByResource() {
-        return aclsByResource;
-    }
-
-    HashMap<Uuid, StandardAcl> getAclsById() {
-        return aclsById;
+    AclCache getAclCache() {
+        return aclCache;
     }
 }
