@@ -54,14 +54,28 @@ public class ActivationRecordsGeneratorTest {
         result = ActivationRecordsGenerator.recordsForEmptyLog(
             logMsg -> assertEquals("Performing controller activation. The metadata log appears to be empty. " +
                 "Appending 1 bootstrap record(s) at metadata.version 3.4-IV0 from bootstrap " +
-                "source 'test'.", logMsg),
+                "source 'test'. Setting the ZK migration state to NONE since this is a de-novo KRaft cluster.", logMsg),
             -1L,
             false,
             BootstrapMetadata.fromVersion(MetadataVersion.IBP_3_4_IV0, "test"),
             MetadataVersion.IBP_3_4_IV0
         );
         assertTrue(result.isAtomic());
-        assertEquals(1, result.records().size());
+        assertEquals(2, result.records().size());
+
+
+        result = ActivationRecordsGenerator.recordsForEmptyLog(
+            logMsg -> assertEquals("Performing controller activation. The metadata log appears to be empty. " +
+                "Appending 1 bootstrap record(s) at metadata.version 3.4-IV0 from bootstrap " +
+                "source 'test'. Putting the controller into pre-migration mode. No metadata updates will be allowed " +
+                "until the ZK metadata has been migrated.", logMsg),
+            -1L,
+            true,
+            BootstrapMetadata.fromVersion(MetadataVersion.IBP_3_4_IV0, "test"),
+            MetadataVersion.IBP_3_4_IV0
+        );
+        assertTrue(result.isAtomic());
+        assertEquals(2, result.records().size());
 
         assertEquals(
             "The bootstrap metadata.version 3.3-IV2 does not support ZK migrations. Cannot continue with ZK migrations enabled.",
@@ -75,18 +89,6 @@ public class ActivationRecordsGeneratorTest {
                 )).getMessage()
         );
 
-        assertEquals(
-            "The bootstrap metadata.version 3.4-IV0 does not support ZK migrations. Cannot continue with ZK migrations enabled.",
-            assertThrows(RuntimeException.class, () ->
-                ActivationRecordsGenerator.recordsForEmptyLog(
-                    logMsg -> fail(),
-                    -1L,
-                    true,
-                    BootstrapMetadata.fromVersion(MetadataVersion.IBP_3_4_IV0, "test"),
-                    MetadataVersion.IBP_3_4_IV0
-                )).getMessage()
-        );
-
         result = ActivationRecordsGenerator.recordsForEmptyLog(
             logMsg -> assertEquals("Performing controller activation. The metadata log appears to be empty. " +
                 "Appending 1 bootstrap record(s) in metadata transaction at metadata.version 3.6-IV1 from bootstrap " +
@@ -241,28 +243,28 @@ public class ActivationRecordsGeneratorTest {
 
         assertEquals(
             "Should not have ZK migrations enabled on a cluster that was created in KRaft mode.",
-            assertThrows(RuntimeException.class, () ->
+            assertThrows(RuntimeException.class, () -> {
                 ActivationRecordsGenerator.recordsForNonEmptyLog(
                     logMsg -> fail(),
                     -1L,
                     true,
                     buildFeatureControl(MetadataVersion.IBP_3_4_IV0, Optional.empty()),
                     MetadataVersion.IBP_3_4_IV0
-                )).getMessage()
+                );
+            }).getMessage()
         );
 
-        assertEquals(
-            "Should not have ZK migrations enabled on a cluster that was created in KRaft mode.",
-            assertThrows(RuntimeException.class, () ->
-                ActivationRecordsGenerator.recordsForNonEmptyLog(
-                    logMsg -> fail(),
-                    -1L,
-                    true,
-                    buildFeatureControl(MetadataVersion.IBP_3_6_IV1, Optional.empty()),
-                    MetadataVersion.IBP_3_6_IV1
-                )
-            ).getMessage()
+        result = ActivationRecordsGenerator.recordsForNonEmptyLog(
+            logMsg -> assertEquals("Performing controller activation. Loaded ZK migration state of " +
+                "PRE_MIGRATION. Activating pre-migration controller without empty log. There may be a partial " +
+                "migration.", logMsg),
+            -1L,
+            true,
+            buildFeatureControl(MetadataVersion.IBP_3_4_IV0, Optional.of(ZkMigrationState.PRE_MIGRATION)),
+            MetadataVersion.IBP_3_4_IV0
         );
+        assertTrue(result.isAtomic());
+        assertEquals(0, result.records().size());
 
         result = ActivationRecordsGenerator.recordsForNonEmptyLog(
             logMsg -> assertEquals("Performing controller activation. Loaded ZK migration state of " +
@@ -280,8 +282,8 @@ public class ActivationRecordsGeneratorTest {
                 "Staying in ZK migration mode since 'zookeeper.metadata.migration.enable' is still 'true'.", logMsg),
             -1L,
             true,
-            buildFeatureControl(MetadataVersion.IBP_3_6_IV1, Optional.of(ZkMigrationState.MIGRATION)),
-            MetadataVersion.IBP_3_6_IV1
+            buildFeatureControl(MetadataVersion.IBP_3_4_IV0, Optional.of(ZkMigrationState.MIGRATION)),
+            MetadataVersion.IBP_3_4_IV0
         );
         assertTrue(result.isAtomic());
         assertEquals(0, result.records().size());
