@@ -282,11 +282,11 @@ public class PartitionRegistrationTest {
     }
 
     private static Stream<Arguments> metadataVersionsForTestPartitionRegistration() {
-        return Arrays.asList(
+        return Stream.of(
             MetadataVersion.IBP_3_7_IV1,
             MetadataVersion.IBP_3_7_IV2,
-            MetadataVersion.IBP_3_8_IV0
-        ).stream().map(mv -> Arguments.of(mv));
+            MetadataVersion.IBP_3_9_IV0
+        ).map(mv -> Arguments.of(mv));
     }
 
     @ParameterizedTest
@@ -322,7 +322,7 @@ public class PartitionRegistrationTest {
         if (metadataVersion.isElrSupported()) {
             expectRecord.
                 setEligibleLeaderReplicas(Arrays.asList(2, 3)).
-                setLastKnownELR(Arrays.asList(4));
+                setLastKnownElr(Arrays.asList(4));
         }
         if (metadataVersion.isDirectoryAssignmentSupported()) {
             expectRecord.setDirectories(Arrays.asList(
@@ -371,7 +371,7 @@ public class PartitionRegistrationTest {
             setPartitionEpoch(0);
         List<UnwritableMetadataException> exceptions = new ArrayList<>();
         ImageWriterOptions options = new ImageWriterOptions.Builder().
-            setMetadataVersion(MetadataVersion.IBP_3_8_IV0).
+            setMetadataVersion(MetadataVersion.IBP_3_9_IV0).
             setLossHandler(exceptions::add).
             build();
         assertEquals(new ApiMessageAndVersion(expectRecord, (short) 2), partitionRegistration.toRecord(topicID, 0, options));
