@@ -17,9 +17,7 @@
 
 package org.apache.kafka.metalog;
 
-import org.apache.kafka.common.memory.MemoryPool;
 import org.apache.kafka.common.protocol.ObjectSerializationCache;
-import org.apache.kafka.common.record.CompressionType;
 import org.apache.kafka.common.utils.BufferSupplier;
 import org.apache.kafka.common.utils.LogContext;
 import org.apache.kafka.common.utils.MockTime;
@@ -484,11 +482,6 @@ public final class LocalLogManager implements RaftClient<ApiMessageAndVersion>,
      */
     private final EventQueue eventQueue;
 
-    /**
-     * Whether this LocalLogManager has been initialized.
-     */
-    private boolean initialized = false;
-
     /**
      * Whether this LocalLogManager has been shut down.
      */
@@ -633,7 +626,7 @@ public final class LocalLogManager implements RaftClient<ApiMessageAndVersion>,
         @Override
         public void run() throws Exception {
             try {
-                if (initialized && !shutdown) {
+                if (!shutdown) {
                     log.debug("Node {}: beginning shutdown.", nodeId);
                     resign(leader.epoch());
                     for (MetaLogListenerData listenerData : listeners.values()) {
@@ -679,14 +672,6 @@ public final class LocalLogManager implements RaftClient<ApiMessageAndVersion>,
         return shutdownFuture;
     }
 
-    @Override
-    public void initialize() {
-        eventQueue.append(() -> {
-            log.debug("initialized local log manager for node " + nodeId);
-            initialized = true;
-        });
-    }
-
     @Override
     public void register(RaftClient.Listener<ApiMessageAndVersion> listener) {
         CompletableFuture<Void> future = new CompletableFuture<>();
@@ -695,7 +680,7 @@ public final class LocalLogManager implements RaftClient<ApiMessageAndVersion>,
                 log.info("Node {}: can't register because local log manager has " +
                     "already been shut down.", nodeId);
                 future.complete(null);
-            } else if (initialized) {
+            } else {
                 int id = System.identityHashCode(listener);
                 if (listeners.putIfAbsent(listener, new MetaLogListenerData(listener)) != null) {
                     log.error("Node {}: can't register because listener {} already exists", nodeId, id);
@@ -705,11 +690,6 @@ public final class LocalLogManager implements RaftClient<ApiMessageAndVersion>,
                 shared.electLeaderIfNeeded();
                 scheduleLogCheck();
                 future.complete(null);
-            } else {
-                log.info("Node {}: can't register because local log manager has not " +
-                    "been initialized.", nodeId);
-                future.completeExceptionally(new RuntimeException(
-                    "LocalLogManager was not initialized."));
             }
         });
         try {
@@ -835,22 +815,19 @@ public final class LocalLogManager implements RaftClient<ApiMessageAndVersion>,
         OffsetAndEpoch snapshotId,
         long lastContainedLogTimestamp
     ) {
-        return RecordsSnapshotWriter.createWithHeader(
-            () -> createNewSnapshot(snapshotId),
-            1024,
-            MemoryPool.NONE,
-            new MockTime(),
-            lastContainedLogTimestamp,
-            CompressionType.NONE,
-            new MetadataRecordSerde()
+        return Optional.of(
+            new RecordsSnapshotWriter.Builder()
+                .setLastContainedLogTimestamp(lastContainedLogTimestamp)
+                .setTime(new MockTime())
+                .setRawSnapshotWriter(createNewSnapshot(snapshotId))
+                .build(new MetadataRecordSerde())
         );
     }
 
-    private Optional<RawSnapshotWriter> createNewSnapshot(OffsetAndEpoch snapshotId) {
-        return Optional.of(
-            new MockRawSnapshotWriter(snapshotId, buffer -> {
-                shared.addSnapshot(new MockRawSnapshotReader(snapshotId, buffer));
-            })
+    private RawSnapshotWriter createNewSnapshot(OffsetAndEpoch snapshotId) {
+        return new MockRawSnapshotWriter(
+            snapshotId,
+            buffer -> shared.addSnapshot(new MockRawSnapshotReader(snapshotId, buffer))
         );
     }
 
