@@ -25,25 +25,34 @@ import java.util.IdentityHashMap;
 import java.util.List;
 import java.util.NoSuchElementException;
 import java.util.Objects;
+import java.util.Optional;
 import java.util.Random;
 import java.util.Set;
 import java.util.concurrent.atomic.AtomicReference;
 import java.util.stream.Collectors;
+import java.util.stream.IntStream;
 import java.util.stream.Stream;
 import net.jqwik.api.ForAll;
 import net.jqwik.api.Property;
+import org.apache.kafka.common.compress.Compression;
 import org.apache.kafka.common.errors.CorruptRecordException;
-import org.apache.kafka.common.memory.MemoryPool;
+import org.apache.kafka.common.message.KRaftVersionRecord;
 import org.apache.kafka.common.message.LeaderChangeMessage;
 import org.apache.kafka.common.message.SnapshotFooterRecord;
 import org.apache.kafka.common.message.SnapshotHeaderRecord;
+import org.apache.kafka.common.message.VotersRecord;
 import org.apache.kafka.common.protocol.ApiMessage;
+import org.apache.kafka.common.protocol.MessageUtil;
 import org.apache.kafka.common.record.CompressionType;
 import org.apache.kafka.common.record.ControlRecordType;
+import org.apache.kafka.common.record.ControlRecordUtils;
 import org.apache.kafka.common.record.DefaultRecordBatch;
 import org.apache.kafka.common.record.FileRecords;
 import org.apache.kafka.common.record.MemoryRecords;
+import org.apache.kafka.common.record.MemoryRecordsBuilder;
+import org.apache.kafka.common.record.RecordBatch;
 import org.apache.kafka.common.record.Records;
+import org.apache.kafka.common.record.TimestampType;
 import org.apache.kafka.common.utils.BufferSupplier;
 import org.apache.kafka.common.utils.MockTime;
 import org.apache.kafka.raft.Batch;
@@ -145,18 +154,16 @@ public final class RecordsIteratorTest {
     }
 
     @Test
-    public void testControlRecordIteration() {
+    public void testControlRecordIterationWithKraftVersion0() {
         AtomicReference<ByteBuffer> buffer = new AtomicReference<>(null);
-        try (RecordsSnapshotWriter<String> snapshot = RecordsSnapshotWriter.createWithHeader(
-                new MockRawSnapshotWriter(new OffsetAndEpoch(100, 10), snapshotBuf -> buffer.set(snapshotBuf)),
-                4 * 1024,
-                MemoryPool.NONE,
-                new MockTime(),
-                0,
-                CompressionType.NONE,
-                STRING_SERDE
-            )
-        ) {
+        RecordsSnapshotWriter.Builder builder = new RecordsSnapshotWriter.Builder()
+            .setTime(new MockTime())
+            .setKraftVersion((short) 0)
+            .setVoterSet(Optional.empty())
+            .setRawSnapshotWriter(
+                new MockRawSnapshotWriter(new OffsetAndEpoch(100, 10), snapshotBuf -> buffer.set(snapshotBuf))
+            );
+        try (RecordsSnapshotWriter<String> snapshot = builder.build(STRING_SERDE)) {
             snapshot.append(Arrays.asList("a", "b", "c"));
             snapshot.append(Arrays.asList("d", "e", "f"));
             snapshot.append(Arrays.asList("g", "h", "i"));
@@ -169,10 +176,11 @@ public final class RecordsIteratorTest {
                 true
             )
         ) {
-            // Check snapshot header control record
+            // Consume the control record batch
             Batch<String> batch = iterator.next();
-
             assertEquals(1, batch.controlRecords().size());
+
+            // Check snapshot header control record
             assertEquals(ControlRecordType.SNAPSHOT_HEADER, batch.controlRecords().get(0).type());
             assertEquals(new SnapshotHeaderRecord(), batch.controlRecords().get(0).message());
 
@@ -192,24 +200,72 @@ public final class RecordsIteratorTest {
         }
     }
 
+    @Test
+    public void testControlRecordIterationWithKraftVersion1() {
+        AtomicReference<ByteBuffer> buffer = new AtomicReference<>(null);
+        VoterSet voterSet = new VoterSet(
+            VoterSetTest.voterMap(IntStream.of(1, 2, 3), true)
+        );
+        RecordsSnapshotWriter.Builder builder = new RecordsSnapshotWriter.Builder()
+            .setTime(new MockTime())
+            .setKraftVersion((short) 1)
+            .setVoterSet(Optional.of(voterSet))
+            .setRawSnapshotWriter(
+                new MockRawSnapshotWriter(new OffsetAndEpoch(100, 10), snapshotBuf -> buffer.set(snapshotBuf))
+            );
+        try (RecordsSnapshotWriter<String> snapshot = builder.build(STRING_SERDE)) {
+            snapshot.append(Arrays.asList("a", "b", "c"));
+            snapshot.append(Arrays.asList("d", "e", "f"));
+            snapshot.append(Arrays.asList("g", "h", "i"));
+            snapshot.freeze();
+        }
+
+        try (RecordsIterator<String> iterator = createIterator(
+                MemoryRecords.readableRecords(buffer.get()),
+                BufferSupplier.NO_CACHING,
+                true
+            )
+        ) {
+            // Consume the control record batch
+            Batch<String> batch = iterator.next();
+            assertEquals(3, batch.controlRecords().size());
+
+            // Check snapshot header control record
+            assertEquals(ControlRecordType.SNAPSHOT_HEADER, batch.controlRecords().get(0).type());
+            assertEquals(new SnapshotHeaderRecord(), batch.controlRecords().get(0).message());
+
+            // Check kraft version control record
+            assertEquals(ControlRecordType.KRAFT_VERSION, batch.controlRecords().get(1).type());
+            assertEquals(new KRaftVersionRecord().setKRaftVersion((short) 1), batch.controlRecords().get(1).message());
+
+            // Check the voters control record
+            assertEquals(ControlRecordType.KRAFT_VOTERS, batch.controlRecords().get(2).type());
+            assertEquals(voterSet.toVotersRecord((short) 0), batch.controlRecords().get(2).message());
+
+            // Consume the iterator until we find a control record
+            do {
+                batch = iterator.next();
+            }
+            while (batch.controlRecords().isEmpty());
+
+            // Check snapshot footer control record
+            assertEquals(1, batch.controlRecords().size());
+            assertEquals(ControlRecordType.SNAPSHOT_FOOTER, batch.controlRecords().get(0).type());
+            assertEquals(new SnapshotFooterRecord(), batch.controlRecords().get(0).message());
+
+            // Snapshot footer must be last record
+            assertFalse(iterator.hasNext());
+        }
+    }
+
     @ParameterizedTest
-    @EnumSource(value = ControlRecordType.class, names = {"LEADER_CHANGE", "SNAPSHOT_HEADER", "SNAPSHOT_FOOTER"})
+    @EnumSource(
+        value = ControlRecordType.class,
+        names = {"LEADER_CHANGE", "SNAPSHOT_HEADER", "SNAPSHOT_FOOTER", "KRAFT_VERSION", "KRAFT_VOTERS"}
+    )
     void testWithAllSupportedControlRecords(ControlRecordType type) {
         MemoryRecords records = buildControlRecords(type);
-        final ApiMessage expectedMessage;
-        switch (type) {
-            case LEADER_CHANGE:
-                expectedMessage = new LeaderChangeMessage();
-                break;
-            case SNAPSHOT_HEADER:
-                expectedMessage = new SnapshotHeaderRecord();
-                break;
-            case SNAPSHOT_FOOTER:
-                expectedMessage = new SnapshotFooterRecord();
-                break;
-            default:
-                throw new RuntimeException("Should not happen. Poorly configured test");
-        }
+        ApiMessage expectedMessage = defaultControlRecord(type);
 
         try (RecordsIterator<String> iterator = createIterator(records, BufferSupplier.NO_CACHING, true)) {
             assertTrue(iterator.hasNext());
@@ -226,7 +282,7 @@ public final class RecordsIteratorTest {
         // If this test fails then it means that ControlRecordType was changed. Please review the
         // implementation for RecordsIterator to see if it needs to be updated based on the changes
         // to ControlRecordType.
-        assertEquals(6, ControlRecordType.values().length);
+        assertEquals(8, ControlRecordType.values().length);
     }
 
     private void testIterator(
@@ -308,53 +364,47 @@ public final class RecordsIteratorTest {
     }
 
     public static MemoryRecords buildControlRecords(ControlRecordType type) {
-        final MemoryRecords records;
-        switch (type) {
-            case LEADER_CHANGE:
-                records = MemoryRecords.withLeaderChangeMessage(
-                    0,
-                    0,
-                    1,
-                    ByteBuffer.allocate(128),
-                    new LeaderChangeMessage()
-                );
-                break;
-            case SNAPSHOT_HEADER:
-                records = MemoryRecords.withSnapshotHeaderRecord(
-                    0,
-                    0,
-                    1,
-                    ByteBuffer.allocate(128),
-                    new SnapshotHeaderRecord()
-                );
-                break;
-            case SNAPSHOT_FOOTER:
-                records = MemoryRecords.withSnapshotFooterRecord(
-                    0,
-                    0,
-                    1,
-                    ByteBuffer.allocate(128),
-                    new SnapshotFooterRecord()
-                );
-                break;
-            default:
-                throw new RuntimeException(String.format("Control record type %s is not supported", type));
+        ByteBuffer buffer = ByteBuffer.allocate(128);
+
+        try (MemoryRecordsBuilder builder = new MemoryRecordsBuilder(
+                buffer,
+                RecordBatch.CURRENT_MAGIC_VALUE,
+                Compression.NONE,
+                TimestampType.CREATE_TIME,
+                0, // initialOffset
+                0, // timestamp
+                RecordBatch.NO_PRODUCER_ID,
+                RecordBatch.NO_PRODUCER_EPOCH,
+                RecordBatch.NO_SEQUENCE,
+                false,
+                true,
+                1, // leaderEpoch
+                buffer.capacity()
+            )
+        ) {
+            builder.appendControlRecord(
+                0,
+                type,
+                MessageUtil.toByteBuffer(defaultControlRecord(type), defaultControlRecordVersion(type))
+            );
         }
 
-        return records;
+        buffer.flip();
+        return MemoryRecords.readableRecords(buffer);
     }
 
     public static MemoryRecords buildRecords(
         CompressionType compressionType,
         List<TestBatch<String>> batches
     ) {
+        Compression compression = Compression.of(compressionType).build();
         ByteBuffer buffer = ByteBuffer.allocate(102400);
 
         for (TestBatch<String> batch : batches) {
             BatchBuilder<String> builder = new BatchBuilder<>(
                 buffer,
                 STRING_SERDE,
-                compressionType,
+                compression,
                 batch.baseOffset,
                 batch.appendTimestamp,
                 false,
@@ -415,4 +465,38 @@ public final class RecordsIteratorTest {
             return new TestBatch<>(batch.baseOffset(), batch.epoch(), batch.appendTimestamp(), batch.records());
         }
     }
+
+    private static ApiMessage defaultControlRecord(ControlRecordType type) {
+        switch (type) {
+            case LEADER_CHANGE:
+                return new LeaderChangeMessage();
+            case SNAPSHOT_HEADER:
+                return new SnapshotHeaderRecord();
+            case SNAPSHOT_FOOTER:
+                return new SnapshotFooterRecord();
+            case KRAFT_VERSION:
+                return new KRaftVersionRecord();
+            case KRAFT_VOTERS:
+                return new VotersRecord();
+            default:
+                throw new RuntimeException("Should not happen. Poorly configured test");
+        }
+    }
+
+    private static short defaultControlRecordVersion(ControlRecordType type) {
+        switch (type) {
+            case LEADER_CHANGE:
+                return ControlRecordUtils.LEADER_CHANGE_CURRENT_VERSION;
+            case SNAPSHOT_HEADER:
+                return ControlRecordUtils.SNAPSHOT_HEADER_CURRENT_VERSION;
+            case SNAPSHOT_FOOTER:
+                return ControlRecordUtils.SNAPSHOT_FOOTER_CURRENT_VERSION;
+            case KRAFT_VERSION:
+                return ControlRecordUtils.KRAFT_VERSION_CURRENT_VERSION;
+            case KRAFT_VOTERS:
+                return ControlRecordUtils.KRAFT_VOTERS_CURRENT_VERSION;
+            default:
+                throw new RuntimeException("Should not happen. Poorly configured test");
+        }
+    }
 }
