@@ -20,6 +20,7 @@ package org.apache.kafka.queue;
 import java.util.HashMap;
 import java.util.Map;
 import java.util.Objects;
+import java.util.Optional;
 import java.util.OptionalLong;
 import java.util.TreeMap;
 import java.util.concurrent.RejectedExecutionException;
@@ -35,6 +36,9 @@ import org.slf4j.Logger;
 
 
 public final class KafkaEventQueue implements EventQueue {
+
+    public static final String EVENT_HANDLER_THREAD_SUFFIX = "event-handler";
+
     /**
      * A context object that wraps events.
      */
@@ -201,7 +205,7 @@ public final class KafkaEventQueue implements EventQueue {
             }
         }
 
-        private void handleEvents() throws InterruptedException {
+        private void handleEvents() {
             Throwable toDeliver = null;
             EventContext toRun = null;
             boolean wasInterrupted = false;
@@ -245,12 +249,12 @@ public final class KafkaEventQueue implements EventQueue {
                             continue;
                         } else if (interrupted) {
                             remove(eventContext);
-                            toDeliver = new InterruptedException();
+                            toDeliver = new InterruptedException("The event handler thread is interrupted");
                             toRun = eventContext;
                             continue;
                         } else if (shuttingDown) {
                             remove(eventContext);
-                            toDeliver = new RejectedExecutionException();
+                            toDeliver = new RejectedExecutionException("The event queue is shutting down");
                             toRun = eventContext;
                             continue;
                         }
@@ -264,7 +268,7 @@ public final class KafkaEventQueue implements EventQueue {
                         }
                     } else {
                         if (interrupted) {
-                            toDeliver = new InterruptedException();
+                            toDeliver = new InterruptedException("The event handler thread is interrupted");
                         } else {
                             toDeliver = null;
                         }
@@ -300,10 +304,10 @@ public final class KafkaEventQueue implements EventQueue {
             lock.lock();
             try {
                 if (shuttingDown) {
-                    return new RejectedExecutionException();
+                    return new RejectedExecutionException("The event queue is shutting down");
                 }
                 if (interrupted) {
-                    return new InterruptedException();
+                    return new InterruptedException("The event handler thread is interrupted");
                 }
                 OptionalLong existingDeadlineNs = OptionalLong.empty();
                 if (eventContext.tag != null) {
@@ -453,7 +457,7 @@ public final class KafkaEventQueue implements EventQueue {
         this.lock = new ReentrantLock();
         this.log = logContext.logger(KafkaEventQueue.class);
         this.eventHandler = new EventHandler();
-        this.eventHandlerThread = new KafkaThread(threadNamePrefix + "event-handler",
+        this.eventHandlerThread = new KafkaThread(threadNamePrefix + EVENT_HANDLER_THREAD_SUFFIX,
             this.eventHandler, false);
         this.shuttingDown = false;
         this.interrupted = false;
@@ -513,4 +517,36 @@ public final class KafkaEventQueue implements EventQueue {
         eventHandlerThread.join();
         log.info("closed event queue.");
     }
+
+    /**
+     * Returns the deferred event that the queue is waiting for, idling until
+     * its deadline comes, if there is any.
+     * If the queue has immediate work to do, this returns empty.
+     * This is useful for unit tests, where to make progress, we need to
+     * speed the clock up until the next scheduled event is ready to run.
+     */
+    public Optional<Event> firstDeferredIfIdling() {
+        lock.lock();
+        try {
+            if (eventHandler.head.next != eventHandler.head) {
+                // There are events ready to run immediately. The queue is not idling.
+                return Optional.empty();
+            }
+            Map.Entry<Long, EventContext> entry = eventHandler.deadlineMap.firstEntry();
+            if (entry == null) {
+                // The queue is idling, but not waiting for any deadline.
+                return Optional.empty();
+            }
+            EventContext eventContext = entry.getValue();
+            if (eventContext.insertionType != EventInsertionType.DEFERRED) {
+                // Any event with a deadline is put in `deadlineMap`.
+                // But events of type other than DEFERRED will run immediately,
+                // so the queue will not idle waiting for their deadline.
+                return Optional.empty();
+            }
+            return Optional.of(eventContext.event);
+        } finally {
+            lock.unlock();
+        }
+    }
 }
