@@ -47,6 +47,7 @@ class MetadataVersionTest {
     }
 
     @Test
+    @SuppressWarnings("checkstyle:JavaNCSS")
     public void testFromVersionString() {
         assertEquals(IBP_0_8_0, MetadataVersion.fromVersionString("0.8.0"));
         assertEquals(IBP_0_8_0, MetadataVersion.fromVersionString("0.8.0.0"));
@@ -132,6 +133,8 @@ class MetadataVersionTest {
         assertEquals(IBP_2_6_IV0, MetadataVersion.fromVersionString("2.6"));
         assertEquals(IBP_2_6_IV0, MetadataVersion.fromVersionString("2.6-IV0"));
 
+        // 2.7-IV2 is the latest production version in the 2.7 line
+        assertEquals(IBP_2_7_IV2, MetadataVersion.fromVersionString("2.7"));
         assertEquals(IBP_2_7_IV0, MetadataVersion.fromVersionString("2.7-IV0"));
         assertEquals(IBP_2_7_IV1, MetadataVersion.fromVersionString("2.7-IV1"));
         assertEquals(IBP_2_7_IV2, MetadataVersion.fromVersionString("2.7-IV2"));
@@ -150,31 +153,42 @@ class MetadataVersionTest {
         assertEquals(IBP_3_2_IV0, MetadataVersion.fromVersionString("3.2"));
         assertEquals(IBP_3_2_IV0, MetadataVersion.fromVersionString("3.2-IV0"));
 
+        // 3.3-IV3 is the latest production version in the 3.3 line
+        assertEquals(IBP_3_3_IV3, MetadataVersion.fromVersionString("3.3"));
         assertEquals(IBP_3_3_IV0, MetadataVersion.fromVersionString("3.3-IV0"));
         assertEquals(IBP_3_3_IV1, MetadataVersion.fromVersionString("3.3-IV1"));
         assertEquals(IBP_3_3_IV2, MetadataVersion.fromVersionString("3.3-IV2"));
         assertEquals(IBP_3_3_IV3, MetadataVersion.fromVersionString("3.3-IV3"));
 
+        // 3.4-IV0 is the latest production version in the 3.4 line
+        assertEquals(IBP_3_4_IV0, MetadataVersion.fromVersionString("3.4"));
         assertEquals(IBP_3_4_IV0, MetadataVersion.fromVersionString("3.4-IV0"));
 
+        // 3.5-IV2 is the latest production version in the 3.5 line
+        assertEquals(IBP_3_5_IV2, MetadataVersion.fromVersionString("3.5"));
         assertEquals(IBP_3_5_IV0, MetadataVersion.fromVersionString("3.5-IV0"));
         assertEquals(IBP_3_5_IV1, MetadataVersion.fromVersionString("3.5-IV1"));
         assertEquals(IBP_3_5_IV2, MetadataVersion.fromVersionString("3.5-IV2"));
 
+        // 3.6-IV2 is the latest production version in the 3.6 line
+        assertEquals(IBP_3_6_IV2, MetadataVersion.fromVersionString("3.6"));
         assertEquals(IBP_3_6_IV0, MetadataVersion.fromVersionString("3.6-IV0"));
         assertEquals(IBP_3_6_IV1, MetadataVersion.fromVersionString("3.6-IV1"));
         assertEquals(IBP_3_6_IV2, MetadataVersion.fromVersionString("3.6-IV2"));
 
         // 3.7-IV4 is the latest production version in the 3.7 line
         assertEquals(IBP_3_7_IV4, MetadataVersion.fromVersionString("3.7"));
-
         assertEquals(IBP_3_7_IV0, MetadataVersion.fromVersionString("3.7-IV0"));
         assertEquals(IBP_3_7_IV1, MetadataVersion.fromVersionString("3.7-IV1"));
         assertEquals(IBP_3_7_IV2, MetadataVersion.fromVersionString("3.7-IV2"));
         assertEquals(IBP_3_7_IV3, MetadataVersion.fromVersionString("3.7-IV3"));
         assertEquals(IBP_3_7_IV4, MetadataVersion.fromVersionString("3.7-IV4"));
 
+        // 3.8-IV0 is the latest production version in the 3.8 line
+        assertEquals(IBP_3_8_IV0, MetadataVersion.fromVersionString("3.8"));
         assertEquals(IBP_3_8_IV0, MetadataVersion.fromVersionString("3.8-IV0"));
+
+        assertEquals(IBP_3_9_IV0, MetadataVersion.fromVersionString("3.9-IV0"));
     }
 
     @Test
@@ -234,6 +248,8 @@ class MetadataVersionTest {
         assertEquals("3.7", IBP_3_7_IV2.shortVersion());
         assertEquals("3.7", IBP_3_7_IV3.shortVersion());
         assertEquals("3.7", IBP_3_7_IV4.shortVersion());
+        assertEquals("3.8", IBP_3_8_IV0.shortVersion());
+        assertEquals("3.9", IBP_3_9_IV0.shortVersion());
     }
 
     @Test
@@ -283,6 +299,7 @@ class MetadataVersionTest {
         assertEquals("3.7-IV3", IBP_3_7_IV3.version());
         assertEquals("3.7-IV4", IBP_3_7_IV4.version());
         assertEquals("3.8-IV0", IBP_3_8_IV0.version());
+        assertEquals("3.9-IV0", IBP_3_9_IV0.version());
     }
 
     @Test
@@ -350,7 +367,7 @@ class MetadataVersionTest {
     @ParameterizedTest
     @EnumSource(value = MetadataVersion.class)
     public void testIsElrSupported(MetadataVersion metadataVersion) {
-        assertEquals(metadataVersion.isAtLeast(IBP_3_8_IV0), metadataVersion.isElrSupported());
+        assertEquals(metadataVersion.isAtLeast(IBP_3_9_IV0), metadataVersion.isElrSupported());
     }
 
     @ParameterizedTest
