@@ -29,6 +29,8 @@ import java.util.Optional;
 import java.util.OptionalLong;
 import java.util.concurrent.Callable;
 import java.util.concurrent.TimeUnit;
+import java.util.regex.Matcher;
+import java.util.regex.Pattern;
 
 import com.yammer.metrics.core.MetricName;
 import com.yammer.metrics.core.Timer;
@@ -63,6 +65,9 @@ import static java.util.Arrays.asList;
 public class LogSegment implements Closeable {
     private static final Logger LOGGER = LoggerFactory.getLogger(LogSegment.class);
     private static final Timer LOG_FLUSH_TIMER;
+    /* a directory that is used for future partition */
+    private static final String FUTURE_DIR_SUFFIX = "-future";
+    private static final Pattern FUTURE_DIR_PATTERN = Pattern.compile("^(\\S+)-(\\S+)\\.(\\S+)" + FUTURE_DIR_SUFFIX);
 
     static {
         KafkaMetricsGroup logFlushStatsMetricsGroup = new KafkaMetricsGroup(LogSegment.class) {
@@ -89,7 +94,8 @@ public class LogSegment implements Closeable {
     // volatile for LogCleaner to see the update
     private volatile OptionalLong rollingBasedTimestamp = OptionalLong.empty();
 
-    /* The maximum timestamp and offset we see so far */
+    // The maximum timestamp and offset we see so far
+    // NOTED: the offset is the last offset of batch having the max timestamp.
     private volatile TimestampOffset maxTimestampAndOffsetSoFar = TimestampOffset.UNKNOWN;
 
     private long created;
@@ -128,12 +134,6 @@ public class LogSegment implements Closeable {
         this.created = time.milliseconds();
     }
 
-    // Visible for testing
-    public LogSegment(LogSegment segment) {
-        this(segment.log, segment.lazyOffsetIndex, segment.lazyTimeIndex, segment.txnIndex, segment.baseOffset,
-                segment.indexIntervalBytes, segment.rollJitterMs, segment.time);
-    }
-
     public OffsetIndex offsetIndex() throws IOException {
         return lazyOffsetIndex.get();
     }
@@ -214,7 +214,7 @@ public class LogSegment implements Closeable {
     /**
      * Note that this may result in time index materialization.
      */
-    private long offsetOfMaxTimestampSoFar() throws IOException {
+    private long shallowOffsetOfMaxTimestampSoFar() throws IOException {
         return readMaxTimestampAndOffsetSoFar().offset;
     }
 
@@ -238,7 +238,7 @@ public class LogSegment implements Closeable {
      *
      * @param largestOffset The last offset in the message set
      * @param largestTimestampMs The largest timestamp in the message set.
-     * @param shallowOffsetOfMaxTimestamp The offset of the message that has the largest timestamp in the messages to append.
+     * @param shallowOffsetOfMaxTimestamp The last offset of earliest batch with max timestamp in the messages to append.
      * @param records The log entries to append.
      * @throws LogSegmentOffsetOverflowException if the largest offset causes index offset overflow
      */
@@ -247,7 +247,7 @@ public class LogSegment implements Closeable {
                        long shallowOffsetOfMaxTimestamp,
                        MemoryRecords records) throws IOException {
         if (records.sizeInBytes() > 0) {
-            LOGGER.trace("Inserting {} bytes at end offset {} at position {} with largest timestamp {} at shallow offset {}",
+            LOGGER.trace("Inserting {} bytes at end offset {} at position {} with largest timestamp {} at offset {}",
                 records.sizeInBytes(), largestOffset, log.sizeInBytes(), largestTimestampMs, shallowOffsetOfMaxTimestamp);
             int physicalPosition = log.sizeInBytes();
             if (physicalPosition == 0)
@@ -265,7 +265,7 @@ public class LogSegment implements Closeable {
             // append an entry to the index (if needed)
             if (bytesSinceLastIndexEntry > indexIntervalBytes) {
                 offsetIndex().append(largestOffset, physicalPosition);
-                timeIndex().maybeAppend(maxTimestampSoFar(), offsetOfMaxTimestampSoFar());
+                timeIndex().maybeAppend(maxTimestampSoFar(), shallowOffsetOfMaxTimestampSoFar());
                 bytesSinceLastIndexEntry = 0;
             }
             bytesSinceLastIndexEntry += records.sizeInBytes();
@@ -280,7 +280,7 @@ public class LogSegment implements Closeable {
     private int appendChunkFromFile(FileRecords records, int position, BufferSupplier bufferSupplier) throws IOException {
         int bytesToAppend = 0;
         long maxTimestamp = Long.MIN_VALUE;
-        long offsetOfMaxTimestamp = Long.MIN_VALUE;
+        long shallowOffsetOfMaxTimestamp = Long.MIN_VALUE;
         long maxOffset = Long.MIN_VALUE;
         ByteBuffer readBuffer = bufferSupplier.get(1024 * 1024);
 
@@ -291,7 +291,7 @@ public class LogSegment implements Closeable {
         while ((batch = nextAppendableBatch(nextBatches, readBuffer, bytesToAppend)) != null) {
             if (batch.maxTimestamp() > maxTimestamp) {
                 maxTimestamp = batch.maxTimestamp();
-                offsetOfMaxTimestamp = batch.lastOffset();
+                shallowOffsetOfMaxTimestamp = batch.lastOffset();
             }
             maxOffset = batch.lastOffset();
             bytesToAppend += batch.sizeInBytes();
@@ -305,7 +305,7 @@ public class LogSegment implements Closeable {
             readBuffer.limit(bytesToAppend);
             records.readInto(readBuffer, position);
 
-            append(maxOffset, maxTimestamp, offsetOfMaxTimestamp, MemoryRecords.readableRecords(readBuffer));
+            append(maxOffset, maxTimestamp, shallowOffsetOfMaxTimestamp, MemoryRecords.readableRecords(readBuffer));
         }
 
         bufferSupplier.release(readBuffer);
@@ -398,7 +398,7 @@ public class LogSegment implements Closeable {
     /**
      * Equivalent to {@code read(startOffset, maxSize, size())}.
      *
-     * See {@link #read(long, int, long, boolean)} for details.
+     * See {@link #read(long, int, Optional, boolean)} for details.
      */
     public FetchDataInfo read(long startOffset, int maxSize) throws IOException {
         return read(startOffset, maxSize, size());
@@ -407,10 +407,10 @@ public class LogSegment implements Closeable {
     /**
      * Equivalent to {@code read(startOffset, maxSize, maxPosition, false)}.
      *
-     * See {@link #read(long, int, long, boolean)} for details.
+     * See {@link #read(long, int, Optional, boolean)} for details.
      */
     public FetchDataInfo read(long startOffset, int maxSize, long maxPosition) throws IOException {
-        return read(startOffset, maxSize, maxPosition, false);
+        return read(startOffset, maxSize, Optional.of(maxPosition), false);
     }
 
     /**
@@ -421,13 +421,13 @@ public class LogSegment implements Closeable {
      *
      * @param startOffset A lower bound on the first offset to include in the message set we read
      * @param maxSize The maximum number of bytes to include in the message set we read
-     * @param maxPosition The maximum position in the log segment that should be exposed for read
+     * @param maxPositionOpt The maximum position in the log segment that should be exposed for read
      * @param minOneMessage If this is true, the first message will be returned even if it exceeds `maxSize` (if one exists)
      *
      * @return The fetched data and the offset metadata of the first message whose offset is >= startOffset,
      *         or null if the startOffset is larger than the largest offset in this log
      */
-    public FetchDataInfo read(long startOffset, int maxSize, long maxPosition, boolean minOneMessage) throws IOException {
+    public FetchDataInfo read(long startOffset, int maxSize, Optional<Long> maxPositionOpt, boolean minOneMessage) throws IOException {
         if (maxSize < 0)
             throw new IllegalArgumentException("Invalid max size " + maxSize + " for log read from segment " + log);
 
@@ -444,12 +444,14 @@ public class LogSegment implements Closeable {
         if (minOneMessage)
             adjustedMaxSize = Math.max(maxSize, startOffsetAndSize.size);
 
-        // return a log segment but with zero size in the case below
-        if (adjustedMaxSize == 0)
+        // return empty records in the fetch-data-info when:
+        // 1. adjustedMaxSize is 0 (or)
+        // 2. maxPosition to read is unavailable
+        if (adjustedMaxSize == 0 || !maxPositionOpt.isPresent())
             return new FetchDataInfo(offsetMetadata, MemoryRecords.EMPTY);
 
         // calculate the length of the message set to read based on whether or not they gave us a maxOffset
-        int fetchSize = Math.min((int) (maxPosition - startPosition), adjustedMaxSize);
+        int fetchSize = Math.min((int) (maxPositionOpt.get() - startPosition), adjustedMaxSize);
 
         return new FetchDataInfo(offsetMetadata, log.slice(startPosition, fetchSize),
             adjustedMaxSize < startOffsetAndSize.size, Optional.empty());
@@ -494,7 +496,7 @@ public class LogSegment implements Closeable {
                 // Build offset index
                 if (validBytes - lastIndexEntry > indexIntervalBytes) {
                     offsetIndex().append(batch.lastOffset(), validBytes);
-                    timeIndex().maybeAppend(maxTimestampSoFar(), offsetOfMaxTimestampSoFar());
+                    timeIndex().maybeAppend(maxTimestampSoFar(), shallowOffsetOfMaxTimestampSoFar());
                     lastIndexEntry = validBytes;
                 }
                 validBytes += batch.sizeInBytes();
@@ -519,7 +521,7 @@ public class LogSegment implements Closeable {
         log.truncateTo(validBytes);
         offsetIndex().trimToValidSize();
         // A normally closed segment always appends the biggest timestamp ever seen into log segment, we do this as well.
-        timeIndex().maybeAppend(maxTimestampSoFar(), offsetOfMaxTimestampSoFar(), true);
+        timeIndex().maybeAppend(maxTimestampSoFar(), shallowOffsetOfMaxTimestampSoFar(), true);
         timeIndex().trimToValidSize();
         return truncated;
     }
@@ -680,7 +682,7 @@ public class LogSegment implements Closeable {
      * The time index entry appended will be used to decide when to delete the segment.
      */
     public void onBecomeInactiveSegment() throws IOException {
-        timeIndex().maybeAppend(maxTimestampSoFar(), offsetOfMaxTimestampSoFar(), true);
+        timeIndex().maybeAppend(maxTimestampSoFar(), shallowOffsetOfMaxTimestampSoFar(), true);
         offsetIndex().trimToValidSize();
         timeIndex().trimToValidSize();
         log.trim();
@@ -717,9 +719,9 @@ public class LogSegment implements Closeable {
     }
 
     /**
-     * @return the first batch timestamp if the timestamp is available. Otherwise return Long.MaxValue
+     * @return the first batch timestamp if the timestamp is available. Otherwise, return Long.MaxValue
      */
-    long getFirstBatchTimestamp() {
+    public long getFirstBatchTimestamp() {
         loadFirstBatchTimestamp();
         OptionalLong timestamp = rollingBasedTimestamp;
         if (timestamp.isPresent() && timestamp.getAsLong() >= 0)
@@ -732,21 +734,21 @@ public class LogSegment implements Closeable {
      *
      * This method returns an option of TimestampOffset. The returned value is determined using the following ordered list of rules:
      *
-     * - If all the messages in the segment have smaller offsets, return None
-     * - If all the messages in the segment have smaller timestamps, return None
+     * - If all the messages in the segment have smaller offsets, return Empty
+     * - If all the messages in the segment have smaller timestamps, return Empty
      * - If all the messages in the segment have larger timestamps, or no message in the segment has a timestamp
      *   the returned the offset will be max(the base offset of the segment, startingOffset) and the timestamp will be Message.NoTimestamp.
      * - Otherwise, return an option of TimestampOffset. The offset is the offset of the first message whose timestamp
      *   is greater than or equals to the target timestamp and whose offset is greater than or equals to the startingOffset.
      *
-     * This method only returns None when 1) all messages' offset < startOffing or 2) the log is not empty but we did not
+     * This method only returns Empty when 1) all messages' offset < startOffing or 2) the log is not empty, but we did not
      * see any message when scanning the log from the indexed position. The latter could happen if the log is truncated
-     * after we get the indexed position but before we scan the log from there. In this case we simply return None and the
+     * after we get the indexed position but before we scan the log from there. In this case we simply return Empty and the
      * caller will need to check on the truncated log and maybe retry or even do the search on another log segment.
      *
      * @param timestampMs The timestamp to search for.
      * @param startingOffset The starting offset to search.
-     * @return the timestamp and offset of the first message that meets the requirements. None will be returned if there is no such message.
+     * @return the timestamp and offset of the first message that meets the requirements. Empty will be returned if there is no such message.
      */
     public Optional<FileRecords.TimestampAndOffset> findOffsetByTimestamp(long timestampMs, long startingOffset) throws IOException {
         // Get the index entry with a timestamp less than or equal to the target timestamp
@@ -763,7 +765,7 @@ public class LogSegment implements Closeable {
     @Override
     public void close() throws IOException {
         if (maxTimestampAndOffsetSoFar != TimestampOffset.UNKNOWN)
-            Utils.swallow(LOGGER, Level.WARN, "maybeAppend", () -> timeIndex().maybeAppend(maxTimestampSoFar(), offsetOfMaxTimestampSoFar(), true));
+            Utils.swallow(LOGGER, Level.WARN, "maybeAppend", () -> timeIndex().maybeAppend(maxTimestampSoFar(), shallowOffsetOfMaxTimestampSoFar(), true));
         Utils.closeQuietly(lazyOffsetIndex, "offsetIndex", LOGGER);
         Utils.closeQuietly(lazyTimeIndex, "timeIndex", LOGGER);
         Utils.closeQuietly(log, "log", LOGGER);
@@ -806,8 +808,22 @@ public class LogSegment implements Closeable {
         try {
             if (delete.execute())
                 LOGGER.info("Deleted {} {}.", fileType, file.getAbsolutePath());
-            else if (logIfMissing)
-                LOGGER.info("Failed to delete {} {} because it does not exist.", fileType, file.getAbsolutePath());
+            else {
+                if (logIfMissing) {
+                    LOGGER.info("Failed to delete {} {} because it does not exist.", fileType, file.getAbsolutePath());
+                }
+
+                // During alter log dir, the log segment may be moved to a new directory, so async delete may fail.
+                // Fallback to delete the file in the new directory to avoid orphan file.
+                Matcher dirMatcher = FUTURE_DIR_PATTERN.matcher(file.getParent());
+                if (dirMatcher.matches()) {
+                    String topicPartitionAbsolutePath = dirMatcher.group(1) + "-" + dirMatcher.group(2);
+                    File fallbackFile = new File(topicPartitionAbsolutePath, file.getName());
+                    if (fallbackFile.exists() && file.getName().endsWith(LogFileUtils.DELETED_FILE_SUFFIX) && fallbackFile.delete()) {
+                        LOGGER.info("Fallback to delete {} {}.", fileType, fallbackFile.getAbsolutePath());
+                    }
+                }
+            }
             return null;
         } catch (IOException e) {
             throw new IOException("Delete of " + fileType + " " + file.getAbsolutePath() + " failed.", e);
@@ -828,7 +844,7 @@ public class LogSegment implements Closeable {
     }
 
     /**
-     * The largest timestamp this segment contains, if maxTimestampSoFar >= 0, otherwise None.
+     * The largest timestamp this segment contains, if maxTimestampSoFar >= 0, otherwise Empty.
      */
     public OptionalLong largestRecordTimestamp() throws IOException {
         long maxTimestampSoFar = maxTimestampSoFar();
