@@ -51,6 +51,7 @@ import org.apache.kafka.streams.internals.metrics.ClientMetrics;
 import org.apache.kafka.streams.processor.StandbyUpdateListener;
 import org.apache.kafka.streams.processor.StateRestoreListener;
 import org.apache.kafka.streams.processor.TaskId;
+import org.apache.kafka.streams.processor.assignment.ProcessId;
 import org.apache.kafka.streams.processor.internals.assignment.AssignorError;
 import org.apache.kafka.streams.processor.internals.assignment.ReferenceContainer;
 import org.apache.kafka.streams.processor.internals.metrics.StreamsMetricsImpl;
@@ -286,6 +287,7 @@ public class StreamThread extends Thread implements ProcessingThread {
     private final int maxPollTimeMs;
     private final String originalReset;
     private final TaskManager taskManager;
+    private final StateUpdater stateUpdater;
 
     private final StreamsMetricsImpl streamsMetrics;
     private final Sensor commitSensor;
@@ -346,6 +348,7 @@ public class StreamThread extends Thread implements ProcessingThread {
 
     private volatile long fetchDeadlineClientInstanceId = -1;
     private volatile KafkaFutureImpl<Uuid> mainConsumerInstanceIdFuture = new KafkaFutureImpl<>();
+    private volatile KafkaFutureImpl<Uuid> restoreConsumerInstanceIdFuture = new KafkaFutureImpl<>();
     private volatile KafkaFutureImpl<Map<String, KafkaFuture<Uuid>>> producerInstanceIdFuture = new KafkaFutureImpl<>();
     private volatile KafkaFutureImpl<Uuid> threadProducerInstanceIdFuture = new KafkaFutureImpl<>();
 
@@ -427,12 +430,22 @@ public class StreamThread extends Thread implements ProcessingThread {
         final DefaultTaskManager schedulingTaskManager =
             maybeCreateSchedulingTaskManager(processingThreadsEnabled, stateUpdaterEnabled, topologyMetadata, time, threadId, tasks);
         final StateUpdater stateUpdater =
-            maybeCreateAndStartStateUpdater(stateUpdaterEnabled, streamsMetrics, config, changelogReader, topologyMetadata, time, clientId, threadIdx);
+            maybeCreateAndStartStateUpdater(
+                stateUpdaterEnabled,
+                streamsMetrics,
+                config,
+                restoreConsumer,
+                changelogReader,
+                topologyMetadata,
+                time,
+                clientId,
+                threadIdx
+            );
 
         final TaskManager taskManager = new TaskManager(
             time,
             changelogReader,
-            processId,
+            new ProcessId(processId),
             logPrefix,
             activeTaskCreator,
             standbyTaskCreator,
@@ -469,6 +482,7 @@ public class StreamThread extends Thread implements ProcessingThread {
             changelogReader,
             originalReset,
             taskManager,
+            stateUpdater,
             streamsMetrics,
             topologyMetadata,
             threadId,
@@ -512,6 +526,7 @@ public class StreamThread extends Thread implements ProcessingThread {
     private static StateUpdater maybeCreateAndStartStateUpdater(final boolean stateUpdaterEnabled,
                                                                 final StreamsMetricsImpl streamsMetrics,
                                                                 final StreamsConfig streamsConfig,
+                                                                final Consumer<byte[], byte[]> restoreConsumer,
                                                                 final ChangelogReader changelogReader,
                                                                 final TopologyMetadata topologyMetadata,
                                                                 final Time time,
@@ -519,7 +534,15 @@ public class StreamThread extends Thread implements ProcessingThread {
                                                                 final int threadIdx) {
         if (stateUpdaterEnabled) {
             final String name = clientId + "-StateUpdater-" + threadIdx;
-            final StateUpdater stateUpdater = new DefaultStateUpdater(name, streamsMetrics.metricsRegistry(), streamsConfig, changelogReader, topologyMetadata, time);
+            final StateUpdater stateUpdater = new DefaultStateUpdater(
+                name,
+                streamsMetrics.metricsRegistry(),
+                streamsConfig,
+                restoreConsumer,
+                changelogReader,
+                topologyMetadata,
+                time
+            );
             stateUpdater.start();
             return stateUpdater;
         } else {
@@ -536,6 +559,7 @@ public class StreamThread extends Thread implements ProcessingThread {
                         final ChangelogReader changelogReader,
                         final String originalReset,
                         final TaskManager taskManager,
+                        final StateUpdater stateUpdater,
                         final StreamsMetricsImpl streamsMetrics,
                         final TopologyMetadata topologyMetadata,
                         final String threadId,
@@ -598,6 +622,7 @@ public class StreamThread extends Thread implements ProcessingThread {
         this.log = logContext.logger(StreamThread.class);
         this.rebalanceListener = new StreamsRebalanceListener(time, taskManager, this, this.log, this.assignmentErrorCode);
         this.taskManager = taskManager;
+        this.stateUpdater = stateUpdater;
         this.restoreConsumer = restoreConsumer;
         this.mainConsumer = mainConsumer;
         this.changelogReader = changelogReader;
@@ -759,6 +784,27 @@ public class StreamThread extends Thread implements ProcessingThread {
                 }
             }
 
+
+            if (!stateUpdaterEnabled && !restoreConsumerInstanceIdFuture.isDone()) {
+                if (fetchDeadlineClientInstanceId >= time.milliseconds()) {
+                    try {
+                        restoreConsumerInstanceIdFuture.complete(restoreConsumer.clientInstanceId(Duration.ZERO));
+                    } catch (final IllegalStateException disabledError) {
+                        // if telemetry is disabled on a client, we swallow the error,
+                        // to allow returning a partial result for all other clients
+                        restoreConsumerInstanceIdFuture.complete(null);
+                    } catch (final TimeoutException swallow) {
+                        // swallow
+                    } catch (final Exception error) {
+                        restoreConsumerInstanceIdFuture.completeExceptionally(error);
+                    }
+                } else {
+                    restoreConsumerInstanceIdFuture.completeExceptionally(
+                        new TimeoutException("Could not retrieve restore consumer client instance id.")
+                    );
+                }
+            }
+
             if (!processingMode.equals(StreamsConfigUtils.ProcessingMode.EXACTLY_ONCE_ALPHA) &&
                     !threadProducerInstanceIdFuture.isDone()) {
 
@@ -788,7 +834,8 @@ public class StreamThread extends Thread implements ProcessingThread {
     }
 
     private void maybeResetFetchDeadline() {
-        boolean reset = mainConsumerInstanceIdFuture.isDone();
+        boolean reset = mainConsumerInstanceIdFuture.isDone()
+            && (!stateUpdaterEnabled && restoreConsumerInstanceIdFuture.isDone());
 
         if (processingMode.equals(StreamsConfigUtils.ProcessingMode.EXACTLY_ONCE_ALPHA)) {
             throw new UnsupportedOperationException("not implemented yet");
@@ -1580,6 +1627,20 @@ public class StreamThread extends Thread implements ProcessingThread {
         }
         result.put(getName() + "-consumer", mainConsumerInstanceIdFuture);
 
+        if (stateUpdaterEnabled) {
+            restoreConsumerInstanceIdFuture = stateUpdater.restoreConsumerInstanceId(timeout);
+        } else {
+            if (restoreConsumerInstanceIdFuture.isDone()) {
+                if (restoreConsumerInstanceIdFuture.isCompletedExceptionally()) {
+                    restoreConsumerInstanceIdFuture = new KafkaFutureImpl<>();
+                    setDeadline = true;
+                }
+            } else {
+                setDeadline = true;
+            }
+        }
+        result.put(getName() + "-restore-consumer", restoreConsumerInstanceIdFuture);
+
         if (setDeadline) {
             fetchDeadlineClientInstanceId = time.milliseconds() + timeout.toMillis();
         }
@@ -1601,9 +1662,7 @@ public class StreamThread extends Thread implements ProcessingThread {
         }
 
         if (processingMode.equals(StreamsConfigUtils.ProcessingMode.EXACTLY_ONCE_ALPHA)) {
-            log.error("EOS v1 enabled: Producer client instance IDs are not collected." +
-                " Enable producer logging to retrieve the IDs from the producer logs.");
-            producerInstanceIdFuture.complete(Collections.emptyMap());
+            throw new UnsupportedOperationException("not yet implemented");
         } else {
             if (threadProducerInstanceIdFuture.isDone()) {
                 if (threadProducerInstanceIdFuture.isCompletedExceptionally()) {
