@@ -16,8 +16,13 @@
  */
 package org.apache.kafka.streams.processor.internals.assignment;
 
+import java.time.Instant;
+import java.util.Optional;
+import java.util.SortedMap;
 import org.apache.kafka.common.TopicPartition;
 import org.apache.kafka.streams.processor.TaskId;
+import org.apache.kafka.streams.processor.assignment.KafkaStreamsAssignment;
+import org.apache.kafka.streams.processor.assignment.ProcessId;
 import org.apache.kafka.streams.processor.internals.Task;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
@@ -31,7 +36,6 @@ import java.util.Set;
 import java.util.SortedSet;
 import java.util.TreeMap;
 import java.util.TreeSet;
-import java.util.UUID;
 import java.util.stream.Collectors;
 
 import static java.util.Collections.emptyMap;
@@ -40,6 +44,8 @@ import static java.util.Collections.unmodifiableSet;
 import static java.util.Comparator.comparing;
 import static java.util.Comparator.comparingLong;
 import static org.apache.kafka.common.utils.Utils.union;
+import static org.apache.kafka.streams.processor.assignment.KafkaStreamsAssignment.AssignedTask.Type.ACTIVE;
+import static org.apache.kafka.streams.processor.assignment.KafkaStreamsAssignment.AssignedTask.Type.STANDBY;
 import static org.apache.kafka.streams.processor.internals.assignment.SubscriptionInfo.UNKNOWN_OFFSET_SUM;
 
 public class ClientState {
@@ -57,15 +63,16 @@ public class ClientState {
     private final ClientStateTask previousActiveTasks = new ClientStateTask(null, new TreeMap<>());
     private final ClientStateTask previousStandbyTasks = new ClientStateTask(null, null);
     private final ClientStateTask revokingActiveTasks = new ClientStateTask(null, new TreeMap<>());
+    private final ProcessId processId;
 
+    private Optional<Instant> followupRebalanceDeadline = Optional.empty();
     private int capacity;
-    private UUID processId;
 
     public ClientState() {
         this(null, 0);
     }
 
-    public ClientState(final UUID processId, final Map<String, String> clientTags) {
+    public ClientState(final ProcessId processId, final Map<String, String> clientTags) {
         this(processId, 0, clientTags);
     }
 
@@ -73,13 +80,13 @@ public class ClientState {
         this(null, capacity);
     }
 
-    ClientState(final UUID processId, final int capacity) {
+    ClientState(final ProcessId processId, final int capacity) {
         this(processId, capacity, Collections.emptyMap());
     }
 
-    ClientState(final UUID processId, final int capacity, final Map<String, String> clientTags) {
-        previousStandbyTasks.taskIds(new TreeSet<>());
-        previousActiveTasks.taskIds(new TreeSet<>());
+    ClientState(final ProcessId processId, final int capacity, final Map<String, String> clientTags) {
+        previousStandbyTasks.setTaskIds(new TreeSet<>());
+        previousActiveTasks.setTaskIds(new TreeSet<>());
         taskOffsetSums = new TreeMap<>();
         taskLagTotals = new TreeMap<>();
         this.capacity = capacity;
@@ -102,9 +109,9 @@ public class ClientState {
                        final Map<TaskId, Long> taskLagTotals,
                        final Map<String, String> clientTags,
                        final int capacity,
-                       final UUID processId) {
-        this.previousStandbyTasks.taskIds(unmodifiableSet(new TreeSet<>(previousStandbyTasks)));
-        this.previousActiveTasks.taskIds(unmodifiableSet(new TreeSet<>(previousActiveTasks)));
+                       final ProcessId processId) {
+        this.previousStandbyTasks.setTaskIds(unmodifiableSet(new TreeSet<>(previousStandbyTasks)));
+        this.previousActiveTasks.setTaskIds(unmodifiableSet(new TreeSet<>(previousActiveTasks)));
         taskOffsetSums = emptyMap();
         this.taskLagTotals = unmodifiableMap(taskLagTotals);
         this.capacity = capacity;
@@ -128,7 +135,7 @@ public class ClientState {
         return capacity;
     }
 
-    UUID processId() {
+    ProcessId processId() {
         return processId;
     }
 
@@ -140,6 +147,14 @@ public class ClientState {
         return assignedTaskCount() >= capacity;
     }
 
+    public Optional<Instant> followupRebalanceDeadline() {
+        return followupRebalanceDeadline;
+    }
+
+    public void setFollowupRebalanceDeadline(final Instant followupRebalanceDeadline) {
+        this.followupRebalanceDeadline = Optional.of(followupRebalanceDeadline);
+    }
+
     public Set<TaskId> activeTasks() {
         return unmodifiableSet(assignedActiveTasks.taskIds());
     }
@@ -363,7 +378,7 @@ public class ClientState {
     /**
      * Compute the lag for each stateful task, including tasks this client did not previously have.
      */
-    public void computeTaskLags(final UUID uuid, final Map<TaskId, Long> allTaskEndOffsetSums) {
+    public void computeTaskLags(final ProcessId uuid, final Map<TaskId, Long> allTaskEndOffsetSums) {
         if (!taskLagTotals.isEmpty()) {
             throw new IllegalStateException("Already computed task lags for this client.");
         }
@@ -457,6 +472,33 @@ public class ClientState {
         return consumerToPreviousStatefulTaskIds.keySet().toString();
     }
 
+    public Map<TaskId, Long> taskLagTotals() {
+        return taskLagTotals;
+    }
+
+    public SortedSet<TaskId> previousActiveTasks() {
+        return new TreeSet<>(previousActiveTasks.taskIds());
+    }
+
+    public SortedSet<TaskId> previousStandbyTasks() {
+        return new TreeSet<>(previousStandbyTasks.taskIds());
+    }
+
+    public SortedMap<String, Set<TaskId>> taskIdsByPreviousConsumer() {
+        return new TreeMap<>(consumerToPreviousStatefulTaskIds);
+    }
+
+    public void setAssignedTasks(final KafkaStreamsAssignment assignment) {
+        final Set<TaskId> activeTasks = assignment.tasks().values().stream()
+            .filter(task -> task.type() == ACTIVE).map(KafkaStreamsAssignment.AssignedTask::id)
+            .collect(Collectors.toSet());
+        final Set<TaskId> standbyTasks = assignment.tasks().values().stream()
+            .filter(task -> task.type() == STANDBY).map(KafkaStreamsAssignment.AssignedTask::id)
+            .collect(Collectors.toSet());
+        assignedActiveTasks.setTaskIds(activeTasks);
+        assignedStandbyTasks.setTaskIds(standbyTasks);
+    }
+
     public String currentAssignment() {
         return "[activeTasks: (" + assignedActiveTasks.taskIds() +
                ") standbyTasks: (" + assignedStandbyTasks.taskIds() + ")]";
