@@ -44,17 +44,15 @@ import org.apache.kafka.common.utils.LogCaptureAppender;
 import org.apache.kafka.test.MockStandbyUpdateListener;
 import org.apache.kafka.test.MockStateRestoreListener;
 import org.apache.kafka.test.StreamsTestUtils;
-import org.easymock.EasyMock;
-import org.easymock.EasyMockRule;
-import org.easymock.EasyMockSupport;
-import org.easymock.Mock;
-import org.easymock.MockType;
-import org.junit.After;
-import org.junit.Before;
 import org.junit.Rule;
 import org.junit.Test;
 import org.junit.runner.RunWith;
 import org.junit.runners.Parameterized;
+import org.mockito.Mockito;
+import org.mockito.junit.MockitoJUnit;
+import org.mockito.junit.MockitoRule;
+import org.mockito.junit.jupiter.MockitoSettings;
+import org.mockito.quality.Strictness;
 
 import java.time.Duration;
 import java.util.Collections;
@@ -78,13 +76,6 @@ import static org.apache.kafka.test.MockStateRestoreListener.RESTORE_START;
 import static org.apache.kafka.test.MockStandbyUpdateListener.UPDATE_SUSPENDED;
 import static org.apache.kafka.test.MockStandbyUpdateListener.UPDATE_START;
 import static org.apache.kafka.test.MockStandbyUpdateListener.UPDATE_BATCH;
-import static org.easymock.EasyMock.anyBoolean;
-import static org.easymock.EasyMock.anyLong;
-import static org.easymock.EasyMock.anyObject;
-import static org.easymock.EasyMock.expectLastCall;
-import static org.easymock.EasyMock.replay;
-import static org.easymock.EasyMock.resetToDefault;
-import static org.easymock.EasyMock.verify;
 import static org.hamcrest.MatcherAssert.assertThat;
 import static org.hamcrest.Matchers.equalTo;
 import static org.hamcrest.Matchers.hasItem;
@@ -93,27 +84,34 @@ import static org.junit.Assert.assertFalse;
 import static org.junit.Assert.assertNull;
 import static org.junit.Assert.assertThrows;
 import static org.junit.Assert.assertTrue;
+import static org.mockito.ArgumentMatchers.any;
+import static org.mockito.ArgumentMatchers.anyBoolean;
+import static org.mockito.ArgumentMatchers.anyLong;
+import static org.mockito.Mockito.mock;
+import static org.mockito.Mockito.times;
+import static org.mockito.Mockito.when;
 
+@MockitoSettings(strictness = Strictness.STRICT_STUBS)
 @RunWith(Parameterized.class)
 @SuppressWarnings("this-escape")
-public class StoreChangelogReaderTest extends EasyMockSupport {
+public class StoreChangelogReaderTest {
 
     @Rule
-    public EasyMockRule rule = new EasyMockRule(this);
+    public final MockitoRule mockitoRule = MockitoJUnit.rule().strictness(Strictness.STRICT_STUBS);
 
-    @Mock(type = MockType.NICE)
+    @org.mockito.Mock
     private ProcessorStateManager stateManager;
-    @Mock(type = MockType.NICE)
+    @org.mockito.Mock
     private ProcessorStateManager activeStateManager;
-    @Mock(type = MockType.NICE)
+    @org.mockito.Mock
     private ProcessorStateManager standbyStateManager;
-    @Mock(type = MockType.NICE)
+    @org.mockito.Mock
     private StateStoreMetadata storeMetadata;
-    @Mock(type = MockType.NICE)
+    @org.mockito.Mock
     private StateStoreMetadata storeMetadataOne;
-    @Mock(type = MockType.NICE)
+    @org.mockito.Mock
     private StateStoreMetadata storeMetadataTwo;
-    @Mock(type = MockType.NICE)
+    @org.mockito.Mock
     private StateStore store;
 
     @Parameterized.Parameters
@@ -158,36 +156,33 @@ public class StoreChangelogReaderTest extends EasyMockSupport {
     private final StoreChangelogReader changelogReader =
         new StoreChangelogReader(time, config, logContext, adminClient, consumer, callback, standbyListener);
 
-    @Before
-    public void setUp() {
-        EasyMock.expect(stateManager.storeMetadata(tp)).andReturn(storeMetadata).anyTimes();
-        EasyMock.expect(stateManager.taskType()).andReturn(type).anyTimes();
-        EasyMock.expect(activeStateManager.storeMetadata(tp)).andReturn(storeMetadata).anyTimes();
-        EasyMock.expect(activeStateManager.taskType()).andReturn(ACTIVE).anyTimes();
-        EasyMock.expect(standbyStateManager.storeMetadata(tp)).andReturn(storeMetadata).anyTimes();
-        EasyMock.expect(standbyStateManager.taskType()).andReturn(STANDBY).anyTimes();
-
-        EasyMock.expect(storeMetadata.changelogPartition()).andReturn(tp).anyTimes();
-        EasyMock.expect(storeMetadata.store()).andReturn(store).anyTimes();
-        EasyMock.expect(store.name()).andReturn(storeName).anyTimes();
+    private void setupStateManagerMock() {
+        when(stateManager.storeMetadata(tp)).thenReturn(storeMetadata);
+        when(stateManager.taskType()).thenReturn(type);
     }
 
-    @After
-    public void tearDown() {
-        EasyMock.reset(
-            stateManager,
-            activeStateManager,
-            standbyStateManager,
-            storeMetadata,
-            storeMetadataOne,
-            storeMetadataTwo,
-            store
-        );
+    private void setupActiveStateManager() {
+        when(activeStateManager.storeMetadata(tp)).thenReturn(storeMetadata);
+        when(activeStateManager.taskType()).thenReturn(ACTIVE);
+    }
+
+    private void setupStandbyStateManager() {
+        when(standbyStateManager.storeMetadata(tp)).thenReturn(storeMetadata);
+        when(standbyStateManager.taskType()).thenReturn(STANDBY);
+    }
+
+    private void setupStoreMetadata() {
+        when(storeMetadata.changelogPartition()).thenReturn(tp);
+        when(storeMetadata.store()).thenReturn(store);
+    }
+
+    private void setupStore() {
+        when(store.name()).thenReturn(storeName);
     }
 
     @Test
     public void shouldNotRegisterSameStoreMultipleTimes() {
-        EasyMock.replay(stateManager, storeMetadata);
+        setupStateManagerMock();
 
         changelogReader.register(tp, stateManager);
 
@@ -200,19 +195,13 @@ public class StoreChangelogReaderTest extends EasyMockSupport {
 
     @Test
     public void shouldNotRegisterStoreWithoutMetadata() {
-        EasyMock.replay(stateManager, storeMetadata);
-
         assertThrows(IllegalStateException.class,
             () -> changelogReader.register(new TopicPartition("ChangelogWithoutStoreMetadata", 0), stateManager));
     }
 
     @Test
     public void shouldSupportUnregisterChangelogBeforeInitialization() {
-        final Map<TaskId, Task> mockTasks = mock(Map.class);
-        EasyMock.expect(mockTasks.get(null)).andReturn(mock(Task.class)).anyTimes();
-        EasyMock.expect(storeMetadata.offset()).andReturn(9L).anyTimes();
-        EasyMock.expect(stateManager.changelogOffsets()).andReturn(singletonMap(tp, 10L));
-        EasyMock.replay(mockTasks, stateManager, storeMetadata, store);
+        setupStateManagerMock();
 
         adminClient.updateEndOffsets(Collections.singletonMap(tp, 100L));
 
@@ -241,16 +230,19 @@ public class StoreChangelogReaderTest extends EasyMockSupport {
 
     @Test
     public void shouldSupportUnregisterChangelogBeforeCompletion() {
+        setupStateManagerMock();
+        setupStoreMetadata();
+        setupStore();
+        @SuppressWarnings("unchecked")
         final Map<TaskId, Task> mockTasks = mock(Map.class);
-        EasyMock.expect(mockTasks.get(null)).andReturn(mock(Task.class)).anyTimes();
-        EasyMock.expect(mockTasks.containsKey(null)).andReturn(true).anyTimes();
-        EasyMock.expect(storeMetadata.offset()).andReturn(9L).anyTimes();
-        EasyMock.expect(storeMetadata.endOffset()).andReturn(10L).anyTimes();
-        EasyMock.expect(stateManager.changelogOffsets()).andReturn(singletonMap(tp, 10L));
+        when(mockTasks.get(null)).thenReturn(mock(Task.class));
+        when(mockTasks.containsKey(null)).thenReturn(true);
+        when(storeMetadata.offset()).thenReturn(9L);
         if (type == STANDBY) {
-            EasyMock.expect(stateManager.changelogAsSource(tp)).andReturn(true).anyTimes();
+            when(storeMetadata.endOffset()).thenReturn(10L);
+            when(stateManager.changelogAsSource(tp)).thenReturn(true);
         }
-        EasyMock.replay(mockTasks, stateManager, storeMetadata, store);
+
         adminClient.updateEndOffsets(Collections.singletonMap(tp, 100L));
 
         final StoreChangelogReader changelogReader =
@@ -290,16 +282,18 @@ public class StoreChangelogReaderTest extends EasyMockSupport {
 
     @Test
     public void shouldSupportUnregisterChangelogAfterCompletion() {
+        setupStateManagerMock();
+        setupStoreMetadata();
+        setupStore();
+        @SuppressWarnings("unchecked")
         final Map<TaskId, Task> mockTasks = mock(Map.class);
-        EasyMock.expect(mockTasks.get(null)).andReturn(mock(Task.class)).anyTimes();
-        EasyMock.expect(mockTasks.containsKey(null)).andReturn(true).anyTimes();
-        EasyMock.expect(storeMetadata.offset()).andReturn(9L).anyTimes();
-        EasyMock.expect(storeMetadata.endOffset()).andReturn(10L).anyTimes();
-        EasyMock.expect(stateManager.changelogOffsets()).andReturn(singletonMap(tp, 10L));
+        when(mockTasks.get(null)).thenReturn(mock(Task.class));
+        when(mockTasks.containsKey(null)).thenReturn(true);
+        when(storeMetadata.offset()).thenReturn(9L);
         if (type == STANDBY) {
-            EasyMock.expect(stateManager.changelogAsSource(tp)).andReturn(true).anyTimes();
+            when(storeMetadata.endOffset()).thenReturn(10L);
+            when(stateManager.changelogAsSource(tp)).thenReturn(true);
         }
-        EasyMock.replay(mockTasks, stateManager, storeMetadata, store);
 
         adminClient.updateEndOffsets(Collections.singletonMap(tp, 10L));
 
@@ -346,12 +340,14 @@ public class StoreChangelogReaderTest extends EasyMockSupport {
 
     @Test
     public void shouldInitializeChangelogAndCheckForCompletion() {
+        setupStateManagerMock();
+        setupStoreMetadata();
+        setupStore();
+        @SuppressWarnings("unchecked")
         final Map<TaskId, Task> mockTasks = mock(Map.class);
-        EasyMock.expect(mockTasks.get(null)).andReturn(mock(Task.class)).anyTimes();
-        EasyMock.expect(mockTasks.containsKey(null)).andReturn(true).anyTimes();
-        EasyMock.expect(storeMetadata.offset()).andReturn(9L).anyTimes();
-        EasyMock.expect(storeMetadata.endOffset()).andReturn(10L).anyTimes();
-        EasyMock.replay(mockTasks, stateManager, storeMetadata, store);
+        when(mockTasks.get(null)).thenReturn(mock(Task.class));
+        when(mockTasks.containsKey(null)).thenReturn(true);
+        when(storeMetadata.offset()).thenReturn(9L);
 
         adminClient.updateEndOffsets(Collections.singletonMap(tp, 10L));
 
@@ -388,11 +384,13 @@ public class StoreChangelogReaderTest extends EasyMockSupport {
     public void shouldTriggerRestoreListenerWithOffsetZeroIfPositionThrowsTimeoutException() {
         // restore listener is only triggered for active tasks
         if (type == ACTIVE) {
+            setupStateManagerMock();
+            setupStoreMetadata();
+            @SuppressWarnings("unchecked")
             final Map<TaskId, Task> mockTasks = mock(Map.class);
-            EasyMock.expect(mockTasks.get(null)).andReturn(mock(Task.class)).anyTimes();
-            EasyMock.expect(mockTasks.containsKey(null)).andReturn(true).anyTimes();
-            EasyMock.expect(stateManager.changelogOffsets()).andReturn(singletonMap(tp, 5L));
-            EasyMock.replay(mockTasks, stateManager, storeMetadata, store);
+            when(mockTasks.get(null)).thenReturn(mock(Task.class));
+            when(mockTasks.containsKey(null)).thenReturn(true);
+            when(stateManager.changelogOffsets()).thenReturn(singletonMap(tp, 5L));
 
             adminClient.updateEndOffsets(Collections.singletonMap(tp, 10L));
 
@@ -416,11 +414,17 @@ public class StoreChangelogReaderTest extends EasyMockSupport {
 
     @Test
     public void shouldPollWithRightTimeoutWithStateUpdater() {
+        setupStateManagerMock();
+        setupStoreMetadata();
+        setupStore();
         shouldPollWithRightTimeout(true);
     }
 
     @Test
     public void shouldPollWithRightTimeoutWithoutStateUpdater() {
+        setupStateManagerMock();
+        setupStoreMetadata();
+        setupStore();
         shouldPollWithRightTimeout(false);
     }
 
@@ -432,6 +436,9 @@ public class StoreChangelogReaderTest extends EasyMockSupport {
 
     @Test
     public void shouldPollWithRightTimeoutWithStateUpdaterDefault() {
+        setupStateManagerMock();
+        setupStoreMetadata();
+        setupStore();
         final Properties properties = new Properties();
         shouldPollWithRightTimeout(properties);
     }
@@ -439,11 +446,8 @@ public class StoreChangelogReaderTest extends EasyMockSupport {
     private void shouldPollWithRightTimeout(final Properties properties) {
         final TaskId taskId = new TaskId(0, 0);
 
-        EasyMock.expect(storeMetadata.offset()).andReturn(null).andReturn(9L).anyTimes();
-        EasyMock.expect(storeMetadata.endOffset()).andReturn(10L).anyTimes();
-        EasyMock.expect(stateManager.changelogOffsets()).andReturn(singletonMap(tp, 5L));
-        EasyMock.expect(stateManager.taskId()).andReturn(taskId).anyTimes();
-        EasyMock.replay(stateManager, storeMetadata, store);
+        when(storeMetadata.offset()).thenReturn(null).thenReturn(9L);
+        when(stateManager.taskId()).thenReturn(taskId);
 
         consumer.updateBeginningOffsets(Collections.singletonMap(tp, 5L));
         adminClient.updateEndOffsets(Collections.singletonMap(tp, 11L));
@@ -464,23 +468,26 @@ public class StoreChangelogReaderTest extends EasyMockSupport {
             assertEquals(Duration.ofMillis(config.getLong(StreamsConfig.POLL_MS_CONFIG)), consumer.lastPollTimeout());
         } else {
             if (!properties.containsKey(InternalConfig.STATE_UPDATER_ENABLED)
-                    || !((boolean) properties.get(InternalConfig.STATE_UPDATER_ENABLED))) {
-                assertEquals(Duration.ZERO, consumer.lastPollTimeout());
-            } else {
+                    || (boolean) properties.get(InternalConfig.STATE_UPDATER_ENABLED)) {
                 assertEquals(Duration.ofMillis(config.getLong(StreamsConfig.POLL_MS_CONFIG)), consumer.lastPollTimeout());
+            } else {
+                assertEquals(Duration.ZERO, consumer.lastPollTimeout());
             }
         }
     }
 
     @Test
     public void shouldRestoreFromPositionAndCheckForCompletion() {
+        setupStateManagerMock();
+        setupStoreMetadata();
+        setupStore();
         final TaskId taskId = new TaskId(0, 0);
 
-        EasyMock.expect(storeMetadata.offset()).andReturn(5L).anyTimes();
-        EasyMock.expect(storeMetadata.endOffset()).andReturn(10L).anyTimes();
-        EasyMock.expect(stateManager.changelogOffsets()).andReturn(singletonMap(tp, 5L));
-        EasyMock.expect(stateManager.taskId()).andReturn(taskId).anyTimes();
-        EasyMock.replay(stateManager, storeMetadata, store);
+        when(storeMetadata.offset()).thenReturn(5L);
+        if (type == STANDBY) {
+            when(storeMetadata.endOffset()).thenReturn(10L);
+        }
+        when(stateManager.taskId()).thenReturn(taskId);
 
         adminClient.updateEndOffsets(Collections.singletonMap(tp, 10L));
 
@@ -544,18 +551,18 @@ public class StoreChangelogReaderTest extends EasyMockSupport {
 
     @Test
     public void shouldRestoreFromBeginningAndCheckCompletion() {
+        setupStateManagerMock();
+        setupStoreMetadata();
+        setupStore();
         final TaskId taskId = new TaskId(0, 0);
 
         if (type == STANDBY && logContext.logger(StoreChangelogReader.class).isDebugEnabled()) {
-            EasyMock.expect(storeMetadata.offset()).andReturn(null).andReturn(null).andReturn(9L).anyTimes();
-            EasyMock.expect(storeMetadata.endOffset()).andReturn(10L).anyTimes();
+            when(storeMetadata.offset()).thenReturn(null).thenReturn(null).thenReturn(9L);
+            when(storeMetadata.endOffset()).thenReturn(10L);
         } else {
-            EasyMock.expect(storeMetadata.offset()).andReturn(null).andReturn(9L).anyTimes();
-            EasyMock.expect(storeMetadata.endOffset()).andReturn(10L).anyTimes();
+            when(storeMetadata.offset()).thenReturn(null).thenReturn(9L);
         }
-        EasyMock.expect(stateManager.changelogOffsets()).andReturn(singletonMap(tp, 5L));
-        EasyMock.expect(stateManager.taskId()).andReturn(taskId).anyTimes();
-        EasyMock.replay(stateManager, storeMetadata, store);
+        when(stateManager.taskId()).thenReturn(taskId);
 
         consumer.updateBeginningOffsets(Collections.singletonMap(tp, 5L));
         adminClient.updateEndOffsets(Collections.singletonMap(tp, 11L));
@@ -625,11 +632,14 @@ public class StoreChangelogReaderTest extends EasyMockSupport {
 
     @Test
     public void shouldCheckCompletionIfPositionLargerThanEndOffset() {
+        setupActiveStateManager();
+        setupStoreMetadata();
+        setupStore();
+        @SuppressWarnings("unchecked")
         final Map<TaskId, Task> mockTasks = mock(Map.class);
-        EasyMock.expect(mockTasks.get(null)).andReturn(mock(Task.class)).anyTimes();
-        EasyMock.expect(mockTasks.containsKey(null)).andReturn(true).anyTimes();
-        EasyMock.expect(storeMetadata.offset()).andReturn(5L).anyTimes();
-        EasyMock.replay(mockTasks, activeStateManager, storeMetadata, store);
+        when(mockTasks.get(null)).thenReturn(mock(Task.class));
+        when(mockTasks.containsKey(null)).thenReturn(true);
+        when(storeMetadata.offset()).thenReturn(5L);
 
         adminClient.updateEndOffsets(Collections.singletonMap(tp, 0L));
 
@@ -653,18 +663,15 @@ public class StoreChangelogReaderTest extends EasyMockSupport {
 
     @Test
     public void shouldRequestPositionAndHandleTimeoutException() {
+        setupActiveStateManager();
+        setupStoreMetadata();
+        setupStore();
         final TaskId taskId = new TaskId(0, 0);
 
         final Task mockTask = mock(Task.class);
-        mockTask.clearTaskTimeout();
-        mockTask.maybeInitTaskTimeoutOrThrow(anyLong(), anyObject());
-        EasyMock.expectLastCall();
-        mockTask.recordRestoration(anyObject(), anyLong(), anyBoolean());
-        EasyMock.expectLastCall();
-        EasyMock.expect(storeMetadata.offset()).andReturn(10L).anyTimes();
-        EasyMock.expect(activeStateManager.changelogOffsets()).andReturn(singletonMap(tp, 10L));
-        EasyMock.expect(activeStateManager.taskId()).andReturn(taskId).anyTimes();
-        EasyMock.replay(mockTask, activeStateManager, storeMetadata, store);
+        when(storeMetadata.offset()).thenReturn(10L);
+        when(activeStateManager.changelogOffsets()).thenReturn(singletonMap(tp, 10L));
+        when(activeStateManager.taskId()).thenReturn(taskId);
 
         final AtomicBoolean clearException = new AtomicBoolean(false);
         final MockConsumer<byte[], byte[]> consumer = new MockConsumer<byte[], byte[]>(OffsetResetStrategy.EARLIEST) {
@@ -689,28 +696,29 @@ public class StoreChangelogReaderTest extends EasyMockSupport {
         assertEquals(StoreChangelogReader.ChangelogState.RESTORING, changelogReader.changelogMetadata(tp).state());
         assertTrue(changelogReader.completedChangelogs().isEmpty());
         assertEquals(10L, (long) changelogReader.changelogMetadata(tp).endOffset());
-        verify(mockTask);
+        Mockito.verify(mockTask).clearTaskTimeout();
+        Mockito.verify(mockTask).maybeInitTaskTimeoutOrThrow(anyLong(), any());
+        Mockito.verify(mockTask).recordRestoration(any(), anyLong(), anyBoolean());
 
         clearException.set(true);
-        resetToDefault(mockTask);
-        mockTask.clearTaskTimeout();
-        EasyMock.expectLastCall();
-        EasyMock.replay(mockTask);
+        Mockito.reset(mockTask);
         changelogReader.restore(Collections.singletonMap(taskId, mockTask));
 
         assertEquals(StoreChangelogReader.ChangelogState.COMPLETED, changelogReader.changelogMetadata(tp).state());
         assertEquals(10L, (long) changelogReader.changelogMetadata(tp).endOffset());
         assertEquals(Collections.singleton(tp), changelogReader.completedChangelogs());
         assertEquals(10L, consumer.position(tp));
-        verify(mockTask);
+        Mockito.verify(mockTask).clearTaskTimeout();
     }
 
     @Test
     public void shouldThrowIfPositionFail() {
+        setupActiveStateManager();
+        setupStoreMetadata();
+        setupStore();
         final TaskId taskId = new TaskId(0, 0);
-        EasyMock.expect(activeStateManager.taskId()).andReturn(taskId);
-        EasyMock.expect(storeMetadata.offset()).andReturn(10L).anyTimes();
-        EasyMock.replay(activeStateManager, storeMetadata, store);
+        when(activeStateManager.taskId()).thenReturn(taskId);
+        when(storeMetadata.offset()).thenReturn(10L);
 
         final MockConsumer<byte[], byte[]> consumer = new MockConsumer<byte[], byte[]>(OffsetResetStrategy.EARLIEST) {
             @Override
@@ -735,16 +743,16 @@ public class StoreChangelogReaderTest extends EasyMockSupport {
 
     @Test
     public void shouldRequestEndOffsetsAndHandleTimeoutException() {
+        setupActiveStateManager();
+        setupStoreMetadata();
+        setupStore();
         final TaskId taskId = new TaskId(0, 0);
 
-        final Task mockTask = niceMock(Task.class);
-        mockTask.maybeInitTaskTimeoutOrThrow(anyLong(), anyObject());
-        EasyMock.expectLastCall();
+        final Task mockTask = mock(Task.class);
 
-        EasyMock.expect(storeMetadata.offset()).andReturn(5L).anyTimes();
-        EasyMock.expect(activeStateManager.changelogOffsets()).andReturn(singletonMap(tp, 5L));
-        EasyMock.expect(activeStateManager.taskId()).andReturn(taskId).anyTimes();
-        EasyMock.replay(mockTask, activeStateManager, storeMetadata, store);
+        when(storeMetadata.offset()).thenReturn(5L);
+        when(activeStateManager.changelogOffsets()).thenReturn(singletonMap(tp, 5L));
+        when(activeStateManager.taskId()).thenReturn(taskId);
 
         final AtomicBoolean functionCalled = new AtomicBoolean(false);
 
@@ -778,29 +786,26 @@ public class StoreChangelogReaderTest extends EasyMockSupport {
         assertEquals(StoreChangelogReader.ChangelogState.REGISTERED, changelogReader.changelogMetadata(tp).state());
         assertNull(changelogReader.changelogMetadata(tp).endOffset());
         assertTrue(functionCalled.get());
-        verify(mockTask);
+        Mockito.verify(mockTask).maybeInitTaskTimeoutOrThrow(anyLong(), any());
 
-        EasyMock.resetToDefault(mockTask);
-        mockTask.clearTaskTimeout();
-        mockTask.recordRestoration(anyObject(), anyLong(), anyBoolean());
-        EasyMock.expectLastCall();
-        EasyMock.replay(mockTask);
+        Mockito.reset(mockTask);
 
         changelogReader.restore(Collections.singletonMap(taskId, mockTask));
 
         assertEquals(StoreChangelogReader.ChangelogState.RESTORING, changelogReader.changelogMetadata(tp).state());
         assertEquals(10L, (long) changelogReader.changelogMetadata(tp).endOffset());
         assertEquals(6L, consumer.position(tp));
-        verify(mockTask);
+        Mockito.verify(mockTask).clearTaskTimeout();
+        Mockito.verify(mockTask).recordRestoration(any(), anyLong(), anyBoolean());
     }
 
     @Test
     public void shouldThrowIfEndOffsetsFail() {
+        setupActiveStateManager();
+        when(storeMetadata.changelogPartition()).thenReturn(tp);
         final TaskId taskId = new TaskId(0, 0);
 
-        EasyMock.expect(storeMetadata.offset()).andReturn(10L).anyTimes();
-        EasyMock.expect(activeStateManager.taskId()).andReturn(taskId).anyTimes();
-        EasyMock.replay(activeStateManager, storeMetadata, store);
+        when(activeStateManager.taskId()).thenReturn(taskId);
 
         final MockAdminClient adminClient = new MockAdminClient() {
             @Override
@@ -825,21 +830,20 @@ public class StoreChangelogReaderTest extends EasyMockSupport {
 
     @Test
     public void shouldRequestCommittedOffsetsAndHandleTimeoutException() {
+        setupStateManagerMock();
+        setupStoreMetadata();
+        setupStore();
+
         final TaskId taskId = new TaskId(0, 0);
 
         final Task mockTask = mock(Task.class);
         if (type == ACTIVE) {
             mockTask.clearTaskTimeout();
         }
-        mockTask.maybeInitTaskTimeoutOrThrow(anyLong(), anyObject());
-        EasyMock.expectLastCall();
 
-        EasyMock.expect(stateManager.changelogAsSource(tp)).andReturn(true).anyTimes();
-        EasyMock.expect(storeMetadata.offset()).andReturn(5L).anyTimes();
-        EasyMock.expect(storeMetadata.endOffset()).andReturn(10L).anyTimes();
-        EasyMock.expect(stateManager.changelogOffsets()).andReturn(singletonMap(tp, 5L));
-        EasyMock.expect(stateManager.taskId()).andReturn(taskId).anyTimes();
-        EasyMock.replay(mockTask, stateManager, storeMetadata, store);
+        when(stateManager.changelogAsSource(tp)).thenReturn(true);
+        when(storeMetadata.offset()).thenReturn(5L);
+        when(stateManager.taskId()).thenReturn(taskId);
 
         final AtomicBoolean functionCalled = new AtomicBoolean(false);
         final MockAdminClient adminClient = new MockAdminClient() {
@@ -875,34 +879,30 @@ public class StoreChangelogReaderTest extends EasyMockSupport {
             assertEquals(0L, (long) changelogReader.changelogMetadata(tp).endOffset());
         }
         assertTrue(functionCalled.get());
-        verify(mockTask);
+        Mockito.verify(mockTask).maybeInitTaskTimeoutOrThrow(anyLong(), any());
 
-        resetToDefault(mockTask);
-        if (type == ACTIVE) {
-            mockTask.clearTaskTimeout();
-            mockTask.clearTaskTimeout();
-            expectLastCall();
-            mockTask.recordRestoration(anyObject(), anyLong(), anyBoolean());
-            expectLastCall();
-        }
-        replay(mockTask);
+        Mockito.reset(mockTask);
 
         changelogReader.restore(Collections.singletonMap(taskId, mockTask));
 
         assertEquals(StoreChangelogReader.ChangelogState.RESTORING, changelogReader.changelogMetadata(tp).state());
         assertEquals(type == ACTIVE ? 10L : 0L, (long) changelogReader.changelogMetadata(tp).endOffset());
         assertEquals(6L, consumer.position(tp));
-        verify(mockTask);
+        if (type == ACTIVE) {
+            Mockito.verify(mockTask, times(2)).clearTaskTimeout();
+            Mockito.verify(mockTask).recordRestoration(any(), anyLong(), anyBoolean());
+        }
     }
 
     @Test
     public void shouldThrowIfCommittedOffsetsFail() {
+        setupStateManagerMock();
+        when(storeMetadata.changelogPartition()).thenReturn(tp);
+
         final TaskId taskId = new TaskId(0, 0);
 
-        EasyMock.expect(stateManager.taskId()).andReturn(taskId);
-        EasyMock.expect(stateManager.changelogAsSource(tp)).andReturn(true).anyTimes();
-        EasyMock.expect(storeMetadata.offset()).andReturn(10L).anyTimes();
-        EasyMock.replay(stateManager, storeMetadata, store);
+        when(stateManager.taskId()).thenReturn(taskId);
+        when(stateManager.changelogAsSource(tp)).thenReturn(true);
 
         final MockAdminClient adminClient = new MockAdminClient() {
             @Override
@@ -926,8 +926,6 @@ public class StoreChangelogReaderTest extends EasyMockSupport {
 
     @Test
     public void shouldThrowIfUnsubscribeFail() {
-        EasyMock.replay(stateManager, storeMetadata, store);
-
         final MockConsumer<byte[], byte[]> consumer = new MockConsumer<byte[], byte[]>(OffsetResetStrategy.EARLIEST) {
             @Override
             public void unsubscribe() {
@@ -943,13 +941,15 @@ public class StoreChangelogReaderTest extends EasyMockSupport {
 
     @Test
     public void shouldOnlyRestoreStandbyChangelogInUpdateStandbyState() {
+        setupStandbyStateManager();
+        setupStoreMetadata();
+        setupStore();
+        @SuppressWarnings("unchecked")
         final Map<TaskId, Task> mockTasks = mock(Map.class);
-        EasyMock.expect(mockTasks.get(null)).andReturn(mock(Task.class)).anyTimes();
-        EasyMock.expect(mockTasks.containsKey(null)).andReturn(true).anyTimes();
-        EasyMock.expect(stateManager.taskType()).andReturn(type);
-        EasyMock.expect(storeMetadata.offset()).andReturn(3L).anyTimes();
-        EasyMock.expect(storeMetadata.endOffset()).andReturn(20L).anyTimes();
-        EasyMock.replay(mockTasks, standbyStateManager, storeMetadata, store);
+        when(mockTasks.get(null)).thenReturn(mock(Task.class));
+        when(mockTasks.containsKey(null)).thenReturn(true);
+        when(storeMetadata.offset()).thenReturn(3L);
+        when(storeMetadata.endOffset()).thenReturn(20L);
 
         consumer.updateBeginningOffsets(Collections.singletonMap(tp, 0L));
         changelogReader.register(tp, standbyStateManager);
@@ -985,15 +985,16 @@ public class StoreChangelogReaderTest extends EasyMockSupport {
 
     @Test
     public void shouldNotUpdateLimitForNonSourceStandbyChangelog() {
+        setupStandbyStateManager();
+        setupStoreMetadata();
+        setupStore();
+        @SuppressWarnings("unchecked")
         final Map<TaskId, Task> mockTasks = mock(Map.class);
-        //EasyMock.expect(storeMetadata.offset()).andReturn(0L).anyTimes();
-        EasyMock.expect(mockTasks.get(null)).andReturn(mock(Task.class)).anyTimes();
-        EasyMock.expect(mockTasks.containsKey(null)).andReturn(true).anyTimes();
-        EasyMock.expect(stateManager.taskType()).andReturn(type);
-        EasyMock.expect(storeMetadata.offset()).andReturn(3L).anyTimes();
-        EasyMock.expect(storeMetadata.endOffset()).andReturn(20L).anyTimes();
-        EasyMock.expect(standbyStateManager.changelogAsSource(tp)).andReturn(false).anyTimes();
-        EasyMock.replay(mockTasks, standbyStateManager, storeMetadata, store);
+        when(mockTasks.get(null)).thenReturn(mock(Task.class));
+        when(mockTasks.containsKey(null)).thenReturn(true);
+        when(storeMetadata.offset()).thenReturn(3L);
+        when(storeMetadata.endOffset()).thenReturn(20L);
+        when(standbyStateManager.changelogAsSource(tp)).thenReturn(false);
 
         final MockAdminClient adminClient = new MockAdminClient() {
             @Override
@@ -1043,14 +1044,16 @@ public class StoreChangelogReaderTest extends EasyMockSupport {
 
     @Test
     public void shouldRestoreToLimitInStandbyState() {
+        setupStandbyStateManager();
+        setupStoreMetadata();
+        setupStore();
+        @SuppressWarnings("unchecked")
         final Map<TaskId, Task> mockTasks = mock(Map.class);
-        EasyMock.expect(mockTasks.get(null)).andReturn(mock(Task.class)).anyTimes();
-        EasyMock.expect(mockTasks.containsKey(null)).andReturn(true).anyTimes();
-        EasyMock.expect(standbyStateManager.changelogAsSource(tp)).andReturn(true).anyTimes();
-        EasyMock.expect(stateManager.taskType()).andReturn(type);
-        EasyMock.expect(storeMetadata.offset()).andReturn(3L).anyTimes();
-        EasyMock.expect(storeMetadata.endOffset()).andReturn(20L).anyTimes();
-        EasyMock.replay(mockTasks, standbyStateManager, storeMetadata, store);
+        when(mockTasks.get(null)).thenReturn(mock(Task.class));
+        when(mockTasks.containsKey(null)).thenReturn(true);
+        when(standbyStateManager.changelogAsSource(tp)).thenReturn(true);
+        when(storeMetadata.offset()).thenReturn(3L);
+        when(storeMetadata.endOffset()).thenReturn(20L);
 
         final long now = time.milliseconds();
         final Properties properties = new Properties();
@@ -1159,24 +1162,27 @@ public class StoreChangelogReaderTest extends EasyMockSupport {
 
     @Test
     public void shouldRestoreMultipleChangelogs() {
+        setupActiveStateManager();
+        setupStoreMetadata();
+        setupStore();
+        @SuppressWarnings("unchecked")
         final Map<TaskId, Task> mockTasks = mock(Map.class);
-        EasyMock.expect(mockTasks.get(null)).andReturn(mock(Task.class)).anyTimes();
-        EasyMock.expect(mockTasks.containsKey(null)).andReturn(true).anyTimes();
-        EasyMock.expect(storeMetadataOne.changelogPartition()).andReturn(tp1).anyTimes();
-        EasyMock.expect(storeMetadataOne.store()).andReturn(store).anyTimes();
-        EasyMock.expect(storeMetadataTwo.changelogPartition()).andReturn(tp2).anyTimes();
-        EasyMock.expect(storeMetadataTwo.store()).andReturn(store).anyTimes();
-        EasyMock.expect(storeMetadata.offset()).andReturn(0L).anyTimes();
-        EasyMock.expect(storeMetadataOne.offset()).andReturn(0L).anyTimes();
-        EasyMock.expect(storeMetadataTwo.offset()).andReturn(0L).anyTimes();
-        EasyMock.expect(activeStateManager.storeMetadata(tp1)).andReturn(storeMetadataOne).anyTimes();
-        EasyMock.expect(activeStateManager.storeMetadata(tp2)).andReturn(storeMetadataTwo).anyTimes();
-        EasyMock.expect(activeStateManager.changelogOffsets()).andReturn(mkMap(
+        when(mockTasks.get(null)).thenReturn(mock(Task.class));
+        when(mockTasks.containsKey(null)).thenReturn(true);
+        when(storeMetadataOne.changelogPartition()).thenReturn(tp1);
+        when(storeMetadataOne.store()).thenReturn(store);
+        when(storeMetadataTwo.changelogPartition()).thenReturn(tp2);
+        when(storeMetadataTwo.store()).thenReturn(store);
+        when(storeMetadata.offset()).thenReturn(0L);
+        when(storeMetadataOne.offset()).thenReturn(0L);
+        when(storeMetadataTwo.offset()).thenReturn(0L);
+        when(activeStateManager.storeMetadata(tp1)).thenReturn(storeMetadataOne);
+        when(activeStateManager.storeMetadata(tp2)).thenReturn(storeMetadataTwo);
+        when(activeStateManager.changelogOffsets()).thenReturn(mkMap(
             mkEntry(tp, 5L),
             mkEntry(tp1, 5L),
             mkEntry(tp2, 5L)
-        )).anyTimes();
-        EasyMock.replay(mockTasks, activeStateManager, storeMetadata, store, storeMetadataOne, storeMetadataTwo);
+        ));
 
         setupConsumer(10, tp);
         setupConsumer(5, tp1);
@@ -1208,20 +1214,23 @@ public class StoreChangelogReaderTest extends EasyMockSupport {
 
     @Test
     public void shouldTransitState() {
+        setupActiveStateManager();
+        setupStoreMetadata();
+        setupStore();
+        when(standbyStateManager.taskType()).thenReturn(STANDBY);
         final TaskId taskId = new TaskId(0, 0);
-        EasyMock.expect(storeMetadataOne.changelogPartition()).andReturn(tp1).anyTimes();
-        EasyMock.expect(storeMetadataOne.store()).andReturn(store).anyTimes();
-        EasyMock.expect(storeMetadataTwo.changelogPartition()).andReturn(tp2).anyTimes();
-        EasyMock.expect(storeMetadataTwo.store()).andReturn(store).anyTimes();
-        EasyMock.expect(storeMetadata.offset()).andReturn(5L).anyTimes();
-        EasyMock.expect(storeMetadataOne.offset()).andReturn(5L).anyTimes();
-        EasyMock.expect(storeMetadataTwo.offset()).andReturn(5L).anyTimes();
-        EasyMock.expect(standbyStateManager.storeMetadata(tp1)).andReturn(storeMetadataOne).anyTimes();
-        EasyMock.expect(standbyStateManager.storeMetadata(tp2)).andReturn(storeMetadataTwo).anyTimes();
-        EasyMock.expect(activeStateManager.changelogOffsets()).andReturn(singletonMap(tp, 5L));
-        EasyMock.expect(activeStateManager.taskId()).andReturn(taskId).anyTimes();
-        EasyMock.expect(standbyStateManager.taskId()).andReturn(taskId).anyTimes();
-        EasyMock.replay(activeStateManager, standbyStateManager, storeMetadata, store, storeMetadataOne, storeMetadataTwo);
+        when(storeMetadataOne.changelogPartition()).thenReturn(tp1);
+        when(storeMetadataOne.store()).thenReturn(store);
+        when(storeMetadataTwo.changelogPartition()).thenReturn(tp2);
+        when(storeMetadataTwo.store()).thenReturn(store);
+        when(storeMetadata.offset()).thenReturn(5L);
+        when(storeMetadataOne.offset()).thenReturn(5L);
+        when(storeMetadataTwo.offset()).thenReturn(5L);
+        when(standbyStateManager.storeMetadata(tp1)).thenReturn(storeMetadataOne);
+        when(standbyStateManager.storeMetadata(tp2)).thenReturn(storeMetadataTwo);
+        when(activeStateManager.changelogOffsets()).thenReturn(singletonMap(tp, 5L));
+        when(activeStateManager.taskId()).thenReturn(taskId);
+        when(standbyStateManager.taskId()).thenReturn(taskId);
 
         adminClient.updateEndOffsets(Collections.singletonMap(tp, 10L));
         adminClient.updateEndOffsets(Collections.singletonMap(tp1, 10L));
@@ -1287,11 +1296,9 @@ public class StoreChangelogReaderTest extends EasyMockSupport {
 
     @Test
     public void shouldTransitStateBackToActiveRestoringAfterRemovingLastTask() {
+        when(standbyStateManager.taskType()).thenReturn(STANDBY);
         final StoreChangelogReader changelogReader = new StoreChangelogReader(time, config, logContext, adminClient, consumer, callback, standbyListener);
-        EasyMock.expect(standbyStateManager.storeMetadata(tp1)).andReturn(storeMetadataOne).anyTimes();
-        EasyMock.expect(storeMetadataOne.changelogPartition()).andReturn(tp1).anyTimes();
-        EasyMock.expect(storeMetadataOne.store()).andReturn(store).anyTimes();
-        EasyMock.replay(standbyStateManager, store, storeMetadataOne);
+        when(standbyStateManager.storeMetadata(tp1)).thenReturn(storeMetadataOne);
         changelogReader.register(tp1, standbyStateManager);
         changelogReader.transitToUpdateStandby();
 
@@ -1302,11 +1309,13 @@ public class StoreChangelogReaderTest extends EasyMockSupport {
 
     @Test
     public void shouldThrowIfRestoreCallbackThrows() {
+        setupActiveStateManager();
+        setupStoreMetadata();
+        setupStore();
         final TaskId taskId = new TaskId(0, 0);
 
-        EasyMock.expect(storeMetadata.offset()).andReturn(5L).anyTimes();
-        EasyMock.expect(activeStateManager.taskId()).andReturn(taskId).anyTimes();
-        EasyMock.replay(activeStateManager, storeMetadata, store);
+        when(storeMetadata.offset()).thenReturn(5L);
+        when(activeStateManager.taskId()).thenReturn(taskId);
 
         adminClient.updateEndOffsets(Collections.singletonMap(tp, 10L));
 
