@@ -1,10 +1,10 @@
-/**
+/*
  * Licensed to the Apache Software Foundation (ASF) under one or more
- * contributor license agreements.  See the NOTICE file distributed with
+ * contributor license agreements. See the NOTICE file distributed with
  * this work for additional information regarding copyright ownership.
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
- * the License.  You may obtain a copy of the License at
+ * the License. You may obtain a copy of the License at
  *
  *    http://www.apache.org/licenses/LICENSE-2.0
  *
@@ -14,7 +14,6 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-
 package org.apache.kafka.test;
 
 import org.apache.kafka.streams.kstream.ValueJoiner;
@@ -26,7 +25,7 @@ public class MockValueJoiner {
     public static <V1, V2> ValueJoiner<V1, V2, String> instance(final String separator) {
         return new ValueJoiner<V1, V2, String>() {
             @Override
-            public String apply(V1 value1, V2 value2) {
+            public String apply(final V1 value1, final V2 value2) {
                 return value1 + separator + value2;
             }
         };
