@@ -17,6 +17,7 @@ from ducktape.services.background_thread import BackgroundThreadService
 
 from kafkatest.directory_layout.kafka_path import KafkaPathResolverMixin
 from kafkatest.services.security.security_config import SecurityConfig
+from kafkatest.services.kafka.util import fix_opts_for_new_jvm
 
 
 class KafkaLog4jAppender(KafkaPathResolverMixin, BackgroundThreadService):
@@ -27,16 +28,19 @@ class KafkaLog4jAppender(KafkaPathResolverMixin, BackgroundThreadService):
             "collect_default": False}
     }
 
-    def __init__(self, context, num_nodes, kafka, topic, max_messages=-1, security_protocol="PLAINTEXT"):
+    def __init__(self, context, num_nodes, kafka, topic, max_messages=-1, security_protocol="PLAINTEXT", tls_version=None):
         super(KafkaLog4jAppender, self).__init__(context, num_nodes)
 
         self.kafka = kafka
         self.topic = topic
         self.max_messages = max_messages
         self.security_protocol = security_protocol
-        self.security_config = SecurityConfig(self.context, security_protocol)
+        self.security_config = SecurityConfig(self.context, security_protocol, tls_version=tls_version)
         self.stop_timeout_sec = 30
 
+        for node in self.nodes:
+            node.version = kafka.nodes[0].version
+
     def _worker(self, idx, node):
         cmd = self.start_cmd(node)
         self.logger.debug("VerifiableLog4jAppender %d command: %s" % (idx, cmd))
@@ -44,7 +48,8 @@ class KafkaLog4jAppender(KafkaPathResolverMixin, BackgroundThreadService):
         node.account.ssh(cmd)
 
     def start_cmd(self, node):
-        cmd = self.path.script("kafka-run-class.sh", node)
+        cmd = fix_opts_for_new_jvm(node)
+        cmd += self.path.script("kafka-run-class.sh", node)
         cmd += " "
         cmd += self.java_class_name()
         cmd += " --topic %s --broker-list %s" % (self.topic, self.kafka.bootstrap_servers(self.security_protocol))
