@@ -53,6 +53,7 @@ import java.io.IOException;
 import java.io.PrintStream;
 import java.time.Duration;
 import java.util.ArrayList;
+import java.util.Arrays;
 import java.util.Collection;
 import java.util.Collections;
 import java.util.HashMap;
@@ -60,6 +61,7 @@ import java.util.List;
 import java.util.Map;
 import java.util.Properties;
 import java.util.concurrent.CountDownLatch;
+import java.util.stream.Collectors;
 
 import static net.sourceforge.argparse4j.impl.Arguments.store;
 import static net.sourceforge.argparse4j.impl.Arguments.storeTrue;
@@ -96,10 +98,9 @@ public class VerifiableConsumer implements Closeable, OffsetCommitCallback, Cons
     private final boolean useAsyncCommit;
     private final boolean verbose;
     private final int maxMessages;
+    private final CountDownLatch shutdownLatch = new CountDownLatch(1);
     private int consumedMessages = 0;
 
-    private CountDownLatch shutdownLatch = new CountDownLatch(1);
-
     public VerifiableConsumer(KafkaConsumer<String, String> consumer,
                               PrintStream out,
                               String topic,
@@ -537,7 +538,8 @@ public class VerifiableConsumer implements Closeable, OffsetCommitCallback, Cons
                 .setDefault(ConsumerConfig.DEFAULT_GROUP_PROTOCOL)
                 .metavar("GROUP_PROTOCOL")
                 .dest("groupProtocol")
-                .help(String.format("Group protocol (must be one of %s)", Utils.join(GroupProtocol.values(), ", ")));
+                .help(String.format("Group protocol (must be one of %s)", Arrays.stream(GroupProtocol.values())
+                        .map(Object::toString).collect(Collectors.joining(", "))));
 
         parser.addArgument("--group-remote-assignor")
                 .action(store())
@@ -646,7 +648,10 @@ public class VerifiableConsumer implements Closeable, OffsetCommitCallback, Cons
         // command line arguments are passed in by the system test framework.
         if (groupProtocol.equalsIgnoreCase(GroupProtocol.CONSUMER.name())) {
             consumerProps.put(ConsumerConfig.GROUP_PROTOCOL_CONFIG, groupProtocol);
-            consumerProps.put(ConsumerConfig.GROUP_REMOTE_ASSIGNOR_CONFIG, res.getString("groupRemoteAssignor"));
+            String groupRemoteAssignor = res.getString("groupRemoteAssignor");
+
+            if (groupRemoteAssignor != null)
+                consumerProps.put(ConsumerConfig.GROUP_REMOTE_ASSIGNOR_CONFIG, groupRemoteAssignor);
         } else {
             // This means we're using the old consumer group protocol.
             consumerProps.put(ConsumerConfig.PARTITION_ASSIGNMENT_STRATEGY_CONFIG, res.getString("assignmentStrategy"));
