@@ -79,7 +79,7 @@ import static org.junit.jupiter.api.Assertions.assertTrue;
 
 public class TransactionsCommandTest {
 
-    private final MockExitProcedure exitProcedure = new MockExitProcedure();
+    private final ToolsTestUtils.MockExitProcedure exitProcedure = new ToolsTestUtils.MockExitProcedure();
     private final ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
     private final PrintStream out = new PrintStream(outputStream);
     private final MockTime time = new MockTime();
@@ -177,7 +177,7 @@ public class TransactionsCommandTest {
         List<List<String>> table = readOutputAsTable();
         assertEquals(3, table.size());
 
-        List<String> expectedHeaders = asList(TransactionsCommand.DescribeProducersCommand.HEADERS);
+        List<String> expectedHeaders = TransactionsCommand.DescribeProducersCommand.HEADERS;
         assertEquals(expectedHeaders, table.get(0));
 
         Set<List<String>> expectedRows = Utils.mkSet(
@@ -187,14 +187,25 @@ public class TransactionsCommandTest {
         assertEquals(expectedRows, new HashSet<>(table.subList(1, table.size())));
     }
 
-    @Test
-    public void testListTransactions() throws Exception {
+    @ParameterizedTest
+    @ValueSource(booleans = {true, false})
+    public void testListTransactions(boolean hasDurationFilter) throws Exception {
         String[] args = new String[] {
             "--bootstrap-server",
             "localhost:9092",
             "list"
         };
 
+        if (hasDurationFilter) {
+            args = new String[] {
+                "--bootstrap-server",
+                "localhost:9092",
+                "list",
+                "--duration-filter",
+                Long.toString(Long.MAX_VALUE)
+            };
+        }
+
         Map<Integer, Collection<TransactionListing>> transactions = new HashMap<>();
         transactions.put(0, asList(
             new TransactionListing("foo", 12345L, TransactionState.ONGOING),
@@ -204,7 +215,11 @@ public class TransactionsCommandTest {
             new TransactionListing("baz", 13579L, TransactionState.COMPLETE_COMMIT)
         ));
 
-        expectListTransactions(transactions);
+        if (hasDurationFilter) {
+            expectListTransactions(new ListTransactionsOptions().filterOnDuration(Long.MAX_VALUE), transactions);
+        } else {
+            expectListTransactions(transactions);
+        }
 
         execute(args);
         assertNormalExit();
@@ -213,7 +228,7 @@ public class TransactionsCommandTest {
         assertEquals(4, table.size());
 
         // Assert expected headers
-        List<String> expectedHeaders = asList(TransactionsCommand.ListTransactionsCommand.HEADERS);
+        List<String> expectedHeaders = TransactionsCommand.ListTransactionsCommand.HEADERS;
         assertEquals(expectedHeaders, table.get(0));
 
         Set<List<String>> expectedRows = Utils.mkSet(
@@ -272,7 +287,7 @@ public class TransactionsCommandTest {
         List<List<String>> table = readOutputAsTable();
         assertEquals(2, table.size());
 
-        List<String> expectedHeaders = asList(TransactionsCommand.DescribeTransactionsCommand.HEADERS);
+        List<String> expectedHeaders = TransactionsCommand.DescribeTransactionsCommand.HEADERS;
         assertEquals(expectedHeaders, table.get(0));
 
         List<String> expectedRow = asList(
@@ -428,13 +443,7 @@ public class TransactionsCommandTest {
         AbortTransactionResult abortTransactionResult = Mockito.mock(AbortTransactionResult.class);
         KafkaFuture<Void> abortFuture = completedFuture(null);
 
-        final int expectedCoordinatorEpoch;
-        if (coordinatorEpoch < 0) {
-            expectedCoordinatorEpoch = 0;
-        } else {
-            expectedCoordinatorEpoch = coordinatorEpoch;
-        }
-
+        int expectedCoordinatorEpoch = Math.max(coordinatorEpoch, 0);
         AbortTransactionSpec expectedAbortSpec = new AbortTransactionSpec(
             topicPartition, producerId, producerEpoch, expectedCoordinatorEpoch);
 
@@ -542,7 +551,7 @@ public class TransactionsCommandTest {
         Map<String, TopicDescription> descriptions
     ) {
         DescribeTopicsResult result = Mockito.mock(DescribeTopicsResult.class);
-        Mockito.when(result.all()).thenReturn(completedFuture(descriptions));
+        Mockito.when(result.allTopicNames()).thenReturn(completedFuture(descriptions));
         Mockito.when(admin.describeTopics(new ArrayList<>(descriptions.keySet()))).thenReturn(result);
     }
 
@@ -703,7 +712,7 @@ public class TransactionsCommandTest {
         List<List<String>> table = readOutputAsTable();
         assertEquals(1, table.size());
 
-        List<String> expectedHeaders = asList(TransactionsCommand.FindHangingTransactionsCommand.HEADERS);
+        List<String> expectedHeaders = TransactionsCommand.FindHangingTransactionsCommand.HEADERS;
         assertEquals(expectedHeaders, table.get(0));
     }
 
@@ -742,7 +751,7 @@ public class TransactionsCommandTest {
         List<List<String>> table = readOutputAsTable();
         assertEquals(1, table.size());
 
-        List<String> expectedHeaders = asList(TransactionsCommand.FindHangingTransactionsCommand.HEADERS);
+        List<String> expectedHeaders = TransactionsCommand.FindHangingTransactionsCommand.HEADERS;
         assertEquals(expectedHeaders, table.get(0));
     }
 
@@ -940,7 +949,7 @@ public class TransactionsCommandTest {
         List<List<String>> table = readOutputAsTable();
         assertEquals(2, table.size());
 
-        List<String> expectedHeaders = asList(TransactionsCommand.FindHangingTransactionsCommand.HEADERS);
+        List<String> expectedHeaders = TransactionsCommand.FindHangingTransactionsCommand.HEADERS;
         assertEquals(expectedHeaders, table.get(0));
 
         long durationMinutes = TimeUnit.MILLISECONDS.toMinutes(time.milliseconds() - lastTimestamp);
@@ -1048,27 +1057,14 @@ public class TransactionsCommandTest {
     }
 
     private void assertNormalExit() {
-        assertTrue(exitProcedure.hasExited);
-        assertEquals(0, exitProcedure.statusCode);
+        assertTrue(exitProcedure.hasExited());
+        assertEquals(0, exitProcedure.statusCode());
     }
 
     private void assertCommandFailure(String[] args) throws Exception {
         execute(args);
-        assertTrue(exitProcedure.hasExited);
-        assertEquals(1, exitProcedure.statusCode);
-    }
-
-    private static class MockExitProcedure implements Exit.Procedure {
-        private boolean hasExited = false;
-        private int statusCode;
-
-        @Override
-        public void execute(int statusCode, String message) {
-            if (!this.hasExited) {
-                this.hasExited = true;
-                this.statusCode = statusCode;
-            }
-        }
+        assertTrue(exitProcedure.hasExited());
+        assertEquals(1, exitProcedure.statusCode());
     }
 
 }
