@@ -18,7 +18,6 @@
 package org.apache.kafka.trogdor.agent;
 
 import static java.util.Arrays.asList;
-import static java.util.concurrent.TimeUnit.MILLISECONDS;
 import static org.junit.jupiter.api.Assertions.assertEquals;
 import static org.junit.jupiter.api.Assertions.assertNotEquals;
 import static org.junit.jupiter.api.Assertions.assertThrows;
@@ -54,6 +53,7 @@ import org.apache.kafka.trogdor.rest.WorkerRunning;
 import org.apache.kafka.trogdor.task.NoOpTaskSpec;
 import org.apache.kafka.trogdor.task.SampleTaskSpec;
 import org.apache.kafka.trogdor.task.TaskSpec;
+import org.junit.jupiter.api.AfterEach;
 import org.junit.jupiter.api.Test;
 
 import java.io.ByteArrayOutputStream;
@@ -68,9 +68,11 @@ import java.util.HashMap;
 import java.util.TreeMap;
 import org.junit.jupiter.api.Timeout;
 
-@Timeout(value = 120000, unit = MILLISECONDS)
+@Timeout(value = 120)
 public class AgentTest {
 
+    private Agent agent;
+
     private static BasicPlatform createBasicPlatform(Scheduler scheduler) {
         TreeMap<String, Node> nodes = new TreeMap<>();
         HashMap<String, String> config = new HashMap<>();
@@ -86,8 +88,17 @@ public class AgentTest {
         JsonRestServer restServer = new JsonRestServer(0);
         AgentRestResource resource = new AgentRestResource();
         restServer.start(resource);
-        return new Agent(createBasicPlatform(scheduler), scheduler,
+        agent = new Agent(createBasicPlatform(scheduler), scheduler,
                 restServer, resource);
+        return agent;
+    }
+
+    @AfterEach
+    public void cleanUp() throws Exception {
+        if (agent != null) {
+            agent.beginShutdown();
+            agent.waitForShutdown();
+        }
     }
 
     @Test
@@ -95,6 +106,7 @@ public class AgentTest {
         Agent agent = createAgent(Scheduler.SYSTEM);
         agent.beginShutdown();
         agent.waitForShutdown();
+        this.agent = null;
     }
 
     @Test
@@ -104,6 +116,7 @@ public class AgentTest {
             maxTries(10).target("localhost", agent.port()).build();
         client.invokeShutdown();
         agent.waitForShutdown();
+        this.agent = null;
     }
 
     @Test
@@ -113,8 +126,6 @@ public class AgentTest {
             maxTries(10).target("localhost", agent.port()).build();
         AgentStatusResponse status = client.status();
         assertEquals(agent.status(), status);
-        agent.beginShutdown();
-        agent.waitForShutdown();
     }
 
     @Test
@@ -168,8 +179,6 @@ public class AgentTest {
 
         time.setCurrentTimeMs(150);
         assertNotEquals(agent.uptime(), uptime);
-        agent.beginShutdown();
-        agent.waitForShutdown();
     }
 
     @Test
@@ -223,9 +232,6 @@ public class AgentTest {
                 workerState(new WorkerRunning("baz", bazSpec, 0, new TextNode("active"))).
                 build()).
             waitFor(client);
-
-        agent.beginShutdown();
-        agent.waitForShutdown();
     }
 
     @Test
@@ -284,9 +290,6 @@ public class AgentTest {
                 workerState(new WorkerDone("bar", barSpec, barSpecStartTimeMs, startTimeMs + 7, new TextNode("done"), "")).
                 build()).
             waitFor(client);
-
-        agent.beginShutdown();
-        agent.waitForShutdown();
     }
 
     @Test
@@ -445,9 +448,6 @@ public class AgentTest {
         time.sleep(1);
         client.destroyWorker(new DestroyWorkerRequest(1));
         new ExpectedTasks().waitFor(client);
-
-        agent.beginShutdown();
-        agent.waitForShutdown();
     }
 
     static void testExec(Agent agent, String expected, boolean expectedReturn, TaskSpec spec) throws Exception {
@@ -468,8 +468,6 @@ public class AgentTest {
                 JsonUtil.toPrettyJsonString(rebasedSpec)) +
             String.format("Task failed with status null and error worker expired%n"),
             false, rebasedSpec);
-        agent.beginShutdown();
-        agent.waitForShutdown();
     }
 
     @Test
@@ -483,8 +481,6 @@ public class AgentTest {
                 JsonUtil.toPrettyJsonString(rebasedSpec)) +
                 String.format("Task succeeded with status \"halted\"%n"),
             true, rebasedSpec);
-        agent.beginShutdown();
-        agent.waitForShutdown();
     }
 
 }
